import sift
import struct
import subprocess
import os
import pickle
import ConfigParser, xml.dom.minidom
from xml.dom.minidom import Node
import time
import shutil,wx



def remap(opath,newbasepath,oldbasepath):
	"takes opath, removes oldbasepath, and replaces it with newbasepath"
	
	start = opath.find(oldbasepath)
	if start>-1:
		newpath = opath[0:start]+newbasepath+opath[start+len(oldbasepath):]
		return newpath
	else:
		return ""
	
def readconfig(configFilename = 'config.ini'):
	config = ConfigParser.ConfigParser()
	config.readfp(open(configFilename))
	directories = config.items('Directories')
	files = config.items('Files')
	modes = config.items('Modes')
	data = dict()
	for pair in directories:
		data[pair[0]] = os.path.normpath(pair[1])
	for pair in files:
		data[pair[0]] = os.path.normpath(pair[1])
	for pair in modes:
		data[pair[0].swapcase()] = (pair[1]=='True') #config makes everything lowercase
	return data
	
def getAnImageSource(xmlFilename):
	doc = xml.dom.minidom.parse(xmlFilename)
	for index,node in enumerate(doc.getElementsByTagName("class")):
		for imageNode in node.childNodes: 	
			if imageNode.nodeType != Node.TEXT_NODE:
				impath = imageNode.getAttribute("path")
				return impath
		

def loadImagesFromXML(xmlFilename,oldpath=None,newpath=None):
	
	doc = xml.dom.minidom.parse(xmlFilename)

	clsList=[]
	for index,node in enumerate(doc.getElementsByTagName("class")):
		imgList = []
		cls = node.getAttribute("name")
		
		clspath = ""

		for imageNode in node.childNodes: 	
			if imageNode.nodeType != Node.TEXT_NODE: #I have no idea what this is for
				impath = imageNode.getAttribute("path")
				quality = imageNode.getAttribute("quality")
				if len(quality) == 0:
					quality = -1
				if oldpath!=None:
					impath = remap(impath,newpath,oldpath)
				image = sift.Datum(impath, imageNode.getAttribute("source"), float(quality))
				imgList.append(image)
		clsList.append([cls,clspath,imgList,index])
	return clsList

def formatImgList2ClassList(imgList,clsName = 'Default'):
	return [[clsName,None,imgList,0]]
	

def loadSingleImageAsClsList(imgName,clsName = 'Default'):
	img = sift.Datum(imgName)
	return [[clsName,None,[img],0]]

def loadImagesFromDir(dirName,filetypes = ['.jpg']):
	'Returns sift.Datums for all files in dirName that are jpgs'
	tmpfiles = os.listdir(dirName)
	tmpfiles.sort()
	imgList = []
	for f in tmpfiles:
		junk,ext = os.path.splitext(f)
		if filetypes==None:
			imgList.append(os.path.join(dirName,f))
		elif any([ext.lower() == filetype for filetype in filetypes]):
			imgList.append(sift.Datum(os.path.join(dirName,f)))

	return imgList
		
		
def loadImagesFromFileWatcher(fileWatcher):
	testList = []
	filenames = fileWatcher.getNewContent(readAllOnFirst = True)
	for filename in filenames:
		im = wx.Image(filename)
		if im.Ok():
			maxSize = max(im.GetSize().Get())
			imfile = filename
			testList.append(sift.Datum(imfile,workpath = None))
	return testList



class FileWatcher:
	'Watches a directory, and provides the ability to return new files of certain filetypes'
	def __init__(self,directory,filetypes = None,quitFile = None):
		self.directory = directory
		if filetypes!=None:
			self.filetypes = [file.lower() for file in filetypes]
		else:
			self.filetypes = None
		self.lasttime = os.path.getmtime(directory)
		self.lastlength = None
		self.isFirst = True
		self.quitFile = quitFile
	def getContents(self):
		'Returns all the contents of self.directory'
		tmpfiles = os.listdir(self.directory)
		self.lasttime = time.time() #os.path.getmtime(self.directory)
		files = []
		for f in tmpfiles:
			junk,ext = os.path.splitext(f)	
			
			if self.filetypes==None:
				files.append(os.path.join(self.directory,f))
			elif any([ext.lower() == filetype for filetype in self.filetypes]):
				
				files.append(os.path.join(self.directory,f))
		return files
	def isDone(self):
		
		return os.path.exists(self.quitFile)
			
	def getNewContent(self,readAllOnFirst = False):
		'Returns files of a certain file type since last call'
		
		if readAllOnFirst & self.isFirst:
			self.isFirst = False
			return self.getContents()
		files=[]

		listdir = os.listdir(self.directory)
		thistime = time.time()
		#print listdir,len(listdir), self.lastlength,self.lasttime,thistime
		
		if self.lastlength<len(listdir):
			cwd = os.getcwd()
			os.chdir(self.directory)
			for f in listdir:
				#print f,os.path.getctime(f),
				if (os.path.getctime(f)>self.lasttime) & (os.path.getctime(f)<=thistime):
					
					junk,ext = os.path.splitext(f)
					if self.filetypes==None:
						files.append(os.path.join(self.directory,f))
					elif any([ext.lower() == filetype for filetype in self.filetypes]):
						files.append(os.path.join(self.directory,f))
		
			self.lastlength = len(listdir)
			self.lasttime = thistime
			os.chdir(cwd)
		return files

if __name__=="__main__":
	result = newread()
	print result