from numpy import *
from scipy.weave import converters
from scipy import weave


class Transform:
	def __init__(self,R=None,t=None,scale = None,ori = None):
		if scale==None:
			self.R = R
			self.t = t
			if self.isSimilarity():
				self.scale,self.ori = self.getSimilarity()
			else:
				self.scale = sqrt(0.5*((R[0,0]+R[0,1])**2+(R[1,0]+R[1,1])**2))
			
		else:
			self.R = squeeze(array(((cos(ori),-sin(ori)),(sin(ori),cos(ori)))))
			self.t = t
			self.scale = scale
			self.R = self.R*self.scale
			self.ori = ori
		self.A = vstack((hstack((self.R,self.t.reshape(-1,1))),array((0,0,1))))

	def transform(self,x,o=None,s=None):
	
		outx = dot(x,self.R) + self.t
		if o==None:
			return outx
		else:
			if self.isSimilarity():
			#	print "Used sim"
				outo = mod(o+self.ori+pi,2*pi)-pi
			else:
			#	print "Used aff"
				tmpo = hstack((sin(o).reshape(-1,1),cos(o).reshape(-1,1)))
		
				tmpo = dot(tmpo,self.R)
			
			
				outo = arctan2(tmpo[:,0],tmpo[:,1]).reshape(o.shape)
		
			outs = s*self.scale
			return (outx,outo,outs)
	
	def getSkew(self):
		skew  = dot(self.R,self.R.T)
		return skew[1,0]/sqrt(skew[1,1]*skew[0,0]) #array((skew[1,0]/sqrt(skew[1,1]*skew[0,0]),skew[1,1]/skew[0,0]))
	
	def invtransform(self,x):
		#outx = dot((x-self.t),self.R.T)
		if not hasattr(self,'inv'):
			self.inv = linalg.inv(self.A)
		return dot(x,self.inv[:2,:2]) + self.inv[:2,2]
	
	def getDiff(self,x1,x2,o1,o2,s1,s2):
		
		(tx,to,ts) = self.transform(x1,o1,s1)
		return (x2-tx,mod(o2-to+pi,2*pi)-pi,log(ts/s2))
		
	def getError(self,x1,x2,o1,o2,s1,s2,h,same =None):
		"""Returns the error = ((ex,eo,es),err,sum(err)) after the transform has been applied.
		e? is per match contributions to error for diff variables
		err is array of per match contributions"""
		dx,do,ds = self.getDiff(x1,x2,o1,o2,s1,s2)
		ex = exp(-sum((dx/self.scale)**2,axis=1)/h[0])
		eo = exp(-do**2/h[1])
		es = exp(-ds**2/h[2])
		err =[ex[i]*eo[i]*ex[i] for i in range(len(eo))]
		if same != None:
			maxsame = zeros(max(same)+1)
			for i in range(len(o1)):
				if same[i]>-1:
					if maxsame[same[i]]<err[i]:
						maxsame[same[i]] = err[i]
			for i in range(len(o1)):
				if (same[i]>-1):
					if (err[i]!= maxsame[same[i]]) :
						err[i] = 0
			
			tmperr = array(err).reshape(-1,1)
			tmperr[tmperr<0.02] = 0
			#print hstack((same.reshape(-1,1),tmperr))
		return ((ex,eo,es),array(err),sum(err))
	#	return (sum(err),array(err))
	
	def getErrorFast(self,x1,x2,o1,o2,s1,s2,h):
		dx,do,ds = self.getDiff(x1,x2,o1,o2,s1,s2)
		ex = exp(-sum((dx/self.scale)**2,axis=1)/h[0])
		eo = exp(-do**2/h[1])
		es = exp(-ds**2/h[2])
		err =[ex[i]*eo[i]*ex[i] for i in range(len(eo))]
		return sum(err)
	#	
	def isSimilarity(self):
		return self.R[0,0] == self.R[1,1]		
	def getSimilarity(self):
		scale = sqrt(sum(self.R[0]**2))
		ori = arctan2(self.R[1,0],self.R[0,0])
		return (scale,ori)
	def tostr(self):
		return str(self.t)+ " + %.3f + %.3f" % (self.scale,self.ori) 	


def getTransform(x1,x2,o1=None,o2=None,s1=None,s2=None,rescale = None,reori=None):
	if rescale == None:
		rescale = s2/s1
		reori = mod(o2-o1 + pi,2*pi)-pi
	R = squeeze(array([[cos(reori),sin(reori)],[-sin(reori),cos(reori)]])).T
	t = x2 - rescale*dot(x1,R)

	return Transform(rescale*R,t)

	
#def getUnderTransform(x1,x2,o1,o2,s1,s2,rescale,reori,t,h):
	#reoriAll = mod(o2-o1 + pi,2*pi)-pi
	#rescaleAll = s2/s1
	#R = squeeze(array([[cos(reori),sin(reori)],[-sin(reori),cos(reori)]]))
	#transx = rescale*dot(R,x.T).T + t
	#transDiff = (x2 - transx).reshape(-1,2)


	#oriDiff = mod(reori-reoriAll+pi,2*pi)-pi
	#scaleDiff = log(rescale/rescaleAll)
	#transError = exp(-sum((transDiff/rescale)**2,axis=1)/h[0])
	#oriError = exp(-(oriDiff**2)/h[1])
	#scaleError = exp(-(scaleDiff**2)/h[2]) 
	#err = array([transError[i]*oriError[i]*scaleError[i] for i in range(x1.shape[0])])
	#return ((transDiff,oriDiff,scaleDiff),(transError,oriError,scaleError,err),transx)


def simMatch(x1,x2,o1,o2,s1,s2,h,same=None):
	rescale = s2/s1
	reori = mod(o2-o1 + pi,2*pi)-pi

	totalError,transError,oriError,scaleError=[],[],[],[]
	localError=[]
	
	if same!=None:
		if len(same)>0:
			maxsame = zeros(max(same)+1)
		else:
			maxsame=[]
		
	for i in range(0,x1.shape[0]):
		
		
		R = array([[cos(reori[i]),sin(reori[i])],[-sin(reori[i]),cos(reori[i])]]).T
		t = x2[i] - rescale[i]*dot(x1[i,:],R) 
		transError.append(sum(((x2 - (rescale[i]*dot(x1,R) + t))/rescale[i])**2,axis=1))
		oriError.append( (mod(reori[i]-reori+pi,2*pi)-pi)**2)
		scaleError.append(log(rescale[i]/rescale)**2)
		lE = exp(-(transError[-1]/h[0]+oriError[-1]/h[1] + scaleError[-1]/h[2]))
		#localError.append(exp(-(transError[-1]/h[0]+oriError[-1]/h[1] + scaleError[-1]/h[2])))
		
		
		#Note this is slow
		code = """
				int i,j;
				for (i=0;i<nData;i++) 
					if ((same(i)>-1) & (maxsame(same(i))<lE(i))) 
						maxsame(same(i)) = lE(i);
				for (i=0;i<nData;i++)
					if (same(i)>-1 & lE(i)!= maxsame(same(i)))
						lE(i) = 0;
				"""
		nData = len(same)
		weave.inline(code,['nData','same','lE','maxsame'],type_converters=converters.blitz, compiler = 'gcc')
		#samerange = range(len(same)) 
		#if same!=None:
			#maxsame[:] = 0
			#for i in samerange:
				#if same[i]>-1:
					#if maxsame[same[i]]<lE[i]:
						#maxsame[same[i]] = lE[i]
			#for i in samerange:
				#if (same[i]>-1):
					#if (lE[i]!= maxsame[same[i]]) :
						#lE[i] = 0
					
						
			
		localError.append(lE)
		
		totalError.append(sum(lE))	


	return (fromiter(totalError,'float32'),localError,[transError,oriError,scaleError])


def ransac(x1,x2,o1,o2,s1,s2):
	rescale = (s2/s1).reshape(-1,1)
	reori = (o2/o1).reshape(-1,1)
	scaleDiff = abs(log(rescale / rescale.T))
	oridiff = abs(mod(reori - reori.T + pi,2*pi)-pi)

def leastSquare(x1,x2):
	b = x1.copy()
	b[:,1] *=-1 

	A = concatenate([x1.reshape(-1,1),b[:,[1,0]].reshape(-1,1),tile(array((1,0)).reshape(-1,1), (x1.shape[0],1)), tile(array((0,1)).reshape(-1,1), (x1.shape[0],1))],1)
	
	tr = squeeze(dot(linalg.inv(dot(A.T,A)),dot(A.T,x2.reshape(-1,1))))

	scale = sqrt(tr[0]**2+tr[1]**2)
	theta = arctan2(tr[1]/scale,tr[0]/scale)

	return Transform(t = tr[2:], scale = scale,ori = -theta)


def leastSquareAffine(x1,x2):
	#y = x2.flatten(:)
	n = x1.shape[0]
	A = zeros((n*2,6))
	A[0::2,:] = hstack((x1,zeros((n,2)),ones((n,1)),zeros((n,1))))
	A[1::2,:] = hstack((zeros((n,2)),x1,zeros((n,1)),ones((n,1))))
	tr = squeeze(dot(linalg.inv(dot(A.T,A)),dot(A.T,x2.reshape(-1,1))))
	#print "tr",tr
	return Transform(tr[0:4].reshape(2,2).T,tr[4:].reshape(1,-1))
	#return (tr[0:4].reshape(2,2).T,tr[4:].reshape(1,-1))


if __name__ == "__main__":
	pass
	
	
	