import wx
import wx.lib.scrolledpanel
import os,shutil,sys,time
from numpy import *

from visual import *
from constants import * 
import sift,newdataread,geoMatch,util,training
import convexhull

dataread = newdataread

def flattenImageList(clsList):
	imageList = []
	iterList = []
	for i in range(len(clsList)):
		iterList.append(iter(clsList[i][2]))
	atLeastOne = True
	while atLeastOne:
		
		atLeastOne = False
		for i in range(len(clsList)):
			try:
				imageList.append((i,clsList[i][0],iterList[i].next(),clsList[i][3]))
				atLeastOne = True
			except StopIteration:
				pass
	print "Number of Training Images ",len(imageList)
	return imageList


class listIterator:
	'A simple iterator that also contains a counter'
	def __init__(self,iList):
		self.myiter = iter(iList)
		self.n=0
	def __iter__(self):
		return self
	def next(self):
		self.n+=1
		return self.myiter.next()


class MatchingPairIterator:
	'This class contains the policy for the order in which I match pairs of images. \012 In this case, each new test image is matched to minNImageComps images from the trainingList, after which each test image is run on batches of checkHowOften training images so that each test image has the same number of matches'
	def __init__(self,testList,trainingList,nClasses=1,fileWatcher=None):
		self.fileWatcher = fileWatcher
		self.minNImageComps = nClasses*MIN_PER_CLASS #2 * number of classes
		self.checkHowOften = nClasses
		self.nComps = 0
		self.testList = testList
		self.currentIndex = -1
		self.trainingList = trainingList
		self.queue = []  #Note this queue is reversed for efficiency reasons
		self.__appendToQueue(self.testList) 
	def __appendToQueue(self,testList = None):
		if testList == None:
			testList = self.testList
		self.queue.extend([(testimage,listIterator(self.trainingList)) for testimage in testList])

	def __iter__(self):
		return self
	def getStatus(self):
		return [trn.n for tst,trn in self.queue]
	
	def next(self):
		self.nComps+=1
		while True:
			if len(self.queue)==0: #no test images to compute on
				self.__checkForNewImages()
				return (None,None)
			if (self.queue[-1][1].n<self.minNImageComps) | (self.nComps<self.checkHowOften): #current item might need compute
				try:
					trainingData = self.queue[-1][1].next()
					testData = self.queue[-1][0]
					return (self.queue[-1][0],trainingData)
				except StopIteration:
					#This should be the case where we can remove the item from the queue because it is done processing
					self.queue.pop()[0].freeStuff() 
					self.nComps = 0
					if len(self.queue)==0: 
						return (None,None) 
			else: #current item has had enough for now, move it to last in queue
				self.nComps=0
				if len(self.queue)>1:
					if self.queue[-1][1].n > self.queue[-2][1].n:
						itemToMove = self.queue.pop()
						self.queue.insert(0,itemToMove)
						if self.queue[-1][1].n > self.minNImageComps:
							self.__checkForNewImages()

	def __checkForNewImages(self):
		'Check if the fileWatcher returns a new image, if so put it in the queue'
		if self.fileWatcher!= None:
			newImages = self.fileWatcher()
		else:
			return False
		if len(newImages)>0:
			self.testList.extend(newImages)
			self.__appendToQueue(newImages)
			return True
		else:
			return False
	

class ImgFrame(wx.Frame):
	def __init__(self,parent,imgList,imageSelectHandlers =  None, sizer_width = 4, size = (415,400)):
		wx.Frame.__init__(self,parent,wx.ID_ANY)
		self.imgList = imgList
		
		self.imageSelectHandlers = imageSelectHandlers
		self.sizer_width = sizer_width
		self.SetSize(size)
		#self.Fit()
		self.size = size
		
		self.__id2img = self.populate()
		self.Show(True)
	def populate(self):
		panel = wx.ScrolledWindow(self,wx.ID_ANY,size= self.size)
		panel.EnableScrolling(False,True)
		panel.SetScrollbars(5,5,50,50)
		sizer = wx.GridSizer()
		id2img = dict()
		for img in self.imgList:
			tmpim = wx.StaticBitmap(panel,wx.ID_ANY,img)
			id2img[tmpim.GetId()] = img
			sizer.Add(tmpim)
			if hasattr(self.imageSelectHandlers,'len'):
				tmpim.Bind(wx.EVT_LEFT_DOWN,self.imageSelectHandlers[0])
				tmpim.Bind(wx.EVT_RIGHT_DOWN,self.imageSelectHandlers[1])
			else:
				tmpim.Bind(wx.EVT_LEFT_DOWN,self.imageSelectHandlers)
			
			
		sizer.SetCols(self.sizer_width)
		sizer.SetRows(1+len(self.imgList)/self.sizer_width)
		panel.SetSizer(sizer)
		panel.SetSize(self.size)
		return id2img
	def getImgFromId(self,Id):
		return self.__id2img[Id]
	
	
class GraphicsPanel(wx.Panel):
	def __init__(self,parent,mysize = GRAPHIC_DIM):
		
		newsize = (mysize[0],mysize[1]+20)
		wx.Panel.__init__(self,parent,size = newsize)
		
		self.Bind(wx.EVT_PAINT,self.onPaint)
	def onPaint(self,event):
		dc = wx.PaintDC(self)
		dc.Clear()
		if hasattr(self,'masterImage'):
			dc.DrawBitmap(self.masterImage,0,0)
			
	def visualizeState(self,state,textinfo=None):
		"Resets the state, which contains a pair of images"
		self.state = state
		self.model = VisualDatum(state.model)
		self.test = VisualDatum(state.test)
		#self.ClearBackground()
		self.__resetImageSizes()
		self.__setMasterImage(textinfo)
		self.staticImages(textinfo)	
		self.Fit()
		self.Show(True)
		self.Refresh()
		return self.masterImage
	
	def staticImages(self,textinfo=None):
		
		memory = wx.MemoryDC()
		memory.SelectObject(self.masterImage)
		memory.SetPen(wx.BLACK_PEN)
		self.allmatches = self.state.getMatches()
		self.matches = VisualMatches(self.allmatches,self,*self.__getImagesDisplayFuncs())
		self.matches.draw(memory)
		memory.SetBackgroundMode(wx.SOLID)
		
		VisualRect(displayfunc = self.model.getDisplayLoc,castrect = self.state.modelRect).draw(memory)
		VisualRect(displayfunc = self.test.getDisplayLoc,castrect = self.state.testRect).draw(memory)
		
		if textinfo!=None:
			memory.DrawText(str(textinfo),0,self.masterImage.GetHeight()-21)
		#if hasattr(imgname,'__len__'):
		#	for imname in imgname: 
		#		self.masterImage.SaveFile(imname, wx.BITMAP_TYPE_JPEG )
		#else:
		#	print self.masterImage.SaveFile(imgname, wx.BITMAP_TYPE_JPEG )
		
	
	def __getImagesDisplayFuncs(self):
		return (self.model.getDisplayLoc,self.test.getDisplayLoc)
	
	def __resetImageSizes(self):
		"Resets the images so that each image is visible in the panel"
		modelMaxSize,testMaxSize = self.__optimalRescale(self.model,self.test)
		self.model.setDisplayBounds(*modelMaxSize)
		self.test.setDisplayBounds(*testMaxSize)
		self.test.setOffset( modelMaxSize[0])
		self.test.setViewable()
		self.model.setViewable()
		self.offset = modelMaxSize[0]
		
	def __setMasterImage(self,textinfo = None):
		tmp1 = self.model.getViewableImage()
		tmp2 = self.test.getViewableImage()
		if textinfo==None:
			masterImage = wx.EmptyImage(tmp1.GetWidth()+tmp2.GetWidth(),max(tmp1.GetHeight(),tmp2.GetHeight()))
		else:
			masterImage = wx.EmptyImage(tmp1.GetWidth()+tmp2.GetWidth(),20+max(tmp1.GetHeight(),tmp2.GetHeight()))
		masterImage.Paste(tmp1,0,0)
		masterImage.Paste(tmp2,tmp1.GetWidth(),0)
		self.masterImage = masterImage.ConvertToBitmap()
		self.SetSize(self.masterImage.GetSize())
	
	def __optimalRescale(self,im1,im2,maxX = GRAPHIC_DIM[0],maxY = GRAPHIC_DIM[1]):
		'Takes the two images (wxImage), and returns a (size1,size2) where the sizes are such that concat horizontally the sum is less than maxX and neither are greater maxY'
		x = array((im1.GetWidth(),im2.GetWidth()),dtype = float32)
		y = array((im1.GetHeight(),im2.GetHeight()),dtype = float32)
		#print x,y
		rescale = amin(vstack((0.5*maxX/x,maxY/y)),0)
		#print rescale
		rescale[rescale>1] = 1
		if prod(rescale)<1 & (rescale==1).any() :
			bigi = where(rescale<1)[0][0]
			smalli = mod(bigi+1,2)
			#print bigi,smalli
			maxX = maxX-x[smalli]
			rescale[bigi] = min(maxX/x[bigi],maxY/y[bigi])
			
		#print rescale
			
		return ((rescale[0]*im1.GetWidth(),rescale[0]*im1.GetHeight()),(rescale[1]*im2.GetWidth(),rescale[1]*im2.GetHeight()))

class BasicMenu(wx.Menu):
	def __init__(self,parent):
		wx.Menu.__init__(self)
		self.parent = parent
	def addMenuItem(self,label,handler,type = None):
		ID = wx.NewId()
		if type == None:
			menuItem = self.Append(ID,label)
		elif type == 'RADIO':
			menuItem = self.AppendRadioItem(ID,label)	
			menuItem.Check(False)
		wx.EVT_MENU(self.parent,ID,handler)
		return menuItem
	
class MatcherFrame(wx.Frame):
	def __init__(self,interactive=True,configdata=None):
		wx.Frame.__init__(self,None,wx.ID_ANY,'Nothing')
		self.configdata = configdata
		self.resultDirectory = None
		if interactive:
			self.__setUpMenus()
			self.graphicsPanel = GraphicsPanel(self)
			self.haveTrained = False
			self.Fit()
			self.Show(True)
			
	def __setUpMenus(self):
		self.menuids = dict()
		self.menuitems = dict()
		menuBar = wx.MenuBar()
		self.__fileTypes=dict()
		#Individual Menu############
		menu = BasicMenu(self)
		self.menuids[menu.addMenuItem('Load Model image from thumbnails',self.displayImages).GetId()] = 'MODEL_THUMB'
		self.menuids[menu.addMenuItem('Load Test image from thumbnails',self.displayImages).GetId()] = 'TEST_THUMB'
		if self.configdata != None:
			menu.addMenuItem('Use Training Images from config File',None)
		menuBar.Append(menu,"Ind Matching")
		
		#Set Menu #########################
		menu = BasicMenu(self)
					
		#Test Source  -------
		submenu = BasicMenu(self)
		item = submenu.addMenuItem('Testing from dir',self.selectDirectory,'RADIO')
		self.__itemtrack(item,'TEST_DIR')
		item = submenu.addMenuItem('Testing on single img',self.selectFile,'RADIO')
		self.__itemtrack(item, 'TEST_IM')
		self.__fileTypes[item.GetId()] = 'JPG file (*.jpg)|*.jpg|All (*)|*'
		item = submenu.addMenuItem('Testing from config',self.useConfigTestSource,'RADIO')
		self.__itemtrack(item, 'TEST_CONFIG')
		if self.configdata == None:
			item.Enable(False)
		menu.AppendMenu(wx.ID_ANY,"Testing Source",submenu)
		
		#Result Data ---------
		submenu = BasicMenu(self)
		item = submenu.addMenuItem('Select Result Directory',self.selectDirectory,'RADIO')
		self.__itemtrack(item,'RESULT_DIR')
		item = submenu.addMenuItem('Result Directory from config',self.useConfigResultDirectory,'RADIO')
		menu.AppendMenu(wx.ID_ANY,"Result Storage",submenu)
		
		menu.addMenuItem('Use All config',self.useConfigdata)
		menu.addMenuItem('Run',self.runSetMatch)
		menuBar.Append(menu,"Set Matching")
		
		#Training Options #######################
		menu = BasicMenu(self)
		#Training Source  ---------
		submenu = BasicMenu(self)
		item = submenu.addMenuItem('Training from dir',self.selectDirectory,'RADIO')
		self.__itemtrack(item,'TRAIN_DIR')
		item = submenu.addMenuItem('Training using xml file',self.selectFile,'RADIO')
		self.__itemtrack(item, 'TRAIN_XML')
		self.__fileTypes[item.GetId()] = 'xml (*.xml)|*.xml'
		item = submenu.addMenuItem('Training from config',self.useConfigTrainSource,'RADIO')
		self.__itemtrack(item,'TRAIN_CONFIG')
		if self.configdata== None:
			item.Enable(False)
		menu.AppendMenu(wx.ID_ANY,"Training Source",submenu)	
		
		#Training Type  ---------
		submenu = BasicMenu(self)
		self.trainTypes = []
		item = submenu.addMenuItem('No training - Google ordering',None,'RADIO')
		item.Check(True)
		self.trainTypes.append( (item,'NO_TRAIN'))
		item = submenu.addMenuItem('Other',None,'RADIO')
		self.trainTypes.append( (item,'OTHER'))
		menu.AppendMenu(wx.ID_ANY,"Training Type",submenu)	
		
		item = menu.addMenuItem('Train',self.doTraining)
		menuBar.Append(menu,'Training')	
		
		self.SetMenuBar(menuBar)
		
	def doTraining(self,event):
		itemSelected = filter(lambda x:x[0].IsChecked(), self.trainTypes)[0][1] 
		clsList = self.getTrainingData()
		if itemSelected == 'NO_TRAIN':
			pass
		elif itemSelected == 'OTHER':
			training.train(clsList)
			
		
	def __itemtrack(self,item,type):
		self.menuids[item.GetId()] = type
		self.menuitems[item.GetId()] = item
	
	def useConfigdata(self,event):
		self.useConfigTrainSource()
		self.useConfigTestSource()
		self.useConfigResultDirectory()
		
	def displayImages(self,event):
		'Displays a directory dialog, which once selected pops up a thumbnail view of all images which are then used for selection of either MODEL or TEST'
		imageTypes = ['.jpg']
		dirDialog = wx.DirDialog(self)
		dirDialog.Show(True)
		if dirDialog.ShowModal() == wx.ID_OK:
			directory = dirDialog.GetPath()
			fileNames = os.listdir(directory)
			imgFileNames = filter(lambda fname: os.path.splitext(fname)[1] in imageTypes,fileNames)
			os.chdir(directory)
			images = [sift.Datum(img).loadThumbnail() for img in imgFileNames]
			if self.menuids[event.GetId()] == "MODEL_THUMB":
				myhandler = self.setModelImage
			else:
				myhandler = self.setTestImage
			self.__loadimageframe = ImgFrame(self,images,myhandler)
			
	def setModelImage(self,event):
		self.testImage = self.__loadimageframe.getImgFromId(event.GetId())
		self.__loadimageframe.Destroy()
	def setTestImage(self,event):
		self.trainImage = self.__loadimageframe.getImgFromId(event.GetId())
		self.__loadimageframe.Destroy()
	
	def selectDirectory(self,event=None):
		dirDialog = wx.DirDialog(self)
		dirDialog.Fit()
		dirDialog.Show(True)
		if dirDialog.ShowModal() == wx.ID_OK:
			
			dirname = dirDialog.GetPath()
			if event == None: #from remap
				self.trainREMAP['newpath'] = dirname
			else:
				eventType = self.menuids[event.GetId()]
				if eventType == 'TRAIN_DIR':
					self.trainSource =  dirname
				elif eventType == 'TEST_DIR':
					self.testSource =  dirname
				elif eventType == 'RESULT_DIR':
					self.resultDirectory = dirname
		else: #In this case didn't specify anything
			if event != None: self.menuitems[event.GetId()].Check(False)	
	
	def selectFile(self,event):
		fileDialog = wx.FileDialog(self,wildcard = self.__fileTypes[event.GetId()])
		fileDialog.Fit()
		fileDialog.Show(True)
		if fileDialog.ShowModal() == wx.ID_OK:
			filename = fileDialog.GetFilename()
			directory = fileDialog.GetDirectory()
			pathname = os.path.join(directory,filename)
			if self.menuids[event.GetId()] == 'TEST_IM':
				self.testSource = pathname
			elif self.menuids[event.GetId()] == 'TRAIN_XML':
				self.trainSource = pathname
				self.__shouldREMAP(pathname)
		else: #In this case didn't specify anything
			self.menuitems[event.GetId()].Check(False)
	
	def __shouldREMAP(self,xmlFilename):
		tmpimgsource = newdataread.getAnImageSource(xmlFilename)
		dialog = wx.SingleChoiceDialog(self,"Should Remap image Source \012 Ex: " + tmpimgsource,"Remappinos.path.join(dirName,f)g",['Yes','No','Automap'])
		if dialog.ShowModal() == wx.ID_OK:
			if dialog.GetStringSelection() == 'No':
				self.trainREMAP = {'oldpath':None,'newpath':None}
			elif dialog.GetStringSelection() == 'Yes':
				tmpdialog = wx.TextEntryDialog(self,'Enter old segment',defaultValue = tmpimgsource)
				if tmpdialog.ShowModal() == wx.ID_OK:
					self.trainREMAP = {'oldpath':tmpdialog.GetValue()}
					self.selectDirectory()
			else:
				mypath = tmpimgsource
				head,tail = os.path.split(mypath)
				while (tail!="google") & (head!=""):
					head,tail = os.path.split(head)
				if head=="":
					print "INVALID autoname"
				else:
					self.trainREMAP = {'oldpath':head,'newpath':os.path.split(xmlFilename)[0]}
	
	def useConfigTrainSource(self,event=None):
		if (self.configdata.has_key('trainedfile')) and (os.path.exists(self.configdata['trainedfile'])):
			self.trainSource = self.configdata['trainedfile']
		else: #Need file checking here
			self.trainSource = self.configdata['datafile']
			if self.configdata['REMAP']:
				self.trainREMAP = {'oldpath':self.configdata['oldpath'], 'newpath':self.configdata['newpath']}
			else:
				self.trainREMAP = {'oldpath':None,'newpath':None}

	def useConfigTestSource(self,event=None):
		if os.path.isdir(self.configdata['testpath']):
			self.testSource = self.configdata['testpath']
		else:
			self.menuitems[event.GetId()].Check(False) 

	def useConfigResultDirectory(self,event = None):
		if os.path.isdir(self.configdata['resultpath']):
			self.resultDirectory = self.configdata['resultpath']

	def getTrainingData(self):
		if os.path.splitext(self.trainSource)[1] == '.xml':
			return  dataread.loadImagesFromXML(self.trainSource,self.trainREMAP['oldpath'],self.trainREMAP['newpath'])
		elif os.path.isdir(self.trainSource): # Need to put the directory case here
			return dataread.loadImagesFromDir(self.trainSource)
		else: # Need to put the case where we have already done training
			return None


	def runSetMatch(self,event):
		'Runs the matching for a set of training images and a set of test images'
		#Reading in training Data ###############
		timestart = time.time()
		if os.path.splitext(self.trainSource)[1] == '.xml':
			clsList = dataread.loadImagesFromXML(self.trainSource,self.trainREMAP['oldpath'],self.trainREMAP['newpath'])
		elif True: # Need to put the directory case here
			pass
		else: # Need to put the case where we have already done training
			pass
		#Reading in test stuff ###############
		if os.path.isdir(self.testSource):
			fileWatcher = dataread.FileWatcher(self.testSource,['.jpg'])
			testList = dataread.loadImagesFromFileWatcher(fileWatcher)
		else: # Need to put the single image case in here
			fileWatcher = None
			pass
		
		doingWebDisplay = (self.configdata != None) and self.configdata.has_key('webresultpath') and os.path.exists(self.configdata['webresultpath'])
		
		trainingList = flattenImageList(clsList)
		
		nClasses = len(clsList)

		matchingPairIterator = MatchingPairIterator(testList,trainingList,nClasses,fileWatcher)
		
		
		results = [[0,[]] for i in range(nClasses)]
		doneclasses = zeros(nClasses)
		
		detectCount = 0
		
		prev = None
		
		for testImg,trainData in matchingPairIterator:
			
			if prev!=testImg:
				print testImg.imagename
			prev = testImg
		#	print testImg.imagename,trainData[2].imagename
			if time.time()-timestart > TIME_OUT:
				self.Destroy()
				return 0
			elif testImg==None: # sleep if there is nothing to test on
				time.sleep(1)
			else:
				testImg.loadImage()
				testImg.loadSIFT(autoname=True,force = False)
				classId,className,trainImg,realClassId = trainData
				
				stillValid = (trainImg.source == "google") & testImg.isValid
				if stillValid:
					trainImg.loadImage()
					trainImg.loadSIFT(autoname=True,force = False)
					stillValid = stillValid & trainImg.isValid
				if stillValid:
					state = sift.State(trainImg,testImg)
					state.compareDatum() #feature matching on appearance
					error = state.completeMatch() #geometric constraints
					#print error
					
					#This hack is meant to make it such that model images with a huge number of features aren't favoured simply because of that. This was a last minute hack.
					normerror = min(error,5) + error/trainImg.sift.nData
					if (error>MIN_ERROR) and (state.skew<0.5) and (normerror>results[classId][0]):
				
						#Based solely on keypoint locations, how much does the convex hull of the matches cover of the entire model image
						hullarea = convexhull.areaOfConvexHull(state.match.model.x.T)
						hullratio = hullarea/(state.match.model.meanSize**2)

						
						#The hullratio  are used to weed out degerate cases.
						if hullratio>0.03:
							#print error
							tmpdict = {'error':error,'n;ormerror':normerror,'hullratio':hullratio}
							tmpimage = self.graphicsPanel.visualizeState(state,tmpdict)
							self.Refresh()
							self.Update()
							tmpimage.SaveFile(os.path.join(self.resultDirectory,str(detectCount)+'.jpg'),wx.BITMAP_TYPE_JPEG)
							detectCount +=1
							
							
							results[classId][1] = (testImg.imagename,trainImg.imagename)
							results[classId][0] = normerror
						#	self.graphicspanel.resetState(state, [os.path.join(self.resultpath,className+'.jpg')])
							
							#This score is used to determine how good the match is so I know whether to use the results from here, or from Sancho's general object classifier
							howgood= int((error>4)) + int((error>8))
						
							#saving result
							shutil.copyfile(testImg.imagename,os.path.join(self.resultDirectory,str(realClassId)+'.jpg'))
							left,top,width,height =	state.testRect.asRectangle()
							fp = open(os.path.join(self.resultDirectory,str(realClassId)+'.result'),'w')
							fp.write(str(howgood) + " " + str(error)+" " + str(left) + " " + str(top) + " " + str(width) + " " + str(height) + " " + className.replace(' ','_'))
							
							fp = open(os.path.join(self.resultDirectory,className+'.scores'),'w')
							fp.write( str(error)+ " "+ str(hullarea)+ " " + str(hullratio) + str(normerror) + " " + str(trainImg.sift.nData))
							
							#If we are doing web display and our confidence is high
							if doingWebDisplay and (error>8) and ((hullarea>2000) or (hullratio>0.2)) and (doneclasses[classId]==0):
								webpath = self.configdata['webresultpath']
								shutil.copyfile(testImg.imagename,os.path.join(webpath,'confident_' + str(realClassId) + '.jpg'))
								
								fp = open(os.path.join(webpath,'confident_' + str(realClassId) +".txt"),'w')
								fp.write(className)
								fp = open(os.path.join(webpath,str(realClassId)+'.result'),'w')
								fp.write( str(error)+ " "+ str(hullarea)+ " " + str(hullratio))
								doneclasses[classId] = 1000
			
			
if __name__=="__main__":
	set_printoptions(precision = 3)
	set_printoptions(threshold=nan)
	
	if len(sys.argv)>1:
		if sys.argv[1] == "interactive":
			app = wx.PySimpleApp()
			frame = MatcherFrame()
			app.MainLoop()
		else:
			app = wx.PySimpleApp()
			
			data = dataread.newread()
			frame = MatcherFrame(configdata = data)
			app.MainLoop()
	else:
		app = wx.PySimpleApp()
		frame = MatcherFrame()
		app.MainLoop()
	