from numpy import *
import struct,subprocess,os,wx,time
import geoMatch
import util
from constants import *



lor = logical_or

def geth(modelSize,factor = 10):
	"Weights for space, rotation, scale"
	return ((modelSize/factor)**2,0.3,0.16)

def errorfunc(dx,do,ds,h):
	"Given the errors dx,do,ds, return gaussian score using the wieghting vector h"
	ex = -sum((dx/self.scale)**2,axis=1)/h[0]
	eo = -do**2/h[1]
	es = -ds**2/h[2]
	err =[exp(ex[i]+eo[i]+ex[i]) for i in range(len(eo))]
	return (sum(err),array(err))

def myrescale(im,maxX,maxY):
	"Returns a tuple for an image so that width,heigh<maxX,maxY"
	maxX= float(maxX)
	maxY = float(maxY)
	rescale = min(maxX/im.GetWidth(),maxY/im.GetHeight())
	if rescale>1:
		rescale = 1
	return (int(im.GetWidth()*rescale),int(im.GetHeight()*rescale))

def getL2(x,y):
	"returns a matrix D[i,j] = euclidist between x[i,:] and y[j,:]" 
	return sqrt((-2*dot(x,y.T)+sum(x**2,1).reshape(-1,1))+ sum(y**2,1).reshape(1,-1)) 

def myrange(a,b):
	return int16(linspace(a,b,b-a,False))

def land(x):
	if len(x)>1:
		return logical_and(x[0],land(x[1:]))
	else:
		return x


class Matches:
	'This class contains a set of matched SIFT features, and various operations dealing with geometric relations between these two point sets'
	def __init__(self,model=None,test=None,indexFrom=None,indexTo=None,parent=None, matchList = None):
		if parent!=None:
			self.model = parent.model
			self.test = parent.test
			self.nData = parent.nData
		elif matchList!=None:
			self.model = SIFT(siftList = [data.model for data in matchList])
			self.test = SIFT(siftList = [data.test for data in matchList])
			self.nData = len(matchList)
		else:
			self.model = SIFT(baseSIFT = model,index = indexFrom)
			self.test = SIFT(baseSIFT = test,index = indexTo)
			self.nData = self.model.nData
	def getAllLocations(self):
		"Returns a tuple (m,t), where m is an array of locations from model, t is same from test" 
		return (self.model.x, self.test.x)
	def getClosestTo(self,yourx,which = 'MODEL'):
		"Given which data to look at ('MODEL' or test), returns a new match containing only the pair that is closest to yourx"
		if which == 'MODEL':
			myx = self.model.x
		else:
			myx = self.test.x
		dists = getL2(yourx,myx).reshape(-1,1)
		index =  [int(argmin(dists))]
		return self.subset(index)
	def getTransform(self):
		'Returns the transform induced by a match, assumes a single pair'
		return geoMatch.getTransform(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s)
	
	#def getUnderTransform(self,rescale,reori,t):
		#'Returns ((transDiff,oriDiff,scaleDiff),(transError,oriError,scaleError,err)'
		#modelSize = self.model.meanSize 
		#return geoMatch.getUnderTransform(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s,rescale,reori,t,geth(modelSize))
			
	def getDiff(self,transform):
		'Returns the difference the model and test features once the transform has been applied to the model features'
		return transform.getDiff(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s)
	
	def getError(self,transform,factor = 10):
		"""Returns the error = ((ex,eo,es),err,sum(err)) after the transform has been applied. 
		e? is per match contributions to error for diff variables \012
		err is array of per match contributions"""
		modelSize = self.model.meanSize 
		return transform.getError(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s,geth(modelSize,factor))
		
	def getErrorFast(self,transform):
		modelSize = self.model.meanSize 
		return transform.getErrorFast(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s,geth(modelSize))
	
#	def geoMatch(self,modelSize,transform = None,factor=10):
	def getBestGeoMatch(self,modelSize,transform = None,factor=10):
		'If no transform provided, finds best similarity transform and returns (score for best transform, subset of matches that agree reasonably with the transform,transform)' 
		modelSize = self.model.meanSize
		if transform==None:
			same = self.model.markSame() 
			totalError,localError,otherError = geoMatch.simMatch(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s,geth(modelSize),same)
			bestMatch = argmax(totalError)
			i = bestMatch
			transform = geoMatch.getTransform(self.model.x[i],self.test.x[i],self.model.o[i], self.test.o[i],self.model.s[i],self.test.s[i])
			index = where(localError[bestMatch]>MIN_CONTRIBUTE_ERR)[0]
			return (totalError[bestMatch],self.subset(index),transform)
		else:
			same = self.model.markSame()
			junk,itemError,totalError = transform.getError(self.model.x,self.test.x,self.model.o, self.test.o,self.model.s,self.test.s,geth(modelSize,factor = factor),same)
		#	print itemError[itemError>0.05]
			index = where(itemError>MIN_CONTRIBUTE_ERR)[0]
			self.tmperr = itemError[index]
			return (totalError,self.subset(index),transform)
			
		
	def restrictMatches(self,start,end,which='MODEL'):
		"Returns a Matches such that it retains only those pairs where the location of the 'which' feature is between start and end"
		if which == 'MODEL':
			x = self.model.x
		else:
			x = self.test.x
		insideIndex = where(land((x[:,0]>start[0],x[:,0]<end[0], x[:,1]>start[1],x[:,1]<end[1])).reshape(-1))[0]

		return Matches(self.model,self.test,insideIndex,insideIndex)
	def subset(self,index):
		'Return only the Matches in index'
		return Matches(self.model,self.test,index,index)	
	
	
class State:
	'This contains the pair of images to be compared and the data used to compute that comparison and the functions to do the comparison'
	def __init__(self,modelDatum,testDatum):
		self.test = testDatum
		self.model = modelDatum
		self.match = None

	def getMatches(self):
		if self.match==None:
			self.match = Matches(self.model.sift,self.test.sift,self.indexFrom,self.indexTo)
		return self.match
	
	def compareDatum(self,mode = "1_TO_1",restrict=None):
		"""Finds the nearest neighbours based upon the L2 norm. 
		There are two modes the first 1_TO_1, which insists a match are closest to each other, and NN which is standard.
		The results are stored in two index arrays where indexTo[i] is the nearest neighbour, from test, to indexFrom[i] of model """
		if restrict==None:
						
			self.dist =  self.model.sift.compare(self.test.sift)
			index1 = argmin(self.dist,axis=1) 
			if (self.dist.shape[0]<2) | (self.dist.shape[1]<2):
				self.indexFrom=[]
				self.indexTo=[]
				return "Bad"
			if mode == "1_TO_1":
				index2 = argmin(self.dist,axis=0)
				self.indexFrom = myrange(0,len(index1))[index2[index1]==myrange(0,len(index1))]
				self.indexTo = index1[self.indexFrom]
			elif mode == "NN":
				self.indexFrom = myrange(0,len(index1))
				self.indexTo = int16(index1)
		else:
			#visualization code
			#This case is used where I restrict matches to some subregion on either the model or test image. 
			indexModel = where(restrict[0].Contains(self.model.sift.x))[0]
			indexTest = where(restrict[1].Contains(self.test.sift.x))[0]
			if self.dist!=None:
				tmpdist = self.dist[indexModel][:,indexTest]
			else:
				tmpdist = self.model.sift.compare(self.test.sift,indexModel,indexTest)
			if (len(indexModel)<2) | (len(indexTest)<2) | size(tmpdist)<4:
				self.indexFrom =[]
				self.indexTo = []
				return "Bad"
			index1 = argmin(tmpdist,axis=1)
			index2 = argmin(tmpdist,axis=0)
			indexFrom = myrange(0,len(index1))[index2[index1]==myrange(0,len(index1))]
			indexTo = index1[indexFrom]
			self.indexFrom = indexModel[indexFrom]
			self.indexTo = indexTest[indexTo]

		return "Good"
		
	def completeMatch(self):
		'Takes the matches induced by indexFrom/indexTo, and finds if there is a subset that agree on some affine transformation and returns a score for that affine transformation'
		if not hasattr(self,'indexTo'):
			self.compareDatum()
			
		matches = Matches(self.model.sift,self.test.sift,self.indexFrom,self.indexTo)
		error1,tmpmatches,simtransform = matches.getBestGeoMatch(self.model.sift.meanSize)
		self.skew = 1
		
		if tmpmatches.nData<3: # 2 matches or less to no good
			self.error = 0
			return 0	
		transform = geoMatch.leastSquareAffine(tmpmatches.model.x,tmpmatches.test.x)
		
		self.error,self.match,junk = matches.getBestGeoMatch(self.model.sift.meanSize,transform,factor = 20)
		
		if self.match.nData<3:
			self.error = 0
			return 0	
		
		modelRect = self.model.rect
		testRect = util.MyRect(rect = modelRect,transform = transform)

		self.tmperr = matches.tmperr
		self.modelRect = modelRect
		self.transform = transform
		self.testRect = util.MyRect(rect = modelRect,transform = transform) 
		self.skew = abs(self.transform.getSkew())
	
		return self.error
	
		
	def computeGeoMatch(self,transform=None):
		x = self.model.sift.x
		modelSize = mean(amax(x,axis=0)-amin(x,axis=0))
		matches = Matches(self.model.sift,self.test.sift,self.indexFrom,self.indexTo)
		self.error,self.match,transform =  matches.getBestGeoMatch(modelSize,transform)
	#	print "Best Trans",transform
		return (self.error,self.match)
	
	def getClosestTo(self,data,yourx,mode = 'AppearanceMatches'):
		if data==self.model:
			myx = data.sift.x[self.indexFrom,:] 
		else:
			myx = data.sift.x[self.indexTo,:]
		dists = getL2(yourx,myx).reshape(-1,1)
		index = argmin(dists)
		
		return (self.indexFrom[index],self.indexTo[index])

	
class Datum:
	'Typically an image'
	def __init__(self,imagename,source = 'Undefined',quality = -1,workpath = None):
		self.imagename = imagename
		self.source = source
		self.quality = quality
		if workpath == None:
			self.workpath = os.path.splitext(imagename)[0]
		else:
			self.workpath = os.path.join(workpath,os.path.splitext(os.path.split(imagename)[1])[0])
		
		self.isValid = os.path.isfile(imagename)
	def freeStuff(self):
		if hasattr(self,'image'):
			delattr(self,'image')
		if hasattr(self,'sift'):
			delattr(self,'sift')
	def hasImage(self):
		if hasattr(self,'image'):
			return isinstance(self.image,wx.Image)
		else:
			return false
	def returnImage(self):
		return wx.Image(self.imagename)
	
	def loadThumbnail(self,force = False, maxSize = 100):
		if (not hasattr(self,'thumbnail')) | force:
			if hasattr(self,'image') & self.isValid:
				rescale = myrescale(self.image,maxSize,maxSize) 
				self.thumbnail = self.image.Scale(rescale[0],rescale[1]).ConvertToBitmap()
			else:
				tmpim = wx.Image(self.imagename)
				rescale = myrescale(tmpim,maxSize,maxSize)
				self.thumbnail = tmpim.Rescale(*rescale).ConvertToBitmap()
		return self.thumbnail
	def loadImage(self,imagename = None):
		if imagename!=None:
			self.imagename = imagename
		if os.path.isfile(self.imagename):
			self.image = wx.Image(self.imagename)
			self.isValid  = self.image.Ok()
			if self.isValid:
				self.imsize = self.image.GetSize()
		else:
			self.isValid = False	
	def GetSize(self):
		if hasattr(self,'imsize'):
			return self.imsize
		else:
			self.imsize = wx.Image(self.imagename).GetSize()
			return self.imsize
	
	def loadSIFT(self,filename = None,autoname=False,force = False,renew = False):
		'Compute/load a sift description for this Datum'
		if hasattr(self,'sift') & (not renew):
			return None
		if not self.isValid:
			return None
		if filename!=None:
			self.siftname = filename
		if autoname: 
			#self.siftname = self.imagename[0:self.imagename.rfind('.')] + '.sift'
			self.siftname = self.workpath + '.sift'
		if (not os.path.isfile(self.siftname)) | force:
			print "Running sift at",self.siftname
			self.runSIFT(self.siftname,waitToFinish = True)
		
		self.sift = SIFT(self.siftname)
		if self.sift.nData<3:
			self.isValid = False
			return
		minx = amin(self.sift.x,axis=0)
		maxx = amax(self.sift.x,axis=0)
		self.rect = util.MyRect([(minx[0],minx[1]),(maxx[0],minx[1]),(maxx[0],maxx[1]),(minx[0],maxx[1])])
		
		
	def runSIFT(self,outputFileName = None,waitToFinish = False):
		if outputFileName == None:
			outputFileName = self.workpath + '.sift'
			 #self.imagename[0:self.imagename.find('.')] + '.sift'
		command = [siftExecPath,self.imagename,outputFileName]
		print command
		myprocess = subprocess.Popen(command)
		if waitToFinish:
			myprocess.wait()
		#os.spawnvp(os.P_WAIT,siftExecPath ,['keypoints',self.imagename,outputPath])
		

		
class SIFT:
	'This is basically a datatype class, which has 3 different ways to initialize'
	def __init__(self,filename=None,baseSIFT=None,index=None,siftList= None):
		if filename != None:
			self.filename = filename
			self.load(filename)
			if self.nData>0:
				self.meanSize = mean(amax(self.x,axis=0)-amin(self.x,axis=0))
			else:
				self.meanSize=0
		elif index!=None:
			#Assumes basesIFT is a SIFT
			self.filename = baseSIFT.filename
			if index.__class__ == int:
				index = [index]
			self.nData = len(index)
			self.nDim = baseSIFT.nDim
			self.data = baseSIFT.data[index,:]
			self.X = baseSIFT.X[index,:]
			self.x = baseSIFT.x[index,:]
			self.o = baseSIFT.o[index]
			self.s = baseSIFT.s[index]
			#self.id = baseSIFT.id[index]
			self.ssq = baseSIFT.ssq[index,:]
			self.meanSize = baseSIFT.meanSize
		elif siftList!=None:
			self.filename = siftList[0].filename
			self.nData = len(siftList)
			self.nDim = siftList[0].nDim
			self.data = vstack(tuple([sift.data for sift in siftList]))
			self.X = vstack(tuple([sift.X for sift in siftList]))
			self.x = vstack(tuple([sift.x for sift in siftList]))
			self.o = array([sift.o for sift in siftList])
			self.s = array([sift.s for sift in siftList])
			self.ssq = vstack(tuple([sift.x for sift in siftList]))
			self.meanSize = siftList[0].meanSize
			
			
	def isolate(self,subindex):
		'Returns a list of sifts from this sift, that lies at index'
		return [SIFT(baseSIFT =self,index=i) for i in subindex]
	
	def subset(self,subindex):
		'returns a new SIFT class with features at subindex'
		return SIFT(baseSIFT=self,index = subindex)
		
	def load(self,filename):
		'Reads in the sift descriptors from a file'
		fid = open(filename,'r')
		self.nData = struct.unpack('i',fid.read(4))[0]
		self.nDim = struct.unpack('i',fid.read(4))[0]
		tmpData  = fromfile(fid,'uint8', self.nData*self.nDim).reshape(self.nData, self.nDim)
		self.data = float32(tmpData)
		tmpData = fromfile(fid,'float32',4*self.nData).reshape(-1,4)
		
		if tmpData.shape[0]>MAX_NDATA:
			#If too many keypoints, keep only the the largest
			index = argsort(tmpData[:,2])
			keepindex = index[-MAX_NDATA:]
			tmpData = tmpData[keepindex,:]
			self.data = self.data[keepindex,:]
			self.nData = self.data.shape[0]
		self.X = tmpData[:,[3,2,0,1]].copy()
		self.x,self.o,self.s = self.X[:,[0,1]], self.X[:,3], self.X[:,2]
		self.ssq = sum(self.data**2,axis=1).reshape(-1,1)
	
	
	def markSame(self):
		'Demarkates which features lie at the same location (ie. multi representations of same region). Returns an array if same[i]=same[j] then i and j are at the same point, but only for non-zero values'
		a = (diff(self.x,axis=0)==0).all(axis=1)
		same = zeros(self.nData)
		c=1
		for i in range(len(a)):
			if a[i]:
				same[i] = c
				same[i+1] = c
				if i+1<len(a):
					if a[i] != a[i+1]:
						c+=1
		return same-1

	def compare(self,other,myindex = None,otherindex = None):
		'Compares a set of sift features to another, result[i,j], i of self, j of other'
		if myindex == None:
			result = sqrt((-2*dot(self.data,other.data.T) + self.ssq) + other.ssq.T)
		else:
			result = sqrt((-2*dot(self.data[myindex],other.data[otherindex].T) + self.ssq[myindex]) + other.ssq[otherindex].T)
		return result
		
	def getLocation(self,index):
		return self.x[index,:]




if __name__ == "__main__":

	pass
	
