from numpy import *
from constants import *
import sift,geoMatch,convexhull,util



class SupportImage:
	def __init__(self,matches,errors,imsize,MIN_MEM = True):
		self.x = matches.model.x
		self.s = matches.model.s
		self.imsize = imsize
		self.errors = errors
		self.modelSize = matches.model.meanSize
		self.computeSupport(MIN_MEM = MIN_MEM)
	def setToMax(self):
		self.data = 1
		self.err = 1
	def computeSupport(self,KEEP_DATA = False,MIN_MEM = True):
		"""Goes through each matched feature and adds a box over the region associated with the feature (8*s) with weight error/area.
		In essence, this is determining a rough score how much (area) of the model image agrees with something in the test image. 
		"""
		
		data = zeros(self.imsize).T
		for i in range(len(self.errors)):
			x = int16(self.x[i]).reshape(-1)
			s = int16(self.s[i]*8)
			area = s # 1 or s or s*s
			sx = x-s
			sx[sx<0]=0
			if self.errors[i]>MIN_CONTRIBUTE_ERR:
				data[sx[1]:x[1]+s,sx[0]:x[0]+s] = data[sx[1]:x[1]+s,sx[0]:x[0]+s]+self.errors[i]/area
		data[data>1] = 1
		self.data = data
		if KEEP_DATA:
			self.err = sum(data)/self.modelSize**2
			self.origerr = self.err
			if self.err<0.01:
				self.data= None
			else:
				if MIN_MEM & (self.err>IDENTICAL):
					self.data = 1
				else:
					self.data = data/self.modelSize**2
			return self.err
		else:
			self.err = sum(data)/self.modelSize**2
			return self.err
	def subtract(self,other):
		if self.data!=None:
			self.data -= other.data
			self.err = sum(self.data(self.data>0))
			if self.err<0.01:
				self.data = 0
			return self.err
		else:
			return 0 
	def getErr(self):
		if self.err>1:
			self.err =1
		return self.err
	#def Show(self):
		#frame = wx.Frame(None,size = image.GetSize())
		#wx.StaticBitmap(self,bitmap = self.im.ConvertToBitmap())
		#frame.Fit()
		#frame.Show(True)


def findIdentical(A):
	'Finds those images that can be considered to be reasonably identical, and returns them as a list of lists of indices'
	D = A>IDENTICAL
	D = D*D.T
	soloIndex = where(sum(D,axis=1)<2)[0]
	D[soloIndex] = False
	mylist = []
	while True:
		index = where(D)[0]
		if len(index)>0:
			index = where(D[index[0]])[0]
			mylist.append(index)
			D[index] = False
			D[:,index] = False
		else:
			break
	return mylist

def rankImages(supportDist,identical,invalid,quality):
	n = supportDist.shape[0]
	score = sum(supportDist,axis=1)
	indkeep = zeros((len(identical)))
	unusedImages = ones((n))
	unusedImages[invalid] = 0
	for index in identical:
		maxind = argmax(score[index])
		unusedImages[index] = 0
		supportDist[:,index[0:maxind]] = 0
		supportDist[:,index[maxind+1:-1]] = 0
		unusedImages[index[maxind]] = 1
	
	currentBest = zeros((1,n))
	unusedImages = unusedImages==1
	rankIndex = []
	i=0
	while False:
		i=i+1
	#	print "New Loop"
		score =	sum((supportDist[unusedImages]>currentBest)*(supportDist[unusedImages] - currentBest),axis=1)
		if len(score)==0 :
			break
		maxind = where(unusedImages)[0][argmax(score)]
		betterWhere = (supportDist[maxind]>currentBest).reshape(-1)
	#	print currentBest
	#	print "MAX ", argmax(score),score[argmax(score)]
	#	print supportDist[unusedImages]
		
		if sum(betterWhere)<2:
			break

		currentBest[:,betterWhere] = supportDist[maxind][betterWhere]
		unusedImages[maxind] = False
		#print unusedImages
		if not betterWhere.any():
			break
		rankIndex.append(maxind)


	remain = where(unusedImages)[0]

	rankIndex.extend(list(remain))
#Uses quality
#	remainorder = argsort(quality[remain])[-1::-1]
#	rankIndex.extend(list(remain[remainorder]))
	
	return rankIndex

def train(classList,savedresults = None):
	results= range(len(classList))
	for k,cls in enumerate(classList):
		imgList = cls[2]
		print cls[0]
		invalidList=[]
		quality = zeros(len(imgList))
		for i,im in enumerate(imgList):
			if (not im.isValid) | (im.source==bad_source) | (im.quality<0.01):
				invalidList.append(i)
			quality[i] = im.quality
				
		if savedresults==None:
			dists = zeros((len(imgList),len(imgList)))
			supportDist = zeros((len(imgList),len(imgList)))
			areaRatioDist = zeros(((len(imgList)),len(imgList)))
			for im in imgList:
				if im.isValid & (not im.source==bad_source):
					im.loadSIFT(autoname=True,force= False)
					if im.sift.nData<3:
						im.isValid = False
			print "Starting training for", cls[0]
			for i,imi in enumerate(imgList):
			#	print imi.imagename
				for j,imj in enumerate(imgList):
					if (not imi.isValid) | (imi.source==bad_source):
						break
					if imj.source == bad_source:
						pass
					elif (i!=j) & imj.isValid:
						state = sift.State(imi,imj)
						error = state.completeMatch()
						dists[j,i] = error
						if error>30:
							supportDist[j,i] = 1
						elif error>MIN_ERROR:
							supportDist[j,i] = SupportImage(state.match,state.tmperr,imsize =state.model.GetSize()).getErr()
							areaRatioDist[j,i] = convexhull.areaOfConvexHull(state.match.model.x)/(state.match.model.meanSize**2)
							
					elif i==j:
						dists[j,i]=100
						supportDist[j,i] = 1
		#	print dists	
			for im in imgList:
				im.freeStuff()
		
			results[k] = [supportDist.copy(),dists,areaRatioDist]
		else:
			supportDist,dists = savedresults[k]

		identical = findIdentical(supportDist)
		rankIndex = rankImages(supportDist.copy(),identical,invalidList,quality)
		print cls[0]
		print k,"Identical",identical

		cls[2] = [cls[2][i] for i in rankIndex]
		
	print "Done"
	return results

