from numpy import *
import wx,time,os,pickle
#These are my files
import sift, util,geoMatch
from constants import *
import dataread
import time

def addButtonToSizer(parent,sizer,mylabel,handler):
	"Adds a button, along with it's handler to a sizer"
	button = wx.Button(parent,wx.ID_ANY,label = mylabel)
	button.Bind(wx.EVT_BUTTON,handler)
	sizer.Add(button,1)
	return button 



class VisualRect(util.MyRect):
	"This class is simply to visualize a transform of some rectangle in an image"
	def __init__(self,rect=None,displayfunc=None,castrect = None):
		if castrect==None:
			plist = [rect.GetTopLeft(),rect.GetTopRight(),rect.GetBottomRight(), rect.GetBottomLeft()]
			util.MyRect.__init__(self,plist)
		else:
			util.MyRect.__init__(self,rect = castrect)
			#print self.transform
		
		self.display = displayfunc
	def draw(self,canvas=None,pen = None,brush = wx.TRANSPARENT_BRUSH):
		if self.transform != None:
			tlist = self.display(self.transform.transform(self.plist),asPoints = True)
		else:
			tlist = self.display(self.plist,asPoints = True)
		canvas.SetBrush(brush)
		if pen == None:
			pen = wx.Pen(wx.Colour(0,0,255))
		canvas.SetPen(pen)
		tlist.append(tlist[0])	
		canvas.DrawPolygon(tlist)

		
class SuggestTransform:
	def __init__(self,modelDisplay,testDisplay):
		self.modellines= []
		self.testlines = []
		self.modelDisplay =modelDisplay
		self.testDisplay = testDisplay
	def addLine(self,mytype,line):
	
		if mytype == "MODEL":
			self.modellines.extend(line)
		else:
			self.testlines.extend(line)
		return self.getTransform()
	def getTransform(self):
		if (len(self.modellines)==2) & (len(self.testlines)==2):
			transform = geoMatch.leastSquare(array(self.modellines,ndmin=2),array(self.testlines[0],ndmin=2))
		elif (len(self.modellines)==len(self.testlines)) & (len(self.modellines)>2):
			transform = geoMatch.leastSquareAffine(array(self.modellines,ndmin=2),array(self.testlines,ndmin=2))
		else:
			transform = None
		return transform
			
	def draw(self,canvas,pen=None):
		if pen!=None:
			canvas.SetPen(pen)
		
		if len(self.modellines)>0:
			l = self.modelDisplay(array(self.modellines)).reshape(-1,4)
			canvas.DrawLineList(list(l))
		if len(self.testlines)>0:
			l = self.testDisplay(array(self.testlines)).reshape(-1,4)
			canvas.DrawLineList(list(l))
		
		
class VisualTransform:
	def __init__(self,parent,modelDisplay,testDisplay):
		self.parent = parent
		self.modelDisplay = modelDisplay
		self.testDisplay = testDisplay
		self.is_active = False
		self.transform=None
		self.container = None
	def transform(self,x):
		return self.transform(x)
	def drawTransform(self,rect,display,canvas):
		rect = VisualRect(rect,display)
		rect.setTransform(self.transform)
		rect.draw(canvas)
	def setTransform(self,transform):
		self.transform = transform
	def setLine(self,which,line):
		'This presumes line is a tuple of (start,end)'
		if self.container.__class__ != SuggestTransform:
			self.container = SuggestTransform(self.modelDisplay,self.testDisplay)
		self.transform = self.container.addLine(which,line)
		self.is_active = True
	def setTransformUsingPair(self,pair):
		self.container = pair
		self.transform = pair.getTransform()
		self.is_active = True
	def getTransform(self):
		return self.transform
	def getUnderTransform(self,matches):
		diff = matches.getDiff(self.transform)
		dimErr,itemErr,allErr = matches.getError(self.transform)
		# matches.getUnderTransform(*self.transform)
		return (diff,dimErr,itemErr,allErr)
		
	def draw(self,canvas = None):
		if self.is_active:
			if canvas == None:
				canvas = wx.PaintDC(self.parent)
			
			self.container.draw(canvas,wx.CYAN_PEN)
		
	def clear(self,event=None):
		self.container = None
		self.is_active = False
		self.transform = None

			
class VisualDatum(wx.Image):
	def __init__(self,imagename = None,offset=0,image = None):
		if image==None:
			if isinstance(imagename,sift.Datum):
				wx.Image.__init__(self,imagename.imagename)
			else:	
				wx.Image.__init__(self,imagename)
		else:
			self.__dict__ = image.__dict__
		#wx.Image.__init__(self,data.imagename)
		#self.data = data
		self.offset =offset
		self.scale = 1
		self.viewable = wx.Rect(0,0,self.GetWidth(),self.GetHeight())
		self.maxX = 500
		self.maxY = 500
	def setOffset(self,offset):
		self.offset = offset	
	def setDisplayBounds(self,x,y):
		self.maxX = x
		self.maxY = y
	def getViewable(self):
		return self.viewable
	def getViewableImage(self):
	
		im = self.GetSubImage(self.viewable)
		im.Rescale(self.scale*self.viewable.GetWidth(),self.scale*self.viewable.GetHeight())
		return im
	def contains(self,point):
		p = self.getImageLoc(point)
		return self.viewable.Contains(p)
	def setViewable(self,p1=None,p2=None,maxX = None, maxY = None):
		if maxX == None:
			maxX = self.maxX
			maxY = self.maxY
		if p1 == None:
			rescale = min(maxX/float(self.GetWidth()),maxY/float(self.GetHeight()))
			self.viewable.Set(0,0,self.GetWidth(),self.GetHeight())
		else:
			p1[p1<0]=0
			p2[p2>=self.GetSize()] = (array(self.GetSize())-1)[p2>=self.GetSize()]
			
			self.viewable.Set(p1[0],p1[1],p2[0]-p1[0],p2[1]-p1[1])
			
			rescale = min(maxX/float(self.viewable.GetWidth()),maxY/float(self.viewable.GetHeight()))
		if rescale<3:
			self.scale = rescale
		else:
			self.scale = 3
	def getBmpSize(self):
		return self.mybmp.GetSize()
	def getImageLoc(self,point):
		if hasattr(point,'Get'):
			point = array(point.Get())
		else:
			point = array(point)
	#	point[0] = point[0]-self.offset
		point[0] = point[0] - self.offset + self.scale*self.viewable.GetX()
		point[1] = point[1] + self.scale*self.viewable.GetY()
		return around(point/self.scale)
	def getDisplayLoc(self,x,asPoints = False):

		if x.__class__ != ndarray:
			x = array(x,ndmin=2)
			
		if x.ndim==2:
			val = x*self.scale
		else:
			val = x.reshape(1,-1)*self.scale
#
		val[:,0] += self.offset - self.scale*self.viewable.GetX()
		val[:,1] += -self.scale*self.viewable.GetY()
		val = around(val)
		
		if asPoints == True:
			val = [wx.Point(*val[i,:]) for i in range(x.shape[0])]
		return val

class VisualMatches(sift.Matches):
	def __init__(self,myparent,mypanel=None,modelDisplayFunc=None,testDispplayFunc=None):
		#sift.Matches.__init__(self,parent = myparent)
		#self.modelimage = parentpanel.model
		#self.testimage = parentpanel.test
		if mypanel==None:
			self.__dict__ = myparent[0].__dict__
			self.getDisplayLocModel = myparent[1].getDisplayLocModel
			self.getDisplayLocTest = myparent[1].getDisplayLocTest
			self.parent = myparent[1].parent
		else:
			self.__dict__ = myparent.__dict__
			self.getDisplayLocModel = modelDisplayFunc
			self.getDisplayLocTest = testDispplayFunc
			self.mypanel = mypanel
	def draw(self,canvas=None,pen = wx.RED_PEN):
		if canvas == None:
			canvas = wx.PaintDC(self.parent)
		modelp,testp = self.getAllLocations()
		#self.drawLines(modelp,testp,canvas = canvas,pen = pen)
		#modelp = self.modelimage.getDisplayLoc(modelp)
		#testp = self.testimage.getDisplayLoc(testp)
		modelp = self.getDisplayLocModel(modelp)
		testp = self.getDisplayLocTest(testp)
		lines = [(modelp[i,0],modelp[i,1],testp[i,0],testp[i,1]) for i in range(modelp.shape[0])]
		canvas.SetPen(pen)
		canvas.DrawLineList(lines)
	def restrictMatches(self,start,end,which='MODEL'):
		return VisualMatches((sift.Matches.restrictMatches(self,start,end,which),self))
	def subset(self,index):
		return VisualMatches((sift.Matches.subset(self,index),self))
	
		
		
class ImgFrame(wx.Frame):
	def __init__(self,parent,imgList,imageSelectHandler):
		self.MIN_IM = [5,5]
		wx.Frame.__init__(self,parent,wx.ID_ANY)
		self.imgList = imgList
		self.currIndex = 0
		self.imageSelectHandler = imageSelectHandler
		self.populate()
		self.Fit()
		self.Show(True)
	def populate(self):
		if hasattr(self,'panel'):
			self.panel.Destroy()
		if hasattr(self,'buttonpanel'):
			self.buttonpanel.Destroy()
		self.panel = wx.Panel(self)
		sizer = wx.GridSizer()
		lasti=0
		for i in range(self.MIN_IM[0]*self.MIN_IM[1]):
			lasti = i
			ri = self.currIndex + i
			if ri>=len(self.imgList):
				break
			if self.imgList[ri].isValid:
				tmpim = wx.StaticBitmap(self.panel,ri,self.imgList[ri].loadThumbnail())
				tmpim.Bind(wx.EVT_LEFT_DOWN,self.imageSelectHandler)
				tmpim.Bind(wx.EVT_RIGHT_DOWN,self.displayInfo)
				
				sizer.Add(tmpim)
		ni = ri-self.currIndex+1
		r = int(sqrt(ni))
	
		c = r+2 - 2*int(r==sqrt(ni))

		sizer.SetRows(r)
		sizer.SetCols(c)
		self.panel.SetSizerAndFit(sizer)
		
		if not hasattr(self,'buttonheight'):
			self.buttonheight = self.panel.GetSize().y+1
		self.buttonpanel = wx.Panel(self,pos=(0,self.buttonheight+1))
		sizer = wx.BoxSizer(wx.HORIZONTAL)
		if ri-self.MIN_IM[0]*self.MIN_IM[1]>0:
			addButtonToSizer(self.buttonpanel,sizer,"Prev",self.prev)
		if ri<len(self.imgList):
			addButtonToSizer(self.buttonpanel,sizer,"Next",self.next)
		self.buttonpanel.SetSizerAndFit(sizer)
		self.buttonpanel.Center(wx.HORIZONTAL)
		self.DISPLAY_ON_RIGHT = True
	def next(self,event):
		self.currIndex += self.MIN_IM[0]*self.MIN_IM[1]
		self.populate()
	def setInfo(self,info):
		self.DISPLAY_ON_RIGHT = True
		self.info = info
	#	self.Bind(wx.EVT_RIGHT_DOWN,self.displayInfo)
	#	self.Bind(wx.EVT_RIGHT_UP,self.displayInfo)
	#	self.isDown = False
	def prev(self,event):
		self.currIndex -= self.MIN_IM[0]*self.MIN_IM[1]
		if self.currIndex<0:
			self.currIndex = 0
		self.populate()	
	def displayInfo(self,event):
		if not self.DISPLAY_ON_RIGHT:
			return None
		if event.ButtonDown(wx.MOUSE_BTN_RIGHT):
			#self.isDown = True
			menu = wx.Menu()
			menu.Append(wx.ID_ANY,str(self.info[event.GetEventObject().GetId()]))
			event.GetEventObject().PopupMenu(menu, event.GetPosition() )

	def addButton(self,label,bindfunc):
		addButtonToSizer(self.buttonpanel,self.buttonpanel.GetSizer(),label,bindfunc)
		self.buttonpanel.Fit()
	#	self.buttonpanel.SetSizerAndFit(sizer)
		self.buttonpanel.Center(wx.HORIZONTAL)
		self.Fit()
		self.Show(True)


