import sys, os, pickle, struct

from PIL import Image
from pylab import *
from optparse import OptionParser

def quadrant(width, height, x, y) :
    if ( x < width / 2  and y < height / 2 ):
        return 1
    elif ( x >= width / 2 and y < height / 2):
        return 2
    elif ( x < width / 2 and y >= height / 2):
        return 3
    else:
        return 4

def main() :
    parser = OptionParser()
    parser.add_option("-d", "--codeword-directory", dest="codewordDirectory")
    parser.add_option("-c", "--codebook", dest="codebookPath")
    parser.add_option("-s", "--selection-file", dest="selectionPath")
    parser.add_option("-t", "--top", type="int", dest="top")
    (options, args) = parser.parse_args()

    if ( not options.codebookPath ) :
        parser.error("option --codebook is necessary")
        exit
    if ( not options.selectionPath ) :
        parser.error("option --selection-file is necessary")
        exit

    selectionFile = open(os.path.normpath(options.selectionPath), 'r')
    (selectionSize,) = struct.unpack( 'i', selectionFile.read(struct.calcsize('i')) )
    codewordWeights = dict.fromkeys(range(0,selectionSize))
    for s in range(0,selectionSize):
        (w,) = struct.unpack( 'f', selectionFile.read(struct.calcsize('f')) )
        codewordWeights[s] = w;
    selectionFile.close()
    orderedCodewords = sorted(codewordWeights.iteritems(), key=lambda (k,v): (v,k), reverse=True)
    
    if ( not options.top ):
        options.top = selectionSize

    ranks = []
    for s in range(0, selectionSize) :
        for i in range(0, selectionSize) :
            if ( orderedCodewords[i][0] == s ) :
                ranks.append(i)

    codebookFile = open(os.path.normpath(options.codebookPath), 'r')
    centroids = pickle.load(codebookFile)
    codebookFile.close()

    if ( selectionSize != len(centroids) ):
        raise RuntimeError, ('Selection file size %d doesn\'t match codebook size %d' % (selectionSize,len(centroids)))
    
    histogramSize = options.top + 4 * options.top
    print 'Histogram size: %d' % histogramSize

    figure(0)

    count = 1
    for imagePath in args :
        if ( not options.codewordDirectory ) :
            codewordDirectory = os.path.dirname(imagePath)
        else :
            codewordDirectory = options.codewordDirectory

        img = Image.open(imagePath)
        filebase = os.path.splitext(os.path.basename(imagePath))[0]
        codewordPath = os.path.join(codewordDirectory, filebase + ".codewords")
        codewordFile = open(codewordPath, 'rb')
        (numCodewords,) = struct.unpack('i',codewordFile.read(struct.calcsize('i')))
        print '%s: %d' % (codewordPath,numCodewords)
        codewords = []
        for i in range(0,numCodewords) :
            (ident,x,y,scale) = struct.unpack('ifff',codewordFile.read(struct.calcsize('ifff')))
            if (ranks[ident] < options.top):
                codewords.append(ranks[ident])
                q = quadrant(img.size[0], img.size[1], x, y)
                codewords.append(q * options.top + ranks[ident])
        subplot(len(args), 1, count)
        hist(codewords,histogramSize,normed=True)
        ylim(0, 0.05)
        count += 1
    show()

if ( __name__ == "__main__" ) :
    main()
