import sys, os, glob
from optparse import OptionParser
import wx
import numpy
import scipy.special
from wx.lib.pubsub import Publisher
from math import sqrt
from math import exp
from math import pi
sys.path.append( '../../../flann/devel/src/bindings/python' )
sys.path.append( os.path.normpath(os.path.join(os.getcwd(), '../SRVC') ) )
import sjm_util
import pypyramid_ext
ID_OPEN=101
ID_CLASS=100
ID_EXIT=110
ID_TESTDATA=111
ID_INFO1=112

class ImageExplorer(wx.Panel):
    def __init__(self, parent, id, imagePath):
        wx.Panel.__init__(self, parent, id)
        self.imagePath = imagePath
        if ( imagePath != None ):
            self.bitmap = wx.Bitmap(imagePath)
            self.integralHistogram = pypyramid_ext.get_integral_histogram(imagePath.encode('utf-8'))
        else:
            self.bitmap = None
        self.startx = None
        self.endx = None
        self.starty = None
        self.endy = None

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseDown)
        self.Bind(wx.EVT_MOTION, self.OnMouseMotion)
        self.Bind(wx.EVT_LEFT_UP, self.OnMouseUp)
        
        self.Show(True)

    def SetCodewordPath(self,codewordPath):
        self.codewordPath = codewordPath

    def SetSelectionSet(self,selectionSet):
        self.selectionSet = selectionSet

    def SetBitmap(self,imagePath):
        self.imagePath = imagePath
        self.bitmap = wx.Bitmap(imagePath)
        self.integralHistogram = pypyramid_ext.get_integral_histogram(imagePath.encode('utf-8'))
        self.startx = None
        self.endx = None
        self.starty = None
        self.endy = None
        self.Refresh()

    def image_size(self):
        return self.bitmap.GetSize()

    def OnPaint(self, e):
        dc = wx.PaintDC(self)
        if self.bitmap:
            dc.DrawBitmap(self.bitmap, 0,0)
        if ( self.startx and self.endx and self.starty and self.endy ):
            dc.SetBrush(wx.Brush(wx.Colour(), wx.TRANSPARENT))
            dc.SetPen(wx.Pen(wx.Colour(255,0,0,0.8)))
            bbox_left = min(self.startx, self.endx)
            bbox_right = max(self.startx, self.endx)
            bbox_top = min(self.starty, self.endy)
            bbox_bottom = max(self.starty, self.endy)
            dc.DrawRectangleRect(wx.Rect(bbox_left, bbox_top, bbox_right-bbox_left, bbox_bottom-bbox_top))
            
    def OnMouseDown(self, e):
        if ( self.bitmap ):
            self.startx = e.GetX()
            self.starty = e.GetY()
            self.endx = e.GetX()
            self.endy = e.GetY()

    def OnMouseMotion(self, e):
        if ( self.bitmap ):
            if ( e.ButtonIsDown(wx.MOUSE_BTN_LEFT) ) :
                self.endx = e.GetX()
                self.endy = e.GetY()
            self.Refresh()

    def OnMouseUp(self, e):
        if ( self.bitmap ):
            tempCodewordPath = "temp_subset.codewords"
            bbox_left = min(self.startx, self.endx)
            bbox_right = max(self.startx, self.endx)
            bbox_top = min(self.starty, self.endy)
            bbox_bottom = max(self.starty, self.endy)
            sjm_util.subset_codewords(self.codewordPath, tempCodewordPath, bbox_left, bbox_top, bbox_right, bbox_bottom, 200, 200)
            self.pyramid = pypyramid_ext.pyramid_from_codewords(tempCodewordPath, self.selectionSet, bbox_right - bbox_left, bbox_bottom - bbox_top, 3)
            os.remove(tempCodewordPath)
            self.cpyramid = pypyramid_ext.colour_pyramid_from_histogram(self.integralHistogram, bbox_left, bbox_top, bbox_right - bbox_left + 1, bbox_bottom - bbox_top + 1, 3)
            Publisher().sendMessage(('pyramid','ready'),'Ready')

class ColouredBitmap(wx.Panel):
    def __init__(self, parent, id, imagePath):
        wx.Panel.__init__(self, parent, id)
        if ( imagePath != None ):
            self.bitmap = wx.Bitmap(imagePath)
            self.SetSize(self.bitmap.GetSize())
        else:
            self.bitmap = None
        self.brush = wx.Brush(wx.Colour(), wx.TRANSPARENT)
        self.Show(True)
        self.Bind(wx.EVT_PAINT,self.OnPaint)

    def SetBitmap(self, imagePath):
        self.bitmap = wx.Bitmap(imagePath)

    def OnPaint(self, e):
        dc = wx.PaintDC(self)
        if self.bitmap:
            dc.DrawBitmap(self.bitmap, 0,0)
        dc.SetBrush(self.brush)
        dc.DrawRectangleRect(wx.Rect(0, 0, self.bitmap.GetSize()[0], self.bitmap.GetSize()[1]))
            
    def SetScore(self, score):
        #self.brush = wx.Brush(wx.Colour(0,0,0,255))
        self.Refresh()

class RightFrame(wx.ScrolledWindow):
    def __init__(self, parent, id):
        wx.ScrolledWindow.__init__(self, parent, id)
        self.Show(True)
        self.imageList = None

    def SetImageList(self, l):
        self.imageList = l
        self.sizer = wx.BoxSizer(wx.VERTICAL)

        self.colouredBitmaps = []
        index = 0
        for img in l:
            self.colouredBitmaps.append(ColouredBitmap(self, wx.ID_ANY, img))
            self.sizer.Add(self.colouredBitmaps[index], 0, wx.EXPAND, 2)
            index += 1
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)
        self.sizer.Fit(self)
        self.SetScrollbars(5, 5, self.sizer.GetSize()[0], self.sizer.GetSize()[1])
        self.Refresh()

    def SetScores(self, scoreList):
        for i in range(0, len(scoreList)):
            self.colouredBitmaps[i].SetScore(scoreList[i])
            

class MainWindow(wx.Frame):
    def __init__(self, parent, id, title, dirname, selectionSize):
        wx.Frame.__init__(self, parent, id, title)
        
        filemenu= wx.Menu()
        filemenu.Append(ID_CLASS, "&Specify Class Directory")
        filemenu.Append(ID_TESTDATA, "&Test Data Directory")
        filemenu.Append(ID_OPEN, "&Open Image"," Open an image to explore")
        filemenu.AppendSeparator()
        filemenu.Append(ID_EXIT,"E&xit"," Quit the program")
        infomenu = wx.Menu()
        infomenu.Append(ID_INFO1, "Show class consistancy")
        infomenu.Append(ID_INFO1+1, "Show colour consistancy")
        
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar
        menuBar.Append(infomenu,"&Info") # Adding the "infomenu" to the MenuBar
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content
        self.CreateStatusBar(3) # A Statusbar in the bottom of the window

        self.dirname = dirname
        self.selectionSize = selectionSize
        self.classDir = None
        self.codewordsDir = "/cs/SCRATCH/sanchom/SRVC_local/testdata"
        
        wx.EVT_MENU(self, ID_OPEN, self.OnOpen)
        wx.EVT_MENU(self, ID_CLASS, self.OnSpecifyClassDir)
        wx.EVT_MENU(self, ID_TESTDATA, self.OnSelectTestData)
        wx.EVT_MENU(self, ID_INFO1, self.OnClassConsistancy)
        wx.EVT_MENU(self, ID_INFO1+1, self.OnColourConsistancy)
        
        self.imagePath = None
        self.codewordsPath = None
        self.splitter = wx.SplitterWindow(self, wx.ID_ANY)
        self.imagePane = ImageExplorer(self.splitter, wx.ID_ANY, self.imagePath)
        self.rightPane = RightFrame(self.splitter, wx.ID_ANY)
        self.splitter.SplitVertically(self.imagePane, self.rightPane)
        self.splitter.SetSashPosition(self.GetSize()[0] - 100)
        Publisher().subscribe(self._OnPyramidReady, ('pyramid','ready'))
        self.Show(True)

    def _OnPyramidReady(self,message):
        testPyramid = self.imagePane.pyramid
        testCPyramid = self.imagePane.cpyramid
        nearestNeighborScore = 0
        nearestNeighbor = 0
        index = 0
        self.matchScores = []
        for pyramid in self.pyramids:
            score = pypyramid_ext.match_score(pyramid, testPyramid, 3)
            self.matchScores.append(score)
            if ( score > nearestNeighborScore ):
                nearestNeighborScore = score
                nearestNeighbor = index
            index += 1
        self.negativeScores = []
        for pyramid in self.negativePyramids:
            score = pypyramid_ext.match_score(pyramid, testPyramid, 3)
            self.negativeScores.append(score)

        colourNearestNeighborScore = 0
        colourNearestNeighbor = 0
        index = 0
        self.colourMatchScores = []
        for cpyramid in self.colourPyramids:
            score = pypyramid_ext.match_score(cpyramid, testCPyramid, 3)
            self.colourMatchScores.append(score)
            if ( score > colourNearestNeighborScore ):
                colourNearestNeighborScore = score
                colourNearestNeighbor = index
            index += 1
        self.colourNegativeScores = []
        for cpyramid in self.negativeCPyramids:
            score = pypyramid_ext.match_score(cpyramid, testCPyramid, 3)
            self.colourNegativeScores.append(score)

        print 'Positive sift match scores'
        print self.matchScores
        self.matchScores.sort()
        self.matchScores.reverse()
        print 'Positive colour match scores'
        print self.colourMatchScores
        self.colourMatchScores.sort()
        self.colourMatchScores.reverse()
        print 'Negative sift match scores'
        self.negativeScores.sort()
        self.negativeScores.reverse()
        print self.negativeScores[0:5]
        print 'Negative colour match scores'
        self.colourNegativeScores.sort()
        self.colourNegativeScores.reverse()
        print self.colourNegativeScores[0:5]
        print
        m = numpy.mean(self.negativeScores[0:5])
        s = numpy.std(self.negativeScores[0:5])
        pm = numpy.mean(self.matchScores[0:2])
        siftProb = 0.5 * ( 1 + scipy.special.erf( (pm - m)/(s*sqrt(2)) ) )
        if ( pm < 0.1 ):
            siftProb = 0
        print siftProb

        m = numpy.mean(self.colourNegativeScores[0:5])
        s = numpy.std(self.colourNegativeScores[0:5])
        pm = numpy.mean(self.colourMatchScores[0:2])
        colourProb = 0.5 * ( 1 + scipy.special.erf( (pm - m)/(s*sqrt(2)) ) )
        if ( pm < 0.1 ):
            colourProb = 0
        print colourProb

        #self.rightPane.SetScores(self.matchScores)
        self.GetStatusBar().SetStatusText("Detection prob: %f " % ((siftProb+colourProb)*0.5),2)

    def OnClassConsistancy(self, e):
        print
        self.weightMatrix = []
        for i in range(0,len(self.pyramids)):
            self.weightMatrix.append([])
            for j in range(0,len(self.pyramids)):
                self.weightMatrix[i].append( pypyramid_ext.match_score(self.pyramids[i], self.pyramids[j], 3))

        for i in xrange(len(self.pyramids)):
            for j in xrange(len(self.pyramids)):
                print "%.2f " % self.weightMatrix[i][j],
            print

    def OnColourConsistancy(self, e):
        print
        self.weightMatrix = []
        for i in range(0,len(self.colourPyramids)):
            self.weightMatrix.append([])
            for j in range(0,len(self.colourPyramids)):
                self.weightMatrix[i].append( pypyramid_ext.match_score(self.colourPyramids[i], self.colourPyramids[j], 3))

        for i in xrange(len(self.colourPyramids)):
            for j in xrange(len(self.colourPyramids)):
                print "%.2f " % self.weightMatrix[i][j],
            print

    def OnSpecifyClassDir(self, e):
        dlg = wx.DirDialog(self, "Choose a class directory", "/cs/SCRATCH/sanchom/SRVC_local")
        if dlg.ShowModal() == wx.ID_OK:
            self.classDir = dlg.GetPath()
            self.GetStatusBar().SetStatusText("Exploring using: %s" % self.classDir, 0)
            selectionPath = os.path.join(self.classDir, 'features-svm.selection')
            self.selectionSet = pypyramid_ext.get_codeword_selection(selectionPath.encode('utf-8'), self.selectionSize)
            self.trainingImageList = glob.glob(os.path.join(self.classDir, 'trainingimages/*.jpg'))
            self.trainingImageList.sort()
            self.trainingPyramidList = glob.glob(os.path.join(self.classDir, 'trainingpyramids/*.pyramid'))
            self.trainingPyramidList.sort()
            self.colourPyramidList = glob.glob(os.path.join(self.classDir, 'colourpyramids/*.cpyramid'))
            self.colourPyramidList.sort()
            self.negativePyramidList = glob.glob(os.path.join(self.classDir, 'negativepyramids/*.pyramid'))
            self.negativePyramidList.sort()
            self.negativeCPyramidList = glob.glob(os.path.join(self.classDir, 'negativecolourpyramids/*.cpyramid'))
            self.negativeCPyramidList.sort()
            self.colourPyramids = []
            for pyramidPath in self.colourPyramidList:
                self.colourPyramids.append(pypyramid_ext.load_pyramid(pyramidPath.encode('utf-8')))
            self.pyramids = []
            for pyramidPath in self.trainingPyramidList:
                self.pyramids.append(pypyramid_ext.load_pyramid(pyramidPath.encode('utf-8')))
            self.negativePyramids = []
            for pyramidPath in self.negativePyramidList:
                self.negativePyramids.append(pypyramid_ext.load_pyramid(pyramidPath.encode('utf-8')))
            self.negativeCPyramids = []
            for pyramidPath in self.negativeCPyramidList:
                self.negativeCPyramids.append(pypyramid_ext.load_pyramid(pyramidPath.encode('utf-8')))

            self.rightPane.SetImageList(self.trainingImageList)
        dlg.Destroy()

    def OnSelectTestData(self, e):
        dlg = wx.DirDialog(self, "Locate the test data directory", "/cs/SCRATCH/sanchom/SRVC_local")
        if dlg.ShowModal() == wx.ID_OK:
            self.codewordsDir = dlg.GetPath()
        dlg.Destroy()

    def OnOpen(self,e):
        """ Open a file"""
        dlg = wx.FileDialog(self, "Choose a file", self.dirname, "", "(*.JPG)|*.JPG|(*.jpg)|*.jpg", wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.filename=dlg.GetFilename()
            self.dirname=dlg.GetDirectory()
            self.imagePath = os.path.join(self.dirname,self.filename)
            self.codewordsPath = os.path.join(self.codewordsDir, os.path.splitext(os.path.basename(self.filename))[0] + '.codewords')
            self.imagePane.SetBitmap(self.imagePath)
            self.imagePane.SetCodewordPath(self.codewordsPath)
            self.imagePane.SetSelectionSet(self.selectionSet)
            self.splitter.SplitVertically(self.imagePane, self.rightPane)
            self.SetSizeWH(self.imagePane.image_size()[0] + 100, self.imagePane.image_size()[1] + self.GetMenuBar().GetSize()[1] + self.GetStatusBar().GetSize()[1])
            self.splitter.SetSashPosition(self.imagePane.image_size()[0])
            self.GetStatusBar().SetStatusText("Exploring %s (%s)" % (self.imagePath, self.codewordsPath),1)
        dlg.Destroy()

def main():
    parser = OptionParser(usage='usage: %prog [options] selectionSize')
    parser.add_option("-d", "--image-directory", dest="dirname", default="")
    (options, args) = parser.parse_args()

    if ( len(args) == 1 ):
        selectionSize = int(args[0])
    else:
        parser.error('Missing mandatory selectionSize')
    
    app = wx.PySimpleApp()
    frame = MainWindow(None, wx.ID_ANY, 'Match Viewer', options.dirname, selectionSize)
    app.MainLoop()

if ( __name__ == "__main__" ):
    main()
