function feature_selection(featureSelectionTrainingFile,featureSelectionOutput)
% Selects codewords that best discriminate between positive and negative
% examples
%
% FEATURE_SELECTION(FEATURE_COUNTS, FEATURE_SELECTION_BASENAME)
%    FEATURE_COUNTS is a file containing histograms of feature occurance
%    for each training point
%    FEATURE_SELECTION_BASENAME is the basename of the output. Two files
%    will be created FEATURE_SELECTION_BASENAME-svm.selection,
%    and FEATURE_SELECTION_BASENAME-odds.selection

addpath('../../lib/stprtool/svm/');
addpath('../../lib/stprtool/visual/');
addpath('../../lib/stprtool/data/');
addpath('../../lib/stprtool/kernels/');

svmOutputFile = strcat(featureSelectionOutput, '-svm.selection');
oddsRatioOutputFile = strcat(featureSelectionOutput, '-odds.selection');
randomOutputFile = strcat(featureSelectionOutput, '-random.selection');

% Train a linear SVM on the feature histograms
trainingData = load_feature_selection_training(featureSelectionTrainingFile)

trainingData.X = trainingData.X + rand(size(trainingData.X)); % add noise to avoid nans

originalX = trainingData.X;
m = mean(trainingData.X, 2);
s = std(trainingData.X, 0, 2);
centredData = trainingData.X - repmat(m, [1 size(trainingData.X,2)]);
spheredData = centredData ./ repmat(s, [1 size(trainingData.X,2)]);
trainingData.X = spheredData;
model = svmlight(trainingData, struct('ker','linear','C',0))

% Write the weights to file
fid = fopen(svmOutputFile, 'w');
fwrite(fid, trainingData.dim, 'int');
fwrite(fid, abs(model.W), 'float');
fclose(fid);

% Determine the odds ratios for each feature
positiveCounts = sum(originalX(:,trainingData.y==1)');
negativeCounts = sum(originalX(:,trainingData.y==2)');
discriminability = positiveCounts ./ (negativeCounts+0.01);

% Write the weights to file
fid = fopen(oddsRatioOutputFile, 'w');
fwrite(fid, trainingData.dim, 'int');
fwrite(fid, discriminability, 'float');
fclose(fid);

% Write random selection to file
fid = fopen(randomOutputFile, 'w');
fwrite(fid, trainingData.dim, 'int');
fwrite(fid, rand(1, trainingData.dim), 'float');
fclose(fid);

end

function trainingData = load_feature_selection_training(filename)

fid = fopen(filename);
num_data = fread(fid, 1, 'int');
dimensions = fread(fid, 1, 'int');
X = fread(fid, [dimensions, num_data], 'int');
y = fread(fid, num_data, 'int')';
fclose(fid);

trainingData.X = X;
trainingData.name = 'Feature selection data set';
trainingData.y = y;
trainingData.dim = dimensions;
trainingData.num_data = num_data;

end