function activations = non_maximum_suppression(imagefilename, activationdir, threshold, xResolution, yResolution, xSuppression, ySuppression, objectWidth, objectHeight, display_on)
% Performs non-maximum suppression on an activation map
%
% Returns a 3 rows * num_results cols array
% Row 1: x (column)
% Row 2: y (row)
% Row 3: activation value

addpath('../../lib/util/');

if ( nargin < 10 )
    display_on = false;
end

[pathstr, name, ext, vers] = fileparts(imagefilename);
activationfilename = [activationdir name '.activations'];
%activationfilename = change_extension(imagefilename, '.activations');

if ( display_on )
    img = imread(imagefilename);
    imshow(img);
end
    
activationMap = load(activationfilename);
activationMap = activationMap - threshold;
activationMap = ((activationMap > 0) .* activationMap);
[rows cols] = size(activationMap);
linearActivationMap = reshape(activationMap, [1, rows*cols]);
active = linearActivationMap > 0;
activations = zeros(3,1);

activationId = 2;
while ( sum(sum(active)) ~= 0 )
    [maxValue, linearIndex] = max(linearActivationMap .* active);
    if ( maxInNeighborhood(activationMap, maxValue, linearIndex, xResolution, yResolution, xSuppression, ySuppression) )
        [r c] = ind2sub([rows cols], linearIndex);
        x = (c-1)*xResolution;
        y = (r-1)*yResolution;
        if ( display_on )
            rectangle('Position',[x y objectWidth objectHeight],'EdgeColor','y');
        end
        activations(1:3,activationId) = [x;y;maxValue];
        activationId = activationId + 1;
        

        
        % suppress neighborhood
        halfWidth = floor((xSuppression / 2 / xResolution));
        halfHeight = floor((ySuppression / 2 / yResolution));
        
        active = reshape(active, [rows cols]);
        active(max(1,r-halfHeight):min(rows,r+halfHeight), max(1,c-halfWidth):min(cols,c+halfWidth)) = ...
            zeros(min(rows,r+halfHeight)-max(1,r-halfHeight)+1, min(cols,c+halfWidth)-max(1,c-halfWidth)+1);
        active = reshape(active, [1, rows*cols]);
    end
    
    % suppress maximum
    active(linearIndex) = 0;

end

end

function result = maxInNeighborhood(activationMap, maxValue, linearIndex, xResolution, yResolution, xSuppression, ySuppression)

[rows cols] = size(activationMap);
[r c] = ind2sub([rows cols], linearIndex);

halfWidth = floor((xSuppression / 2 / xResolution));
halfHeight = floor((ySuppression / 2 / yResolution));

if ( maxValue == max(max(activationMap(max(1,r-halfHeight):min(rows,r+halfHeight), max(1,c-halfWidth):min(cols,c+halfWidth)))) )
    result = true;
else
    result = false;
end

end
