import ConfigParser, sys, os, socket, threading, pickle, time, numpy, struct, xml.dom.minidom, subprocess, Queue, linecache, shutil
sys.path.append('../../lib/sift')
import sift_ext
import pylab
from PIL import Image

config = ConfigParser.ConfigParser()
config.readfp(open('config-deployed.ini'))

pyflannDir = os.path.normpath(config.get("Directories","flannDir"))
sys.path.append(pyflannDir)
import pyflann, sjm_util

workingDir = os.path.normpath(config.get("Directories","workingDir"))
priorityFile = os.path.join(workingDir,'priorities')
classDirTemplate = os.path.join(workingDir,'%s')
resultDir = os.path.normpath(config.get("Directories",'resultDir'))
testDataDir = os.path.join(workingDir,'testdata')

positivePyramidDirTemplate = os.path.join(workingDir,'%s/trainingpyramids')
negativePyramidDirTemplate = os.path.join(workingDir,'%s/negativepyramids')

positiveColourPyramidDirTemplate = os.path.join(workingDir, '%s/colourpyramids')
negativeColourPyramidDirTemplate = os.path.join(workingDir, '%s/negativecolourpyramids')

pipelineDir = os.path.normpath(config.get("Directories","pipelineDir"))

selectedClusters = int(config.get("Options","selectedClusters"))
pyramidLevels = int(config.get("Options","pyramidLevels"))
xres=int(config.get("Options","xres"))
yres=int(config.get("Options","yres"))

codebook = os.path.join(workingDir, 'srvc.codebook')

datafile = os.path.normpath(config.get("Files","datafile"))
doc = xml.dom.minidom.parse(datafile)

clsList = []

maxActivation = sjm_util.Activation()

def main():
    if ( len(sys.argv) != 3 ):
        print 'Usage: %s class_id image_path' % sys.argv[0]
        exit(1)

    index = int(sys.argv[1])
    imagePath = sys.argv[2]
    for node in doc.getElementsByTagName("class"):
        cls = node.getAttribute("name")
        clsList.append(cls)
    
    cls = clsList[index]
    print "Detector for class: %s" % cls

    if ( not os.path.exists(testDataDir) ):
        os.makedirs(testDataDir)

    print 'Loading codebook...',
    sys.stdout.flush()
    codebookFile = open(codebook,'r')
    centroids = pickle.load(codebookFile)
    codebookFile.close()
    flann = pyflann.FLANN()
    flann.build_index(centroids,checks=128,trees=10)
    print 'done'
    
    siftParams = sift_ext.SiftParameters()
    siftParams.noNormalize = 0.5
    siftParams.discard = True
    siftParams.octaveSteps = 2
    siftParams.percentage = 1
    siftExtractor = sift_ext.SiftExtractor(siftParams)

    codewordPath = os.path.join(testDataDir, os.path.splitext(os.path.basename(imagePath))[0] + '.codewords')
    if ( not os.path.exists(codewordPath ) ):
        sjm_util.image_to_codewords(siftExtractor, flann, imagePath, testDataDir)
    else:
        print 'Using pre-existing codeword extraction at %s' % codewordPath

    if ( not os.path.exists(resultDir) ):
        os.makedirs(resultDir)
    resultFile = os.path.join(resultDir, str(index) + '.result')

    negativePyramidDir = negativePyramidDirTemplate % cls
    positivePyramidDir = positivePyramidDirTemplate % cls
    negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls
    positiveColourPyramidDir = positiveColourPyramidDirTemplate % cls
    classDir = classDirTemplate % cls
    arLine = linecache.getline(os.path.join(classDir, 'ar.txt'), 1)
    arSplit = arLine.split()
    windowWidth = int(float(arSplit[0]))
    windowHeight = int(float(arSplit[1]))
    colourWeight = float(linecache.getline(os.path.join(classDir,'colourWeight.txt'),1))

    print 'Doing detection on %s' % imagePath
    activationDir = '.'
    activationsFile = os.path.join(activationDir, os.path.splitext(os.path.basename(imagePath))[0] + '.activations')

    print "Sliding window:"
    p = subprocess.Popen([pipelineDir+'/LocateObjects','-I',imagePath,'-D',testDataDir,'-A','.','-N',negativePyramidDir,'-P',positivePyramidDir,'-R','-L',str(pyramidLevels),'-M',classDir+'/svm.model','--colour-svm-model',classDir+'/colour-svm.model','-S',classDir+'/features-svm.selection','-W',str(windowWidth),'-H',str(windowHeight),'--multi-scale','-n',str(selectedClusters),'--positive-colour',positiveColourPyramidDir,'--negative-colour',negativeColourPyramidDir,'--colour-weight',str(colourWeight),'-X',str(xres),'-Y',str(yres)], stdout=sys.stdout)
    p.wait()
    a = sjm_util.max_activation(activationsFile)
    print "Max activation: " + str(a)
    im = Image.open(imagePath)
    pylab.imshow(im, origin='lower')
    if ( a.x != None ):
        f = open(resultFile, 'w')
        f.write(str(0) + ' ' + str(a.value) + ' ' + str(a.x) + ' ' + str(a.y) + ' ' + str(a.width) + ' ' + str(a.height) + ' ' + cls.replace(' ', '_'))
        f.close()
        x_coords = [a.x, a.x + a.width, a.x + a.width, a.x, a.x]
        y_coords = [im.size[1] - a.y, im.size[1] - a.y, im.size[1] - (a.y + a.height), im.size[1] - (a.y + a.height), im.size[1] - a.y]
        averagePixel = sjm_util.average_intensity(im, a.x, a.y, a.x + a.width, a.y + a.height)
        inversePixel = [0,0,0]
        inversePixel[0] = (averagePixel[0] + 128) % 256
        inversePixel[1] = (averagePixel[1] + 128) % 256
        inversePixel[2] = (averagePixel[2] + 128) % 256
        pylab.fill(x_coords,y_coords,edgecolor=(inversePixel[0]/255.0,inversePixel[1]/255.0,inversePixel[2]/255.0),fill=False)
    pylab.axis('image')
    pylab.axis('off')
    shutil.copyfile(imagePath, os.path.join(resultDir, str(index) + '.jpg') )
    pylab.show()
    
    os.remove(activationsFile)
    
if ( __name__ == "__main__" ):
    main()
