import ConfigParser, sys, os, socket, threading, pickle, time, numpy, struct, xml.dom.minidom, subprocess, Queue, linecache, shutil
sys.path.append('../../lib/sift')
import sift_ext

config = ConfigParser.ConfigParser()
config.readfp(open('config-deployed.ini'))

pyflannDir = os.path.normpath(config.get("Directories","flannDir"))
sys.path.append(pyflannDir)
import pyflann, sjm_util

myname=socket.gethostname()
roleIndex=int(config.get("Roles",myname))
detectionPortBase=int(config.get("Ports","detectionServerPortBase"))

workingDir = os.path.normpath(config.get("Directories","workingDir"))
priorityFile = os.path.join(workingDir,'priorities')
classDirTemplate = os.path.join(workingDir,'%s')
resultDir = os.path.normpath(config.get("Directories", "resultDir"))
testDataDir = os.path.join(workingDir,'testdata')

positivePyramidDirTemplate = os.path.join(workingDir,'%s/trainingpyramids')
negativePyramidDirTemplate = os.path.join(workingDir,'%s/negativepyramids')

positiveColourPyramidDirTemplate = os.path.join(workingDir, '%s/colourpyramids')
negativeColourPyramidDirTemplate = os.path.join(workingDir, '%s/negativecolourpyramids')

activationDirTemplate = os.path.join(workingDir, '%s/testactivations')

pipelineDir = os.path.normpath(config.get("Directories","pipelineDir"))

selectedClusters = int(config.get("Options","selectedClusters"))
pyramidLevels = int(config.get("Options","pyramidLevels"))
xres=int(config.get("Options","xres"))
yres=int(config.get("Options","yres"))

datafile = os.path.normpath(config.get("Files","datafile"))
doc = xml.dom.minidom.parse(datafile)

clsList = []

queue = Queue.Queue(0)
maxActivation = sjm_util.Activation()

def detect_from_queue(activationDir,negativePyramidDir,positivePyramidDir,classDir,positiveColourPyramidDir,negativeColourPyramidDir):
    global clsIndex
    global maxActivation
    global clsList
    global threadDone
    cls = clsList[clsIndex]
    resultFile = os.path.join(resultDir, str(clsIndex) + '.result')
    while( not threadDone ):
        imagePath = queue.get()
        print 'Doing detection on %s' % imagePath
        activationsFile = os.path.join(activationDir, os.path.splitext(os.path.basename(imagePath))[0] + '.activations')
        
        print "Sliding window:"
        p = subprocess.Popen([pipelineDir+'/LocateObjects','-I',imagePath,'-D',testDataDir,'-A',activationDir,'-N',negativePyramidDir,'-P',positivePyramidDir,'-R','-L',str(pyramidLevels),'-M',classDir+'/svm.model','--colour-svm-model',classDir+'/colour-svm.model','-S',classDir+'/features-svm.selection','-W',str(windowWidth),'-H',str(windowHeight),'--multi-scale','-n',str(selectedClusters),'--positive-colour',positiveColourPyramidDir,'--negative-colour',negativeColourPyramidDir,'--colour-weight',str(colourWeight),'-X',str(xres),'-Y',str(yres)], stdout=sys.stdout)
        p.wait()
        a = sjm_util.max_activation(activationsFile)
        print "Max activation: " + str(a)
        if ( a.value > maxActivation.value ):
            maxActivation = a
            shutil.copyfile(imagePath, os.path.join(resultDir, str(clsIndex) + '.jpg'))
            f = open(resultFile, 'w')

            if (a.value > 2.0):
                confidence = 1
            else:
                confidence = 0
            f.write(str(confidence) + ' ' + str(a.value) + ' ' + str(a.x) + ' ' + str(a.y) + ' ' + str(a.width) + ' ' + str(a.height) + ' ' + cls.replace(' ', '_') + '\n')
            f.close()


def main():
    global resultDir
    global clsIndex
    global threadDone
    threadDone = False
    if ( len(sys.argv) != 2 ):
        print 'Usage: %s [0-9]' % sys.argv[0]
        exit(1)

    index = int(sys.argv[1])
    for node in doc.getElementsByTagName("class"):
        cls = node.getAttribute("name")
        clsList.append(cls)
    
    pFile = open(priorityFile)
    numClasses = 0
    while (True):
        line = pFile.readline()
        if ( line != '' ):
            numClasses += 1
        else:
            break
    pFile.close()

    priorityIndex = roleIndex+2*index
    if ( priorityIndex >= numClasses ):
        print 'Attempt to launch a detector for a greater index than is available. This detector is not needed and is exiting.'
        exit(1)
    clsIndex = linecache.getline(priorityFile, priorityIndex+1)
    clsIndex = int(clsIndex.strip())
    cls = clsList[clsIndex]
    classDir = classDirTemplate % cls
    print "Detector for class: %s" % cls
    
    global windowWidth
    global windowHeight
    global colourWeight
    arLine = linecache.getline(os.path.join(classDir, 'ar.txt'), 1)
    arSplit = arLine.split()
    windowWidth = int(float(arSplit[0]))
    windowHeight = int(float(arSplit[1]))
    colourWeight = float(linecache.getline(os.path.join(classDir,'colourWeight.txt'),1))

    if ( not os.path.exists(resultDir) ):
        try:
            os.makedirs(resultDir)
        except OSError:
            pass

    activationDir = activationDirTemplate % cls
    if ( not os.path.exists(activationDir) ):
        os.makedirs(activationDir)
    if ( not os.path.exists(testDataDir) ):
        os.makedirs(testDataDir)

    global resultFile
    resultFile = os.path.join(resultDir, cls.replace(' ','_') + '.result')
    negativePyramidDir = negativePyramidDirTemplate % cls
    positivePyramidDir = positivePyramidDirTemplate % cls
    negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls
    positiveColourPyramidDir = positiveColourPyramidDirTemplate % cls

    HOST=''
    PORT=detectionPortBase+index
    print 'Listening on port %d' % PORT

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind((HOST,PORT))

    t = threading.Thread(target=detect_from_queue,args=(activationDir, negativePyramidDir, positivePyramidDir, classDir, positiveColourPyramidDir, negativeColourPyramidDir))
    t.start()
    try:
        while(True):
            s.listen(1)
            conn, addr = s.accept()
            data = conn.recv(1024)
            if ( data.startswith('DETECT') ):
                message = data.split()
                imagePath = message[1]
                queue.put(imagePath)
            if ( data.startswith('STOP') ):
                print 'Received STOP signal: exiting...'
                threadDone = True
                exit(0)
    except KeyboardInterrupt:
        exit(1)
    finally:
        s.close()
                
    
if ( __name__ == "__main__" ):
    main()
