import ConfigParser, sys, os, socket, threading, pickle, time, numpy, struct
sys.path.append('../../lib/sift')
import sift_ext

config = ConfigParser.ConfigParser()
config.readfp(open('config-deployed.ini'))

pyflannDir = os.path.normpath(config.get("Directories","flannDir"))
sys.path.append(pyflannDir)
import pyflann, sjm_util

myPort = int(config.get("Ports","extractionServerPort"))
detectorPort = int(config.get("Ports","detectionServerPortBase"))

workingDir = os.path.normpath(config.get("Directories","workingDir"))
priorityPath = os.path.join(workingDir,'priorities')
priorityFile = open(priorityPath)
numClasses = 0
while (True):
    line = priorityFile.readline()
    if ( line != '' ):
        numClasses += 1
    else:
        break
priorityFile.close()

detectors = []
classCount = 0
for i in xrange(10):
    if ( classCount < numClasses ):
        detectors.append( ('trex',detectorPort + i) )
        classCount += 1
    if ( classCount < numClasses):
       detectors.append( ('raptor',detectorPort + i) )
       classCount += 1
print detectors

def notifyDetectors(imagepath):
    for d in detectors:
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(d)
            s.send('DETECT ' + imagepath)
            s.close()
        except socket.error, msg:
            print msg

def do_extraction(siftExtractor, flann, image, datadir):
    print 'Doing extraction on %s' % image
    sjm_util.image_to_codewords(siftExtractor, flann, image, datadir)
    notifyDetectors(image)

def main():
    testDataDir = os.path.join(workingDir, "testdata")
    if ( not os.path.exists(testDataDir) ):
        try:
            os.makedirs(testDataDir)
        except OSError:
            pass
    codebook = os.path.join(workingDir, "srvc.codebook")

    print 'Loading codebook...',
    sys.stdout.flush()
    codebookFile = open(codebook,'r')
    centroids = pickle.load(codebookFile)
    codebookFile.close()
    flann = pyflann.FLANN()
    flann.build_index(centroids,checks=128,trees=10)
    print 'done'

    siftParams = sift_ext.SiftParameters()
    siftParams.noNormalize = 0.5
    siftParams.discard = True
    siftParams.octaveSteps = 2
    siftParams.percentage = 1
    siftExtractor = sift_ext.SiftExtractor(siftParams)

    HOST=''
    PORT=myPort
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind((HOST,PORT))

    t = None
    try:
        while(True):
            s.listen(1)
            conn, addr = s.accept()
            data = conn.recv(1024)
            if ( data.startswith('EXTRACT') ):
                if ( t == None or not t.isAlive() ):
                    returnMessage = 'OK'
                    message = data.split()
                    imagePath = message[1]
                    t = threading.Thread(target=do_extraction,args=(siftExtractor,flann,imagePath,testDataDir))
                    t.start()
                else:
                    returnMessage = 'NO'
                print 'Returning: %s' % returnMessage
                conn.send(returnMessage)
        
    finally:
        s.close()

if ( __name__ == '__main__' ):
    main()
