import sys, os, glob, linecache

sys.path.append('../../../flann/devel/src/bindings/python')
from optparse import OptionParser
from PIL import Image
import pylab
import sjm_util

def find(list, x):
    try:
        i = list.index(x)
        return i
    except ValueError:
        return -1

def decision(scottDir, sanchoDir, i):
    scottPath = os.path.join(scottDir, str(i) + '.result')
    scottLine = linecache.getline(scottPath, 1)
    data = scottLine.split()
    scottConfidence = int(data[0])
    scottValue = float(data[1])

    sanchoPath = os.path.join(sanchoDir, str(i) + '.result')
    sanchoLine = linecache.getline(sanchoPath, 1)
    data = sanchoLine.split()
    sanchoConfidence = int(data[0])
    sanchoValue = float(data[1])

    toUse = 'scott'
    if (scottConfidence == 0):
        toUse = 'sancho'
    elif (scottConfidence < sanchoConfidence):
        toUse = 'sancho'

    return toUse

def drawResult(baseDir, index, destinationDir):
    dataPath = os.path.join(baseDir, str(index) + '.result')
    imagePath = os.path.join(baseDir, str(index) + '.jpg')

    dataLine = linecache.getline(dataPath, 1)
    data = dataLine.split()
    tlx = int(float(data[2]))
    tly = int(float(data[3]))
    width = int(float(data[4]))
    height = int(float(data[5]))
    classname = data[6]
    resultPath = os.path.join(destinationDir, classname + '.jpeg')
    print 'Drawing rectangle from %s on %s and saving to %s' % (dataPath, imagePath, resultPath)

    im = Image.open(imagePath)    
    pylab.figure()
    pylab.imshow(im, origin='lower')

    if ( tlx < 0 ):
        width += tlx
        tlx = 0
    if ( tly < 0 ):
        height += tly
        tly = 0
    if ( tlx + width >= im.size[0] ):
        width -= ((tlx + width) - im.size[0] + 1)
    if ( tly + height >= im.size[1] ):
        height -= ((tly + height) - im.size[1] + 1)
    
    averagePixel = sjm_util.average_intensity(im, tlx, tly, tlx + width, tly + height)
    x_coords = [tlx, tlx + width, tlx + width, tlx, tlx]
    y_coords = [im.size[1] - tly, im.size[1] - tly, im.size[1] - (tly + height), im.size[1] - (tly + height), im.size[1] - tly]
    inversePixel = [0,0,0]
    inversePixel[0] = (averagePixel[0] + 128) % 256
    inversePixel[1] = (averagePixel[1] + 128) % 256
    inversePixel[2] = (averagePixel[2] + 128) % 256
    pylab.fill(x_coords,y_coords,edgecolor=(inversePixel[0]/255.0,inversePixel[1]/255.0,inversePixel[2]/255.0),fill=False)

    pylab.axis('image')
    pylab.axis('off')
    pylab.savefig(resultPath)

def main():
    parser = OptionParser(usage='usage: %prog scottResults sanchoResults finalResults')
    (options, args) = parser.parse_args()

    if ( len(args) == 3 ):
        scottDir = args[0]
        sanchoDir = args[1]
        resultDir = args[2]
    else:
        parser.error('Missing result directories arguments')

    if ( not os.path.exists(resultDir) ):
        os.makedirs(resultDir)

    scottResults = []
    for r in glob.glob(os.path.join(scottDir,'*.result')):
        scottResults.append( int( os.path.splitext(os.path.basename(r))[0] ) )
    sanchoResults = []
    for r in glob.glob(os.path.join(sanchoDir,'*.result')):
        sanchoResults.append( int( os.path.splitext(os.path.basename(r))[0] ) )
    scottResults.sort()
    sanchoResults.sort()

    for i in scottResults:
        if ( find(sanchoResults,i) == -1 ):
            print 'Using scott result for %d' % i
            drawResult(scottDir, i, resultDir)
        else:
            print 'Deciding between scott and sancho results for %d' % i
            if ( decision( scottDir, sanchoDir, i ) == 'scott' ):
                print 'Chose scott'
                drawResult(scottDir, i, resultDir)
            else:
                print 'Chose sancho'
                drawResult(sanchoDir, i, resultDir)
    
    for i in sanchoResults:
        if ( find(scottResults,i) == -1 ):
            print 'Using sancho result for %d' % i
            drawResult(sanchoDir, i, resultDir)

if ( __name__ == "__main__"):
    main()
