#include <boost/python.hpp>
#include <string>
#include <set>
#include <vector>
#include <boost/filesystem.hpp>
#include "histograms/integral_histogram.hpp"
#include "spatialpyramid/spatial_pyramid.hpp"
#include "util/util.hpp"

using namespace boost::python;
namespace fs = boost::filesystem;
namespace sp = sjm::SpatialPyramid;

sp::SpatialPyramidType loadPyramid(std::string pyramidPathname)
{
  fs::ifstream pyramidFile(pyramidPathname);
  sp::SpatialPyramidType pyramid;
  sp::load(pyramid, pyramidFile);
  return pyramid;
}

sp::SpatialPyramidType pyramidFromCodewords( const std::string & codewordPath, const std::set<int> & selectedCodewords, int width, int height, int depth )
{
  sp::CodewordSet codewordSet;
  int numCodewords;
  fs::ifstream codewordsFile(codewordPath);
  codewordsFile.read((char*)&numCodewords, sizeof(numCodewords));
  for ( int i = 0; i < numCodewords; ++i )
    {
      int codeword;
      float x;
      float y;
      float scale;
      codewordsFile.read((char*)&codeword, sizeof(codeword));
      codewordsFile.read((char*)&x, sizeof(x));
      codewordsFile.read((char*)&y, sizeof(y));
      codewordsFile.read((char*)&scale, sizeof(scale));
      if ( selectedCodewords.find(codeword) != selectedCodewords.end() )
	{
	  int id = std::distance(selectedCodewords.begin(), selectedCodewords.find(codeword));
	  codewordSet.insert((sp::CodewordSet::value_type){id, x, y, scale});
	}
    }
  codewordsFile.close();
  
  sp::SpatialPyramidType pyramid = 
    sp::buildSpatialPyramid( width, height,
			     codewordSet, selectedCodewords.size(), depth, false);

  return pyramid;
}

std::set<int> getCodewordSelection(std::string selectionPathname, int numFeatures)
{
  int numCodewords;
  std::vector<float> featureWeights;
  fs::ifstream selectionFile(selectionPathname);
  selectionFile.read((char*)&numCodewords, sizeof(numCodewords));
  for ( int i = 0; i < numCodewords; ++i )
    {
      float weight;
      selectionFile.read((char*)&weight, sizeof(weight));
      featureWeights.push_back(weight);
    }
  
  std::set<int> selectedCodewords;
  sjm::util::selection::chooseTopFeatures(featureWeights, selectedCodewords, numFeatures);

  return selectedCodewords;
}

BOOST_PYTHON_MODULE(pypyramid_ext)
{
  class_<std::set<int> >("CodewordSelection");
  class_<sp::SpatialPyramidType>("Pyramid");
  class_<sjm::histograms::IntegralHistogram>("IntegralHistogram");

  def("get_codeword_selection", getCodewordSelection);
  def("pyramid_from_codewords", pyramidFromCodewords);
  def("load_pyramid", loadPyramid);
  def("match_score", sp::normalizedSpatialPyramidMatchKernel);
  def("colour_pyramid", sp::colourPyramid);
  def("save_pyramid", sp::save_);
  def("get_integral_histogram", sjm::histograms::integralHistogramFromImage);
  def("colour_pyramid_from_histogram", sp::pyramidFromIntegralHistogram);
}
