import ConfigParser, os, os.path, xml.dom.minidom, subprocess, shutil, glob, numpy, struct, random, sys, time, linecache
import pickle
import pypyramid_ext
from PIL import Image
from PIL import ImageOps

from xml.dom.minidom import Node

config = ConfigParser.ConfigParser()
config.readfp(open('config-deployed.ini'))

workingDir = os.path.normpath(config.get("Directories","workingDir"))
classDirTemplate = workingDir + '/%s'

flannDir = os.path.normpath(config.get("Directories","flannDir"))
sys.path.append(flannDir)
import pyflann
import sjm_util


universalImageDir = os.path.normpath(config.get("Directories","universalImageDir"))
universalSiftDir = os.path.normpath(config.get("Directories","universalSiftDir"))
universalCodewordDir = os.path.normpath(config.get("Directories","universalCodewordDir"))
codebook = workingDir + '/srvc.codebook'

trainingImageDirTemplate = os.path.normpath(workingDir+'/%s/trainingimages')
positiveSiftDirTemplate = os.path.normpath(workingDir+'/%s/trainingsift')
positiveCodewordDirTemplate = os.path.normpath(workingDir+'/%s/trainingcodewords')
negativeCodewordDirTemplate = os.path.normpath(workingDir+'/%s/negativecodewords')
positivePyramidDirTemplate = os.path.normpath(workingDir+'/%s/trainingpyramids')
colourPyramidDirTemplate = os.path.normpath(workingDir+'/%s/colourpyramids')
negativePyramidDirTemplate = os.path.normpath(workingDir+'/%s/negativepyramids')
negativeColourPyramidDirTemplate = os.path.normpath(workingDir+'/%s/negativecolourpyramids')

pipelineDir = os.path.normpath(config.get("Directories","pipelineDir"))
matlabDir = os.path.normpath(config.get("Directories","matlabDir"))
displayDir = os.path.normpath(config.get("Directories","displayDir"))

trainingWidth = int(config.get("Options","trainingWidth"))
trainingHeight = int(config.get("Options","trainingHeight"))

clusterDataSize = int(config.get("Options","clusterDataSize"))
numClusters = int(config.get("Options","numClusters"))
selectedClusters = int(config.get("Options","selectedClusters"))
pyramidLevels = int(config.get("Options","pyramidLevels"))
numNegatives = int(config.get("Options","numNegatives"))
xres = int(config.get("Options","xres"))
yres = int(config.get("Options","yres"))
adaptiveNegatives = int(config.get("Options","adaptiveNegatives"))

datafile = os.path.normpath(config.get("Files","datafile"))
waitfile = os.path.normpath(config.get("Files","waitfile"))

print 'Waiting for %s' % waitfile
while ( not os.path.exists(waitfile) ):
    time.sleep(5)
print 'Proceeding with training'

doc = xml.dom.minidom.parse(datafile)

clsList = []
imgList = []

if ( not os.path.exists(universalSiftDir) ):
    os.mkdir(universalSiftDir)

if ( not os.path.exists(universalCodewordDir) ):
    os.mkdir(universalCodewordDir)

if ( not os.path.exists(workingDir) ) :
    os.mkdir(workingDir)

if ( not os.path.exists(codebook) ):
    sjm_util.build_codebook(universalSiftDir, codebook, clusterDataSize, numClusters)

print 'loading codebook'
# load codebook
codebookFile = open(codebook,'r')
centroids = pickle.load(codebookFile)
codebookFile.close()
flann = pyflann.FLANN()
flann.build_index(centroids,algorithm='linear',checks=128,trees=10)
print 'codebook loaded'

print 'Converting universal sift to codewords... ',
sys.stdout.flush()
siftFileList = glob.glob(universalSiftDir + '/*.sift')
for siftFile in siftFileList:
    sjm_util.cache_codewords(siftFile, universalCodewordDir, flann)
    print '.',
    sys.stdout.flush()
print


for node in doc.getElementsByTagName("class"):
    cls = node.getAttribute("name")
    clsList.append(cls)
    if ( not os.path.exists(trainingImageDirTemplate % cls) ):
        os.makedirs(trainingImageDirTemplate % cls)
    imgList.append([])
    imageIndex = 0
    for node2 in node.childNodes:
        if ( node2.nodeType != Node.TEXT_NODE ):
            src = node2.getAttribute("path")
            if ( not node2.getAttribute("source").startswith("walnet") and os.path.exists(src.encode('utf-8')) ):
                ext=os.path.splitext(src)[1]
                dest = os.path.join( (trainingImageDirTemplate % cls), str(imageIndex)+ext )
                imageIndex += 1
                #shutil.copyfile(src, dest)
                imgList[-1].append(dest)
print 'done.'

if ( os.access(displayDir, os.W_OK) ):
    numClassesPath = os.path.join(displayDir, 'num_classes.txt')
    numClassesFile = open(numClassesPath,'w')
    numClassesFile.write('%d\n' % len(clsList))
    numClassesFile.close()
else:
    print 'No access to %s. Not writing num_classes.txt' % displayDir

# for cls in clsList:
#     print 'Resizing %s images...' % cls,
#     sys.stdout.flush()
#     trainingImageDir = trainingImageDirTemplate % cls
#     subprocess.Popen(['mogrify','-resize',str(trainingWidth)+'x'+str(trainingHeight),trainingImageDir+'/*.jpg'])
#     print 'done.'

# clsIndex = 0
# for cls in clsList:
#     positiveSiftDir = positiveSiftDirTemplate % cls
#     if ( not os.path.exists(positiveSiftDir) ) :
#         os.makedirs(positiveSiftDir)
#     print 'Extracting descriptors for %s' % cls,
#     sys.stdout.flush()
#     for i in imgList[clsIndex]:
#         output = subprocess.Popen([pipelineDir+'/ExtractDescriptors','-F','-I',i,'-N','0.5','-p','1','-S','-D','--output-directory',positiveSiftDir], stdout=subprocess.PIPE).communicate()[0]
#         print '.',
#         sys.stdout.flush()
#     print 'done.'
#     clsIndex += 1
        
# for cls in clsList:
#     positiveSiftDir = positiveSiftDirTemplate % cls
#     positiveCodewordDir = positiveCodewordDirTemplate % cls
#     if ( not os.path.exists(positiveCodewordDir) ) :
#         os.makedirs(positiveCodewordDir)
#     print 'Converting %s sift to codewords' % cls,
#     sys.stdout.flush()
#     siftFileList = glob.glob(positiveSiftDir + '/*.sift')
#     for siftFile in siftFileList:
#         sjm_util.cache_codewords(siftFile, positiveCodewordDir, flann)
#         print '.',
#         sys.stdout.flush()
#     print 'done.'

# # Build training file for SVM based codeword selection and rank codewords
# for cls in clsList:
#     print 'Calculating feature weightings for %s' % cls,
#     sys.stdout.flush()
#     positiveCodewordDir = positiveCodewordDirTemplate % cls
#     classDir = classDirTemplate % cls
#     featureCountFile = classDir + '/codeword-counts'
#     output = subprocess.Popen([pipelineDir+'/FeatureWeighting','-R','-P',positiveCodewordDir,'-N',universalCodewordDir,'-o',featureCountFile,'--num',str(numClusters)], stdout=subprocess.PIPE).communicate()[0]
#     print '.',
#     sys.stdout.flush()
#     #print output
#     output = subprocess.Popen(['matlab','-nodesktop','-nojvm','-nosplash','-nodisplay','-r',"cd('" + matlabDir + "'); feature_selection('" + featureCountFile + "','" + classDir + "/features'); quit;"], stdout=subprocess.PIPE).communicate()[0]
#     print '.',
#     #print output
#     sys.stdout.flush()
#     print 'done.'

# #Get some negative patches to turn into negative exemplars for the pyramids
# for cls in clsList:
#     print 'Gathering negative examples for %s...' % cls
#     negativeCodewordDir = negativeCodewordDirTemplate % cls
#     if ( os.path.exists(negativeCodewordDir) ):
#         shutil.rmtree(negativeCodewordDir)
#     if ( os.path.exists(negativePyramidDirTemplate % cls) ):
#         shutil.rmtree(negativePyramidDirTemplate % cls)
#     if ( os.path.exists(negativeColourPyramidDirTemplate % cls) ):
#         shutil.rmtree(negativeColourPyramidDirTemplate % cls)
#     if ( not os.path.exists(negativeCodewordDir) ):
#         os.makedirs(negativeCodewordDir)
#     sjm_util.gather_negative_examples(universalImageDir, universalCodewordDir, negativeCodewordDir, numNegatives, trainingWidth, trainingHeight)

# # build pyramid for positive examples, get pyramids for negative examples
# for cls in clsList:
#     negativeCodewordDir = negativeCodewordDirTemplate % cls
#     positiveCodewordDir = positiveCodewordDirTemplate % cls
#     trainingImageDir = trainingImageDirTemplate % cls
#     colourPyramidDir = colourPyramidDirTemplate % cls
#     negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls

#     classDir = classDirTemplate % cls
#     selectionFile = classDir + '/features-svm.selection'

#     if ( not os.path.exists(positivePyramidDirTemplate % cls) ):
#         os.makedirs(positivePyramidDirTemplate % cls)
#     if ( not os.path.exists(negativePyramidDirTemplate % cls) ):
#         os.makedirs(negativePyramidDirTemplate % cls)
#     if ( not os.path.exists(colourPyramidDir) ):
#         os.makedirs(colourPyramidDir)
#     if ( not os.path.exists(negativeColourPyramidDir) ):
#         os.makedirs(negativeColourPyramidDir)

#     print 'Building colour pyramids for %s...' % cls,
#     sys.stdout.flush()
#     sjm_util.images_to_cpyramids(trainingImageDir, colourPyramidDir)
    
#     negativeCodewordList = glob.glob(os.path.join(negativeCodewordDir, '*.codewords'))
#     for c in negativeCodewordList:
#         print c.encode('utf-8')
#         coords = os.path.splitext(os.path.basename(c))[0]
#         coords2 = coords.split('_')
#         imageId = coords2[0]
#         tlx = int(coords2[1])
#         tly = int(coords2[2])
#         width = int(coords2[3]) - tlx
#         height = int(coords2[4]) - tly
#         imagePath = os.path.join(universalImageDir, imageId + '.jpg')
#         print 'Getting colour pyramid from %s : (%d, %d) (%d x %d)' % (imagePath, tlx, tly, width, height)
#         cPyramid = pypyramid_ext.colour_pyramid(imagePath.encode('utf-8'), tlx, tly, width, height, pyramidLevels)
#         pypyramid_ext.save_pyramid(cPyramid, os.path.join(negativeColourPyramidDir, coords + '.cpyramid').encode('utf-8'))

#     print 'Building spatial pyramids for %s...' % cls,
#     sys.stdout.flush()
#     output = subprocess.Popen([pipelineDir+'/BuildPyramids','-R',trainingImageDir,'-C',positiveCodewordDir,'-S',selectionFile,'-W',str(trainingWidth),'-H',str(trainingHeight),'-N',str(selectedClusters),'-D',str(pyramidLevels),'--output-directory',positivePyramidDirTemplate % cls], stdout=subprocess.PIPE).communicate()[0]
#     #print output
#     print '.',
#     sys.stdout.flush()
#     output = subprocess.Popen([pipelineDir+'/BuildPyramids','-R',negativeCodewordDir,'-G','-S',selectionFile,'-W',str(trainingWidth),'-H',str(trainingHeight),'-N',str(selectedClusters),'-D',str(pyramidLevels),'--output-directory',negativePyramidDirTemplate % cls], stdout=subprocess.PIPE).communicate()[0]
#     print '.',
#     sys.stdout.flush()
#     #print output
#     print 'done'

# # clean up training data
# clsIndex = 0
# for cls in clsList:
#     print 'Image ranking for %s:' % cls
#     trainingImageDir = trainingImageDirTemplate % cls
#     positivePyramidDir = positivePyramidDirTemplate % cls
#     positiveColourDir = colourPyramidDirTemplate % cls

#     positiveImagePaths = glob.glob(os.path.join(trainingImageDir, '*.jpg'))
#     positivePyramidPaths = glob.glob(os.path.join(positivePyramidDir, '*.pyramid'))
#     positiveColourPaths = glob.glob(os.path.join(positiveColourDir, '*.cpyramid'))
#     positiveImagePaths.sort()
#     positivePyramidPaths.sort()
#     positiveColourPaths.sort()

#     pyramids = []
#     for ppath in positivePyramidPaths:
#         pyramids.append(pypyramid_ext.load_pyramid(ppath.encode('utf-8')))

#     imageTotals = []

#     scoreMatrix = []
#     for a in range(0, len(pyramids)):
#         scoreMatrix.append([])
#         total = 0
#         for b in range(0, len(pyramids)):
#             s = pypyramid_ext.match_score(pyramids[a], pyramids[b], pyramidLevels);
#             scoreMatrix[a].append(s)
#             total += s
#         imageTotals.append(total)

#     classDir = classDirTemplate % cls
#     f = open(os.path.join(classDir, 'pair-wise-matches.pickle'), 'w')
#     pickle.dump(scoreMatrix, f)
#     f.close()

#     imageScores = dict.fromkeys(range(0,len(imageTotals)))
#     for t in range(0,len(imageTotals)):
#         imageScores[t] = imageTotals[t]
#     orderedScores = sorted(imageScores.iteritems(), key=lambda (k,v): (v,k), reverse=True)
#     print orderedScores

#     for s in orderedScores[int(len(orderedScores)*0.66):]:
#         print 'Removing %s' % positivePyramidPaths[s[0]] 
#         print 'Removing %s' % positiveImagePaths[s[0]]
#         print 'Removing %s' % positiveColourPaths[s[0]]
#         os.remove(positivePyramidPaths[s[0]])
#         os.remove(positiveImagePaths[s[0]])
#         os.remove(positiveColourPaths[s[0]])
            
#     clsIndex += 1

# for cls in clsList:
#     imageDir = trainingImageDirTemplate % cls
#     ar = sjm_util.average_aspect_ratio(imageDir)
#     classDir = classDirTemplate % cls
#     arPath = os.path.join(classDir, 'ar.txt')
#     arFile = open(arPath, 'w')
#     arFile.write('%f %f\n' % (ar[0], ar[1]))
#     arFile.close()

# # Re-build training file for SVM based codeword selection and rank codewords
# for cls in clsList:
#     print 'Calculating feature weightings for %s' % cls,
#     sys.stdout.flush()
#     positiveCodewordDir = positiveCodewordDirTemplate % cls
#     classDir = classDirTemplate % cls
#     featureCountFile = classDir + '/codeword-counts'
#     output = subprocess.Popen([pipelineDir+'/FeatureWeighting','-R','-P',positiveCodewordDir,'-N',universalCodewordDir,'-o',featureCountFile,'--num',str(numClusters)], stdout=subprocess.PIPE).communicate()[0]
#     print '.',
#     sys.stdout.flush()
#     #print output
#     output = subprocess.Popen(['matlab','-nodesktop','-nojvm','-nosplash','-nodisplay','-r',"cd('" + matlabDir + "'); feature_selection('" + featureCountFile + "','" + classDir + "/features'); quit;"], stdout=subprocess.PIPE).communicate()[0]
#     print '.',
#     #print output
#     sys.stdout.flush()
#     print 'done.'

# # re-build pyramids for positive examples, and pyramids for negative examples
# for cls in clsList:
#     negativeCodewordDir = negativeCodewordDirTemplate % cls
#     positiveCodewordDir = positiveCodewordDirTemplate % cls
#     trainingImageDir = trainingImageDirTemplate % cls
#     colourPyramidDir = colourPyramidDirTemplate % cls
#     negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls

#     classDir = classDirTemplate % cls
#     selectionFile = classDir + '/features-svm.selection'

#     if ( not os.path.exists(positivePyramidDirTemplate % cls) ):
#         os.makedirs(positivePyramidDirTemplate % cls)
#     if ( not os.path.exists(negativePyramidDirTemplate % cls) ):
#         os.makedirs(negativePyramidDirTemplate % cls)
#     if ( not os.path.exists(colourPyramidDir) ):
#         os.makedirs(colourPyramidDir)
#     if ( not os.path.exists(negativeColourPyramidDir) ):
#         os.makedirs(negativeColourPyramidDir)

#     print 'Building colour pyramids for %s...' % cls,
#     sys.stdout.flush()
#     sjm_util.images_to_cpyramids(trainingImageDir, colourPyramidDir)
    
#     negativeCodewordList = glob.glob(os.path.join(negativeCodewordDir, '*.codewords'))
#     for c in negativeCodewordList:
#         print c.encode('utf-8')
#         coords = os.path.splitext(os.path.basename(c))[0]
#         coords2 = coords.split('_')
#         imageId = coords2[0]
#         tlx = int(coords2[1])
#         tly = int(coords2[2])
#         width = int(coords2[3]) - tlx
#         height = int(coords2[4]) - tly
#         imagePath = os.path.join(universalImageDir, imageId + '.jpg')
#         print 'Getting colour pyramid from %s : (%d, %d) (%d x %d)' % (imagePath, tlx, tly, width, height)
#         cPyramid = pypyramid_ext.colour_pyramid(imagePath.encode('utf-8'), tlx, tly, width, height, pyramidLevels)
#         pypyramid_ext.save_pyramid(cPyramid, os.path.join(negativeColourPyramidDir, coords + '.cpyramid').encode('utf-8'))

#     print 'Building spatial pyramids for %s...' % cls,
#     sys.stdout.flush()
#     output = subprocess.Popen([pipelineDir+'/BuildPyramids','-R',trainingImageDir,'-C',positiveCodewordDir,'-S',selectionFile,'-W',str(trainingWidth),'-H',str(trainingHeight),'-N',str(selectedClusters),'-D',str(pyramidLevels),'--output-directory',positivePyramidDirTemplate % cls], stdout=subprocess.PIPE).communicate()[0]
#     #print output
#     print '.',
#     sys.stdout.flush()
#     output = subprocess.Popen([pipelineDir+'/BuildPyramids','-R',negativeCodewordDir,'-G','-S',selectionFile,'-W',str(trainingWidth),'-H',str(trainingHeight),'-N',str(selectedClusters),'-D',str(pyramidLevels),'--output-directory',negativePyramidDirTemplate % cls], stdout=subprocess.PIPE).communicate()[0]
#     print '.',
#     sys.stdout.flush()
#     #print output
#     print 'done'



# #train model
# for cls in clsList:
#     print 'Building model for %s...' % cls,
#     sys.stdout.flush()
#     positivePyramidDir = positivePyramidDirTemplate % cls
#     negativePyramidDir = negativePyramidDirTemplate % cls
#     positiveColourPyramidDir = colourPyramidDirTemplate % cls
#     negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls
#     classDir = classDirTemplate % cls
#     modelFile = os.path.join(classDir, 'svm')
#     colourModelFile = os.path.join(classDir, 'colour-svm')

#     output = subprocess.Popen([pipelineDir+'/BuildModel','-R','-N',negativePyramidDir,'-P',positivePyramidDir,'-L',str(pyramidLevels),'-o',modelFile], stdout=subprocess.PIPE).communicate()[0]
#     print output
#     output = subprocess.Popen([pipelineDir+'/BuildModel','-R','-N',negativeColourPyramidDir,'-P',positiveColourPyramidDir,'-L',str(pyramidLevels),'-o',colourModelFile], stdout=subprocess.PIPE).communicate()[0]
#     print output
    

#rank classes based on how good the training set is

siftPyramidAverages = []
colourPyramidAverages = []
jointAverages = []
for cls in clsList:
    positivePyramidDir = positivePyramidDirTemplate % cls    
    positivePyramidPaths = glob.glob(os.path.join(positivePyramidDir, '*.pyramid'))
    pyramids = []
    for ppath in positivePyramidPaths:
        pyramids.append(pypyramid_ext.load_pyramid(ppath.encode('utf-8')))

    positiveColourPyramidDir = colourPyramidDirTemplate % cls
    positiveColourPaths = glob.glob(os.path.join(positiveColourPyramidDir, '*.cpyramid'))
    cpyramids = []
    for ppath in positiveColourPaths:
        cpyramids.append(pypyramid_ext.load_pyramid(ppath.encode('utf-8')))
        
    total = 0;
    ctotal = 0;
    scoreMatrix = []
    colourMatrix = []

    print len(pyramids)
    print len(cpyramids)
    for a in range(0, len(pyramids)):
        scoreMatrix.append([])
        colourMatrix.append([])
        for b in range(0, len(pyramids)):
            s = pypyramid_ext.match_score(pyramids[a], pyramids[b], pyramidLevels);
            c = pypyramid_ext.match_score(cpyramids[a], cpyramids[b], pyramidLevels);
            scoreMatrix[a].append(s)
            colourMatrix[a].append(c)
            total += s
            ctotal += c

    print 'Sift:'
    for a in xrange(len(pyramids)):
        for b in xrange(len(pyramids)):
            print "%.2f " % scoreMatrix[a][b],
        print

    print
    print 'Colour:'
    for a in xrange(len(pyramids)):
        for b in xrange(len(pyramids)):
            print "%.2f " % colourMatrix[a][b],
        print

    averageSiftMatch = total/(len(pyramids)*len(pyramids))
    averageColourMatch = ctotal/(len(pyramids)*len(pyramids))
    print 'Sift consistancy for %s: %f' % (cls, averageSiftMatch)
    print 'Colour consistancy for %s: %f' % (cls, averageColourMatch)
    siftPyramidAverages.append(averageSiftMatch)
    colourPyramidAverages.append(averageColourMatch)
    colourWeight = averageColourMatch / ( averageColourMatch + averageSiftMatch )
    print 'Colour weight: %f' % colourWeight
    colourWeightPath = os.path.join( classDirTemplate % cls, 'colourWeight.txt')
    colourWeightFile = open(colourWeightPath, 'w')
    colourWeightFile.write('%f\n' % colourWeight)
    colourWeightFile.close()
    jointAverages.append( (averageSiftMatch + averageColourMatch)/2.0 )


# # Find false positives
# for cls in clsList:
#     positivePyramidDir = positivePyramidDirTemplate % cls
#     positiveColourPyramidDir = colourPyramidDirTemplate % cls
#     negativePyramidDir = negativePyramidDirTemplate % cls
#     negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls
#     classDir = classDirTemplate % cls
#     backgroundActivationDir = os.path.join(classDir, 'backgroundActivations')
#     arLine = linecache.getline(os.path.join(classDir, 'ar.txt'), 1)
#     arSplit = arLine.split()
#     windowWidth = int(float(arSplit[0]))
#     windowHeight = int(float(arSplit[1]))
#     colourWeight = float(linecache.getline(os.path.join(classDir,'colourWeight.txt'),1))

#     selectionFile = os.path.join(classDir,'features-svm.selection')
#     negativeCodewordDir = negativeCodewordDirTemplate % cls
#     negativePyramidDir = negativePyramidDirTemplate % cls
    
#     if ( not os.path.exists(backgroundActivationDir) ):
#         os.makedirs(backgroundActivationDir)
    
#     print 'Finding false positives for %s' % cls

#     falsePositives = []
#     backgroundImageList = glob.glob(os.path.join(universalImageDir, '*.jpg'))
#     for bImage in backgroundImageList:
#         activationFile = os.path.join(backgroundActivationDir, os.path.splitext(os.path.basename(bImage))[0] + '.activations')
#         if ( not os.path.exists(activationFile) ):
#             output = subprocess.Popen([pipelineDir+'/LocateObjects','-I',bImage,'-D',universalCodewordDir,'-A',backgroundActivationDir,'-R','-N',negativePyramidDir,'-P',positivePyramidDir,'--positive-colour',positiveColourPyramidDir,'--negative-colour',negativeColourPyramidDir,'--colour-weight',str(colourWeight),'-X',str(xres),'-Y',str(yres),'-W',str(windowWidth),'-H',str(windowHeight),'-n',str(selectedClusters),'-L',str(pyramidLevels),'-M',os.path.join(classDir,'svm.model'),'--colour-svm-model',os.path.join(classDir,'colour-svm.model'),'-S',selectionFile], stdout=subprocess.PIPE).communicate()[0]
        
#         print '.',
#         sys.stdout.flush()

#         a = sjm_util.max_activation(activationFile)
#         if ( a.x != None ):
#             falsePositives.append( ( bImage, a.x, a.y, a.width, a.height ) )
#             if ( len(falsePositives) >= adaptiveNegatives ):
#                 break

#     print 'Found %d false positives for %s' % (len(falsePositives), cls)
#     for fp in falsePositives:
#         bn = os.path.splitext(os.path.basename(fp[0]))[0]
#         tlx = int(fp[1])
#         tly = int(fp[2])
#         width = int(fp[3])
#         height = int(fp[4])
#         originalCodewords = os.path.join(universalCodewordDir, bn + '.codewords')
#         destinationCodewords = os.path.join(negativeCodewordDir, '%s_%d_%d_%d_%d.codewords' % (bn,tlx,tly,tlx+width,tly+height))
#         print originalCodewords + ' -> ' + destinationCodewords
#         sjm_util.subset_codewords(originalCodewords, destinationCodewords, tlx, tly, tlx+width, tly+height, width, height)
#         p = subprocess.Popen([os.path.join(pipelineDir,'BuildPyramids'),destinationCodewords,'-G','-S',selectionFile,'-W',str(width),'-H',str(height),'-N',str(selectedClusters),'-D',str(pyramidLevels),'--output-directory',negativePyramidDir])
#         p.wait()
#         cPyramid = pypyramid_ext.colour_pyramid( bImage.encode('utf-8'), tlx, tly, width, height, pyramidLevels)
#         pypyramid_ext.save_pyramid(cPyramid, os.path.join(negativeColourPyramidDir, '%s_%d_%d_%d_%d.cpyramid' % (bn,tlx,tly,tlx+width,tly+height)).encode('utf-8'))
        

# #re-train model
# for cls in clsList:
#     print 'Re-building model for %s...' % cls,
#     sys.stdout.flush()
#     positivePyramidDir = positivePyramidDirTemplate % cls
#     negativePyramidDir = negativePyramidDirTemplate % cls
#     positiveColourPyramidDir = colourPyramidDirTemplate % cls
#     negativeColourPyramidDir = negativeColourPyramidDirTemplate % cls
#     classDir = classDirTemplate % cls
#     modelFile = os.path.join(classDir, 'svm')
#     colourModelFile = os.path.join(classDir, 'colour-svm')

#     output = subprocess.Popen([pipelineDir+'/BuildModel','-R','-N',negativePyramidDir,'-P',positivePyramidDir,'-L',str(pyramidLevels),'-o',modelFile], stdout=subprocess.PIPE).communicate()[0]
#     print output
#     output = subprocess.Popen([pipelineDir+'/BuildModel','-R','-N',negativeColourPyramidDir,'-P',positiveColourPyramidDir,'-L',str(pyramidLevels),'-o',colourModelFile], stdout=subprocess.PIPE).communicate()[0]
#     print output


# Write priorities pushing specific objects to the back of the list
priorityPath = os.path.join(workingDir, 'priorities')
priorityFile = open(priorityPath,'w')
scoreDict= dict.fromkeys(range(0,len(jointAverages)))
for t in range(0,len(jointAverages)):
    scoreDict[t] = jointAverages[t]
orderedAverages = sorted(scoreDict.iteritems(), key=lambda (k,v): (v,k), reverse=True)

lowestPriority = []
for o in orderedAverages:
    print o
    if ( o[1] > 0.65 ):
        lowestPriority.append(o)
    else:
        priorityFile.write('%d\n' % o[0])

for o in lowestPriority:
    priorityFile.write('%d\n' % o[0])
priorityFile.close()
