import glob, pyflann, numpy, struct, random, pickle, array, os, sys, time
sys.path.append( os.path.normpath(os.path.join(os.getcwd(), '../../lib/sift')) )
import sift_ext
import pypyramid_ext

from math import *

from PIL import Image

import scipy.io.numpyio

class SiftInfo:
    """
    This class holds header info from my sift files.
    """
    def __init__(self, **kwargs):
        self.rotationInvariance = None
        self.noNormalize = None
        self.discard = None
        self.multiscale = None
        self.octavesteps = None
        self.percentage = None
        self.minradius = None
        self.numDescriptors = None
        self.dimensions = None

    def loadFromFile(self, siftFile):
        (self.rotationInvariance,) = struct.unpack('b',siftFile.read(1))
        (self.noNormalize,) = struct.unpack('f',siftFile.read(struct.calcsize('f')))
        (self.discard,) = struct.unpack('b',siftFile.read(1))
        (self.multiscale,) = struct.unpack('b',siftFile.read(1))
        (self.octavesteps,) = struct.unpack('i',siftFile.read(struct.calcsize('i')))
        (self.percentage,) = struct.unpack('f',siftFile.read(struct.calcsize('f')))
        (self.minradius,) = struct.unpack('f',siftFile.read(struct.calcsize('f')))
        (self.numDescriptors,self.dimensions) = struct.unpack('ii',siftFile.read(struct.calcsize('ii')))

class Activation:
	def __init__(self,x=None,y=None,width=None,height=None,value=None):
            self.x = x
            self.y = y
            self.width = width
            self.height = height
            if ( value != None ):
                self.value = value
            else:
                self.value = float("-infinity")
                
        def __str__(self):
            if ( self.x != None ):
                return 'Activation: [ tl=(%f, %f), br=(%f, %f), value=%f ]' % (self.x,self.y,self.x+self.width,self.y+self.height,self.value)
            else:
                return 'Activation: None'

def average_intensity(image, tlx, tly, brx, bry):

    tlx = int(tlx)
    tly = int(tly)
    brx = int(brx)
    bry = int(bry)

    data = list(image.getdata())
    averagePixel = [0,0,0];
    count = 0;
    for x in range(int(tlx), int(brx)):
        thisPixel = data[ tly * image.size[0] + x ]
        averagePixel[0] += thisPixel[0]
        averagePixel[1] += thisPixel[1]
        averagePixel[2] += thisPixel[2]
        count += 1
    for y in range(int(tly), int(bry)):
        thisPixel = data[ y * image.size[0] + brx ]
        averagePixel[0] += thisPixel[0]
        averagePixel[1] += thisPixel[1]
        averagePixel[2] += thisPixel[2]
        count += 1
    for x in range(int(tlx), int(brx)):
        thisPixel = data[ bry * image.size[0] + x ]
        averagePixel[0] += thisPixel[0]
        averagePixel[1] += thisPixel[1]
        averagePixel[2] += thisPixel[2]
        count += 1
    for y in range(int(tly), int(bry)):
        thisPixel = data[ y * image.size[0] + tlx ]
        averagePixel[0] += thisPixel[0]
        averagePixel[1] += thisPixel[1]
        averagePixel[2] += thisPixel[2]
        count += 1
    averagePixel[0] = averagePixel[0] / count
    averagePixel[1] = averagePixel[1] / count
    averagePixel[2] = averagePixel[2] / count
    return averagePixel
    

def image_to_codewords(siftExtractor, flann, image, datadir):
    start = time.time()
    descriptorSet = siftExtractor.getDescriptorSet(image)
    codewordPath = os.path.join(datadir, os.path.splitext(os.path.basename(image))[0] + '.codewords')
    codewordFile = open(codewordPath, 'wb')
    codewordFile.write(struct.pack('i', 0)) # dummy
    
    count = 0
    for d in descriptorSet:
        numpyDescriptor = numpy.array(d[0:128])
        numpyDescriptor = numpyDescriptor.reshape(1,128)
        index = flann.nn_index(numpyDescriptor, checks=128, trees=10)
        codewordFile.write(struct.pack('ifff', index, d[128], d[129], d[130]))
        count += 1
    codewordFile.seek(0)
    codewordFile.write(struct.pack('i',count))
    codewordFile.close()
    end = time.time()
    print '%d codewords written in %d seconds' % (count, end-start)

def max_activation(activationPath):
	if ( not os.path.exists(activationPath) ):
		raise IOError ('File not found: %s' % activationPath)

	maxActivation = Activation()
	f = open(activationPath, 'r')
	dimensions = f.readline().split()
	dimensions[0] = int(dimensions[0])
	dimensions[1] = int(dimensions[1])
	scales = f.readline().split()
	for s in range(0, len(scales)) :
		scales[s] = float(scales[s])
	done = False
	while ( not done ):
		activationLine = f.readline()
		
		if ( activationLine == "" ):
			done = True
		else:
			activation = activationLine.split()
			x = float(activation[0])
			y = float(activation[1])
			scale = float(activation[2])
			value = float(activation[3])
			if ( value > maxActivation.value ):
				maxActivation = Activation(x,y,scale*dimensions[0],scale*dimensions[1],value)
	f.close()
	return maxActivation

def count_descriptors(dirname):
    filelist = glob.glob(dirname+'/*.sift')

    count = 0
    for f in filelist:
        siftFile = open(f, 'rb')
        s = SiftInfo()
        s.loadFromFile(siftFile)
        siftFile.close()
        count += s.numDescriptors

    return count

def images_to_cpyramids(imageDir, pyramidDir):
    imageList = glob.glob( imageDir + '/*.jpg' )
    for im in imageList:
        pyramidPath = os.path.join(pyramidDir, os.path.splitext(os.path.basename(im))[0] + '.cpyramid')
        print '%s -> %s' % (im, pyramidPath)
        image = Image.open(im)
        pyramid = pypyramid_ext.colour_pyramid(im.encode('utf-8'), 0, 0, image.size[0], image.size[1], 3)
        pypyramid_ext.save_pyramid(pyramid, pyramidPath.encode('utf-8'))

def build_codebook(siftDir, codebook, clusterDataSize, numClusters):
    numDescriptors=count_descriptors(siftDir)
    percentageToCluster=min(1, float(clusterDataSize)/numDescriptors)

    pts = numpy.zeros((clusterDataSize,128), float)
    pIndex = 0
    siftList = glob.glob(siftDir+'/*.sift')
    for siftFilename in siftList:
        if ( pIndex < clusterDataSize ) :
            siftFile = open(siftFilename,'rb')
            siftInfo = SiftInfo()
            siftInfo.loadFromFile(siftFile)
            for d in range(1,siftInfo.numDescriptors+1):
                recordElement = random.random() < percentageToCluster
                descriptor = numpy.fromfile(siftFile, numpy.float32, siftInfo.dimensions)
                descriptor = descriptor.reshape(1,siftInfo.dimensions)
                if ( recordElement and pIndex < clusterDataSize ) :
                    pts[pIndex,:] = descriptor[:]
                    pIndex += 1
                (x,y,s) = struct.unpack('fff',siftFile.read(struct.calcsize('fff')))
            print 'Loaded %d/%d descriptors' % (pIndex, clusterDataSize)
            siftFile.close()
    pts = pts[0:pIndex,:]

    print pts

    flann = pyflann.FLANN()
    centroids = flann.hierarchical_kmeans(pts, numClusters, 1, 40)
    print len(centroids)

    codebookFile = open(codebook,'w')
    pickle.dump(centroids, codebookFile)
    codebookFile.close()

    codebookFile2 = open(codebook,'r')
    centroidCheck = pickle.load(codebookFile2)
    codebookFile2.close()
    
    print len(centroidCheck)

def cache_codewords(siftPath, codewordDir, flann):
    noext = os.path.splitext(os.path.basename(siftPath))[0]
    codewordPath = os.path.join(codewordDir, noext + '.codewords')
    if ( not os.path.exists(codewordPath) ) :

        codewordFile = open(codewordPath, 'wb')
        siftFile = open(siftPath, 'rb')
        siftInfo = SiftInfo()
        siftInfo.loadFromFile(siftFile)

        binaryData = struct.pack('i',siftInfo.numDescriptors)
        codewordFile.write(binaryData)
        for d in range(1,siftInfo.numDescriptors+1):
            descriptor = numpy.fromfile(siftFile, numpy.float32, siftInfo.dimensions)
            descriptor = descriptor.reshape(1,siftInfo.dimensions)
            (x,y,s) = struct.unpack('fff',siftFile.read(struct.calcsize('fff')))
            
            index = flann.nn_index(descriptor,checks=512,trees=10)
            binaryData = struct.pack('ifff',index,x,y,s)
            codewordFile.write(binaryData)

        siftFile.close()
        codewordFile.close()

def average_aspect_ratio(imageDir):
    imageList = glob.glob( os.path.join(imageDir, '*.jpg') )
    widthSum = 0.0;
    heightSum = 0.0;
    for imagePath in imageList:
        im = Image.open(imagePath)
        widthSum += im.size[0]
        heightSum += im.size[1]
    return ( widthSum / len(imageList), heightSum / len(imageList) )

def subset_codewords(orig, dest, tlx, tly, brx, bry, orig_width, orig_height):
    origFile = open(orig, 'rb')
    destFile = open(dest, 'wb')

    windowWidth = brx - tlx + 1
    windowHeight = bry - tly + 1

    widthRatio = float(windowWidth) / float(orig_width)
    heightRatio = float(windowHeight) / float(orig_height)
    ratio = min(widthRatio, heightRatio)
    minRadius = ratio * 8
    minScale = sift_ext.scaleFromRadius(minRadius)

    numWritten = 0
    (numWords,) = struct.unpack('i',origFile.read(struct.calcsize('i')))
    destFile.write(struct.pack('i',numWords)) # just a dummy for now, will get overwritten later
    for c in range(0,numWords):
        (codeword,x,y,s) = struct.unpack('ifff',origFile.read(struct.calcsize('ifff')))
        if ( x >= tlx and x <= brx and y >= tly and y <= bry and s >= minScale ) :
            destFile.write(struct.pack('ifff',codeword,x-tlx,y-tly,s))
            numWritten += 1
    origFile.close()
    destFile.seek(0)
    destFile.write(struct.pack('i',numWritten)) # overwriting the incorrect dummy from before
    destFile.close()

def gather_negative_examples(backgroundImageDir, backgroundCodewordDir, destinationCodewordDir, numExamples, width, height):
    print 'Getting %d %dx%d negative examples from %s into %s' % (numExamples,width,height,backgroundCodewordDir,destinationCodewordDir)

    imageList = glob.glob(backgroundImageDir+'/*.jpg')
    examples = 0
    while ( examples < numExamples ):
        for imagePath in imageList:
            if ( examples < numExamples ) :
                img = Image.open(imagePath)
                maxTopLeftX = img.size[0] - width + 1;
                maxTopLeftY = img.size[1] - height + 1;
            
                if ( not (maxTopLeftX < 1 or maxTopLeftY < 1) ):
                    topLeftX = int(ceil(maxTopLeftX * random.random()))
                    topLeftY = int(ceil(maxTopLeftY * random.random()))
                    index = os.path.splitext(os.path.basename(imagePath))[0]
                    originalCodewords = backgroundCodewordDir+'/'+index+'.codewords'
                    destinationCodewords = destinationCodewordDir+'/%s_%d_%d_%d_%d.codewords' % (index,topLeftX,topLeftY,topLeftX+width,topLeftY+height)
                    #print originalCodewords + ' -> ' + destinationCodewords
                    subset_codewords(originalCodewords, destinationCodewords, topLeftX, topLeftY, topLeftX+width, topLeftY+height, width, height)
                    examples += 1
