#pragma once

#include <map>
#include <boost/numeric/ublas/vector.hpp>

namespace sjm
{
  namespace histograms
  {
    /*!
      Class to represent integer indexed histograms in a sparse way.

      Currently, just a wrapper around std::map<unsigned int, T>, but
      with possibility for extension to implement similarity metrics
      like intersection, or chi-square, etc.

      @author Sancho McCann
    */
    template <typename T>
    class SparseHistogram : public std::map<unsigned int, T>
    {
      typedef std::map<unsigned int, T> self_type;
      typedef typename self_type::size_type size_type;
      typedef T value_type;
      typedef const T &const_reference;
      typedef T &reference;
      
    public:
      inline
      SparseHistogram()
      {
	self_type();
      }

      inline
      SparseHistogram & operator=(const boost::numeric::ublas::vector<float> & orig)
      {
	this->clear();
	
	unsigned int index = 0;
	for ( boost::numeric::ublas::vector<float>::const_iterator i = orig.begin();
	      i != orig.end(); ++i )
	  {
	    if ( *i != 0 )
	      {
		(*this)[index] = *i;
	      }
	    ++index;
	  }
	return *this;
      }

    };
  }
}
