import os,time,socket, sys
import threading
from pyinotify import WatchManager, Notifier, ThreadedNotifier, EventsCodes, ProcessEvent, _Event



class EventHandler:
    extractors = [('trex',50200),('raptor',50200)];
    sleep_time = 2

    def __init__(self):
        self.crt_id = 0

    def process_event(self,event):
        cnt = 0
        processed = False
        while not processed:
            processed = self.try_process(self.extractors[self.crt_id],event.pathname)
            cnt += 1
            self.crt_id = (self.crt_id+1) % len(self.extractors)
            if cnt == len(self.extractors):
                time.sleep(self.sleep_time)
                cnt = 0
    
    def try_process(self,server_socket,file_name):
        try:
            print "Sending message about file %s to" % file_name, server_socket
            self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.s.connect(server_socket)
            self.s.send('EXTRACT '+file_name)
            response = self.s.recv(1024)
            print "I received %s" % response
        except socket.error, msg:
            print msg
            return False
        finally:
            self.s.close()
        if response == 'OK':
            return True
        else:
            return False



class EventWatcher(ProcessEvent):
    
    def __init__(self,path,handler):
        self.handler = handler
        wm = WatchManager()
        mask = EventsCodes.ALL_FLAGS['IN_CLOSE_WRITE'] | EventsCodes.ALL_FLAGS['IN_MOVED_TO'] # watched events
        wdd = wm.add_watch(path, mask, rec=True)
        self.notifier = ThreadedNotifier(wm, self)
        
    def process_IN_CLOSE_WRITE(self, event):
        self.handler.process_event(event)

    def process_IN_MOVED_TO(self, event):
        self.handler.process_event(event)

    def start(self):
        self.notifier.start()


class PollEvent(_Event):
    pass


class PollWatcher(threading.Thread):
    sleep_time = 1
    
    def __init__(self, path,handler):
        threading.Thread.__init__(self)
        self.path_to_watch = path
        self.handler = handler
        
    def run(self):
        before = dict ([(f, None) for f in os.listdir (self.path_to_watch)])
        while 1:
            time.sleep (self.sleep_time)
            after = dict ([(f, None) for f in os.listdir (self.path_to_watch)])
            added = [f for f in after if not f in before]
            removed = [f for f in before if not f in after]
            if added:
                for p in added:
                    self.handler.process_event(PollEvent({'name':p,'pathname':self.path_to_watch+'/'+p,'mask':None}))
#            if removed:
#                for p in removed:
#                    self.event_processor.process_removed(p)
            before = after



if __name__ == '__main__':
    if len(sys.argv) != 2:
        print "Usage: %s dir_to_watch" % sys.argv[0]
        exit(1)
   
    watch_dir = os.path.abspath(sys.argv[1])
    print "Starting watcher on directory: ",watch_dir 
    watcher = EventWatcher(watch_dir,EventHandler())
    #watcher = PollWatcher(watch_dir,EventHandler())    # Use this for poling instread of inotify events
    watcher.start()
    while 1:
        try:
            time.sleep(1)
        except:
            exit(1)

