% These are the camera configuration defaults. By default, the RunRobot.m script
% will run this file unless the configFile variable is present in the
% workspace when RunRobot is run. If this variable is set, then
% configuration information will be loaded from the file named by the
% cameraFile variable.
%
% In other words, to create your own configuration environment, just copy
% this file to its own filename, Eg: CameraExperiment.m, then set 
% cameraFile = 'CameraExperiment.m' before running RunRobot.m.
cameraPars.configFile = '../stereoCode/bb_id5020143.config';
cameraPars.canonfl=true;   % Use Canon camera if set, otherwise use Bumblebee
cameraPars.bbfl=0;      % Use bumblebee camera? Normally 0 now since the 
                        % realtime grabbing code is used instead

if cameraPars.canonfl
    cameraPars.MAX_LOOK_DIST = 5;
    cameraPars.gazeCenter = [326;512];   % Offset origin (y,x) for Canon camera
else
    cameraPars.MAX_LOOK_DIST = 3; % Only look if object is within 3m of camera
    cameraPars.gazeCenter = [384;512];   % Image centre (y,x) (for 1024x768 image)
end;

cameraPars.BB_HEIGHT35 = 1.44; % Height of Bumblebee Camera
cameraPars.BB_HEIGHT40 = 1.41; % Height of Bumblebee Camera
cameraPars.BB_HEIGHT45 = 1.17;

if cameraPars.canonfl
    cameraPars.CC_HEIGHT35 = 1.5; % Height of Canon Camera
    cameraPars.CC_HEIGHT45 = 1.22; % Height of Canon Camera
end;

% Available resolutions for Canon camera
cameraPars.res_name=cell(5,1); cameraPars.res=zeros(5,2);
cameraPars.res(1,:)=[3648 2736]; cameraPars.res_name{1}='large';
cameraPars.res(2,:)=[2816 2112]; cameraPars.res_name{2}='medium1';
cameraPars.res(3,:)=[2272 1704]; cameraPars.res_name{3}='medium2';
cameraPars.res(4,:)=[1600 1200]; cameraPars.res_name{4}='medium3';
cameraPars.res(5,:)=[640 480];   cameraPars.res_name{5}='small';
% Scaling from bb to given Canon resolution
cameraPars.bbtocan_scaling=3.4*[1.2955 1 0.8068 0.5682 0.2273];

% Use this resolution on Canon camera
if configPars.usingLaptop,
    cameraPars.canon_res=4; % medium3
else
    cameraPars.canon_res=3; % medium2
end

% Zoom scalings for Canon camera
% Scalings from zoom 0 to other zoom levels [0:6]
cameraPars.zoom_scaling=[1 1.09 1.228 1.3065 1.44 1.764 2.08];

% Define centre marker
cameraPars.xx=[-1 1 0 0 0];
cameraPars.yy=[0 0 0 -1 1];
cameraPars.msz=20;

% Using 16bit or 8bit disparity?
cameraPars.is16BitDisp = 1;

cameraPars.initialTilt= 30;
