% These are the configuration defaults. By default, the RunRobot.m script
% will run this file unless the configFile variable is present in the
% workspace when RunRobot is run. If this variable is set, then
% configuration information will be loaded from the file named by the
% configFile variable.
%
% In other words, to create your own configuration environment, just copy
% this file to its own filename, Eg: ConfigExperiment.m, then set 
% configFile = 'ConfigExperiment.m' before running RunRobot.m.

configPars.robotName = '192.168.0.100';
configPars.robotName = 'bart.ubcrobots.net';
configPars.robotName = 'bart';

configPars.robotName = 'maggie';
%configPars.robotName = '127.0.0.1';

configPars.havePTU = true;
% ICRA experiment setting:
% 1: Coverage only
% 2: Coverage + look back at everything
% 3: Coverage + look back at best object for each class
configPars.experiment = 2;

% RAS Journal paper experiment setting. This flag disables the attention
% system when set.
% 1: Uses visual attention only
% 2: Uses visual attention plus a single non-zoomed frame 
% 3: Uses visual attention, a non-zoomed and samples zoomed frames
configPars.useAttention = 3;
configPars.numImageSamples = 10;
configPars.unZoomedFramePath = '/home/lciuser/testImages/robotImages/unZoomed/';
configPars.NunZoomedPics = 1;
configPars.zoomSamplesPath = '/home/lciuser/testImages/robotImages/zoomSamples/';
configPars.NzoomedPics = 1;
configPars.attentionPoseNums = [];
configPars.usingLaptop=1; % Are we running on the laptop?

% How many zoomed pictures should we take?
configPars.totalZoomedPics = 1;

% Parameters for spectral saliency
configPars.ss.Kmax=7;           % Pick no more than K salient regions
configPars.ss.scale=4;          % Go three octaves down
configPars.ss.overlap=0.5;      % Remove large regions if >50% overlap
configPars.ss.depthoverlap=.15; % Require 15% overlap with objectMask
configPars.ss.minthr=2.0;       % Require min within region to be above mean(intensity) times this

configPars.runTime = 240;
configPars.numWalks = 1;
configPars.stepSize = 10;
configPars.stepTime = 10;
configPars.tableThreshold = 0.5;

% These parameters control the Bumblebee guided random walk
configPars.interestThreshold = 10;
configPars.interestInterval = 20;

if configPars.usingLaptop
    configPars.output_path=getenv('SUPLEMENTAL_IMAGES');%/home/lciuser/testImages/robotImages_unused/';
    configPars.output_path2=getenv('ROBOT_IMAGES');
    configPars.displayDir = getenv('DISPLAY_IMAGES');
    configPars.objectFile = '/home/lciuser/testImages/list.txt';
    configPars.keypointPath='../onlineMatching';
else
    configPars.output_path='~/robotImages/';
    configPars.output_path2='~/robotImages_bbox/';
end;

configPars.bpattern=0;  % For BB-HICOL-40

%TODO SET THIS BACK
configPars.position_offset = 0;%0.20; % position offset between camera and odometry
configPars.tilt_offset = 0;%1; % tilt offset of 1 degree, determined from PTU calibration
%END


configPars.numPrevious=5; % The number of previous objects to look at
configPars.nTotalPics = uint8(0); % Total number of pictures we've taken
objFilePath1 = sprintf('%sobjects.txt',configPars.output_path);
objFilePath2 = sprintf('%sobjects.txt',configPars.output_path2);
configPars.objectFile1 = fopen(objFilePath1, 'wt');
configPars.objectFile2 = fopen(objFilePath2, 'wt');

if configPars.usingLaptop
    configPars.map_path = '/home/lciuser/testImages/maps/';
else
    configPars.map_path = '~/maps/';
end;

% Sensitivity in degrees of the turnbot command
configPars.turnSensitivity = 3;
configPars.DISPLAY = true;

% 1: print only errors
% 2: print errors and warnings
% 3: print errors, warnings, and checkpoints
% 4: print everything, including debug statements
configPars.PRINTLEVEL = 4;

% If this is set to 1, the robot will dbstop when it sees an error.
configPars.DEBUG = true;

% If we want to run the robot with no cameras (exploration only)
configPars.exploreOnly = false;

% How many seconds between display updates?
configPars.displayRefreshPeriod = 0.25;

% How many seconds should we run for? 
% Set this value to -1 to specify no time limit
configPars.timeLimit = 40 * 60;

% For some testing, we want to run other behaviours without moving the
% robot. This flag will disable all robot calls.
configPars.usingRobot = true;

% Should we save planner pictures in ./data?
configPars.savePlanPics = true;

% Should we reuse the same filename, or use incrementing filenames,
% ie: planpic.png vs planpic-22.png
configPars.samePlanPic = true;

% Should we save a plan movie? Note: this feature doesn't work b/c matlab
% can't save avi files properly.
configPars.savePlanMovie = false;
