% Render any important graphics to screen or disk.
if configPars.nextDisplayTime < toc && ~isempty(goal) && configPars.DISPLAY
    
    figure (10);
    configPars.nextDisplayTime = toc + configPars.displayRefreshPeriod;
    switch (goal.type)
        case 'e'
            desc = 'explore';
        case 'o'
            desc = 'observe';
        case 'x'
            desc = 'unknown';
        case 't'
            desc = 'table';
    end;
    title(sprintf('Current Plan: %s', desc));
    hold on;
    
    % Render planner info
    s = max(1,goal.step);
    imagesc(worldState.oGrid);
    scatter(worldState.rGridPos(2), worldState.rGridPos(1), 'b+'); % robot
    faceAt = worldState.rGridPos + 3 * [ cos(worldState.rPose(3)) sin(worldState.rPose(3)) ];
    line([ faceAt(2) worldState.rGridPos(2) ], [ faceAt(1) worldState.rGridPos(1) ] );
    
    if goal.type ~= 'x'
        plot(goal.path(1:s,2), goal.path(1:s,1), '-c.'); % travelled path
        plot(goal.path(s:end,2), goal.path(s:end,1), ':c.'); % untravelled path
        scatter(goal.path(s,2), goal.path(s,1), 'y.');
        if size(mappingPars.objs(:,:),1) > 0
            scatter(mappingPars.objs(:,2), mappingPars.objs(:,1), 'r.');
        end;
    end;
    
    % Render frontiers
    if goal.type == 'e'
        for i = 1 : size(goal.frontiers,1)
            frontier = goal.frontiers{i};
            scatter(frontier(:,2), frontier(:,1), 'go');
        end
    end;
    
    drawnow;
    hold off;
    
    % Save planner pictures
    if configPars.savePlanMovie
        configPars.planMovie = addframe(configPars.planMovie, 10);
    elseif configPars.savePlanPics
        if ~configPars.samePlanPic
            configPars.planPicNum = configPars.planPicNum + 1;
            fn = sprintf('data/planpic-%d', configPars.planPicNum);
        else
	    fn = sprintf( '%s/planpic', getenv('DISPLAY_IMAGES') );
            %fn = 'data/planpic';
        end;
        saveas(10, fn, 'png');
    end;
    
end;

