function [objs nxtObjId objLocs] = addObjectsToMap(Rx_cam,disp,objects,position_offset,cameraPars,mappingPars)
% Function for obtaining 2D global coordinates of odometry,
% laser readings, and salient objects
%
% Rx_cam - 3x3 matrix to rotate 3D coordinates from camera to robot frame
% disp - 8bit or 16bit disparity map
% objects - salient objects
% configFile - The bumblebee config file
% is16BitDisp - 1 if disparity is 16bit, 0 if it's 8bit
% logFile - The map log file
%

% Unpack camera parameters
configFile = cameraPars.configFile;
CAMERAHEIGHT = cameraPars.BB_HEIGHT45;
is16BitDisp = cameraPars.is16BitDisp;
curObjId = mappingPars.curObjId;
objs = mappingPars.objs;
logFile = mappingPars.logFile;
crStr = sprintf('\n');

% Add salient objects onto map
nxtObjId = curObjId;
objLocs = [];

% % set disparity limits based on bitdepth
% if is16BitDisp
%   DISP_MAX = 65280;
%   DISP_MIN = 1536;
% else
%   DISP_MAX = 252;
%   DISP_MIN = 6;
% end;

for obj=objects
  %topRight = [bbox.topLeft(1) bbox.botRight(2)];
  %botLeft = [bbox.botRight(1) bbox.topLeft(2)];

%   % Use saliency peak as center
%   cen_x = obj.centre(2);
%   cen_y = obj.centre(1);
%   % Calculate center of bounding box
%   %cen_x = (obj.bb.botRight(2)+obj.bb.topLeft(2))/2;
%   %cen_y = (obj.bb.botRight(1)+obj.bb.topLeft(1))/2;
%   
%   % Find the median disparity in the neighbourhood of the center
% %   disparities = [];
%   w=3;
%   [rows,cols]=size(disp);
%   while(w<20)
%     xl=max(1,round(cen_x-w)); xh=min(cols,round(cen_x+w));
%     yl=max(1,round(cen_y+w)); yh=min(rows,round(cen_y+w));
%     dispc=disp(yl:yh,xl:yh);dispc=dispc(:);
%     dispc=dispc((dispc>DISP_MIN)&(dispc<DISP_MAX));
%     if ~isempty(dispc)
%       disp(round(cen_y), round(cen_x)) = median(dispc);
%       break;
%     end
%     w=w+1;
%   end
%   for x=-5:5
%     for y=-5:5
%       disparity = disp(round(cen_y+y),round(cen_x+x));
%       if disparity < DISP_MAX & disparity > DISP_MIN
%         disparities = [disparities; disparity];
%       end;
%     end;
%   end;
%   disp(round(cen_y), round(cen_x)) = median(disparities);

%   % Get object center in camera coordinates
%   if is16BitDisp
%     xyz_c = triclops_getXYZ16([round(cen_y) round(cen_x)], disp, configFile);
%   else
%     xyz_c = triclops_getXYZ([round(cen_y) round(cen_x)], disp, configFile);
%   end;

    xyz_r = obj.xyz_r;
    
    xyz_c = obj.xyz_c;

%   if(xyz_c(3) < 0)
%     fprintf('obj=%d disp=%d Failed to get xyz_c\n',nxtObjId,disp(round(cen_y), round(cen_x)));
%     objLocs = [objLocs; 0 0 0]; % object can never be right at where the robot is
%   else
%     % log object center
%     fwrite(logFile,'s ','char');
%     objStr = sprintf('%g ',xyz_c);
%     fwrite(logFile,objStr,'char');
%     fwrite(logFile,crStr,'char');
% 
    % Translate object center to robot coordinates
    yzx_r = Rx_cam*xyz_c';
    xyz_r_old = [yzx_r(3)+position_offset -yzx_r(1) CAMERAHEIGHT-yzx_r(2)];

    % Add object to list
    objLocs = [objLocs; xyz_r];
%   end;

  % Increment next object ID
  nxtObjId = nxtObjId+1;
end;

if ~isempty(objLocs)
    % Add new objects
    performSlam('add_objects',objLocs);

    % Add objects to list
    nObs = size(objs,1);
    nNewObjs = size(objLocs,1);
    objs = [ objs(1:nObs,:);
        objLocs    zeros(nNewObjs,37) ];
end;

