function wedgeMask = buildWedgeMask(trueTilt,cPose,Zcamera,wedgeSize)
% Build a wedge mask for a single view. This is done by first calculating
% the 4 corners of the camera FOV and then using those to create the
% wedge mask (computeCoveredCells)
%
% trueTilt - true camera tilt (including tilt_offset)
% cPose - camera pose [x y theta] in global coordinates
% Zcamera - height of camera
%

% The following lines copied from BB config file. Should be made
% non-constant
%Resolution 240 320
%FocalLength 0.786217 1.04829
%ImageCenter 0.540642 0.440145

resX = 320;
resY = 240;
  
T_WR = BuildRotMatRPY( cPose(1),cPose(2),0,0,0,cPose(3));

%A = [700 0 resX/2; 0 700 resY/2; 0 0 1];
fovX = 60;
fovY = 50;

fx = resX/2/tan(fovX*pi/180/2);
fy = resY/2/tan(fovY*pi/180/2);
A = [fx 0 resX/2; 0 fy resY/2; 0 0 1];

thetaCamera = -trueTilt * pi / 180;

% Add some constant offsets to make the axes correct initially, before
% taking care of the current configuration
rollOffset = -pi/2;
pitchOffset = 0;
yawOffset = -pi/2;

T_RC = BuildRotMatRPY( 0,0,Zcamera,thetaCamera+rollOffset,pitchOffset,yawOffset);

T_WC = T_WR * T_RC;
        
fig =1;
silent = true;
        
[ cameraInfo.worldCorners ] = plotFOV( A, T_WC,resX, resY, fig, silent);        

for i=1:4
    corners(i,:) = performSlam('get_gridpos_from_vecpos',cameraInfo.worldCorners{i}(1:2)');
end;

wedgeMask = computeCoveredCells(corners,wedgeSize);

