function [ visible_regions wedgeMasks mappingPars] = buildWedgeMasks(binCentres,map_path,positionPars,cameraState,mappingPars,planningPars )
% Build wedge masks for views around our current location
%
% trueTilt - true camera tilt (including tilt_offset)
% Zcamera - Height of camera above ground
% position_offset - offset of camera relative to robot
% binCentres - angles to build wedges for
% 

% Unpack parameters
oGrid = positionPars.oGrid;
gridPos = positionPars.rGridPos;
rPose = positionPars.rPose;
trueTilt = cameraState.trueTilt;
Zcamera = cameraState.CAMERAHEIGHT;
position_offset = cameraState.position_offset;
minDist = planningPars.minViewDist;
maxDist = planningPars.maxViewDist;
coverageMap = mappingPars.coverageMap;

% figure(45);
% colormap(gray(256));imagesc(oGrid);
% hold on;
% plot(gridPos(2),gridPos(1),'g+');

[rows,cols] = size(oGrid);
obstacle_map = im2bw(oGrid,planningPars.OBSTACLE_THRESHOLD);

% First compute the bounding area of the map (map_region)

% figure(47);
% subplot(2,2,1);
% colormap(gray(256));
% imagesc(obstacle_map);
% subplot(2,2,2);
omapdilate=imdilate(obstacle_map,strel('disk',6));
% colormap(gray(256));
% imagesc(omapdilate);
% subplot(2,2,3);
map_region=imerode(omapdilate,strel('disk',6));
% colormap(gray(256));
% imagesc(map_region);
% subplot(2,2,4);
%internal_obstacles = map_region-obstacle_map;
% colormap(gray(256));
% imagesc(internal_obstacles);

figure(45); colormap(gray(256));
imagesc(map_region.*double(coverageMap));
% mappingPars.mapId = saveMap(mappingPars.mapId,map_path);

% Now find area inside map region that hasn't been seen so far
unseen_regions = map_region./(double(coverageMap) + 1);
%maxVal = max([coverageMap(:); 1]);
%unseen_regions = uint8(map_region) .* ( maxVal - coverageMap );

figure(46); colormap(gray(256));
imagesc(unseen_regions);

% mappingPars.mapId = saveMap(mappingPars.mapId,map_path);

% largeMask = zeros([rows cols],'uint8');
% smallMask = ones([rows cols],'uint8');
% 
% cl=-minDist:minDist;
% [x y] = meshgrid(cl,cl);
% [ir ic]=find(x.^2+y.^2<minDist^2);
% ir=cl(ir)+gridPos(1);
% ic=cl(ic)+gridPos(2);
% validl=find((ir>0)&(ir<rows)&(ic>0)&(ic<cols));
% il=sub2ind([rows cols],ir(validl),ic(validl));
% smallMask(il)=0;
% 
% cl=-maxDist:maxDist;
% [x y] = meshgrid(cl,cl);
% [ir ic]=find(x.^2+y.^2<maxDist^2);
% ir=cl(ir)+gridPos(1);
% ic=cl(ic)+gridPos(2);
% validl=find((ir>0)&(ir<rows)&(ic>0)&(ic<cols));
% il=sub2ind([rows cols],ir(validl),ic(validl));
% largeMask(il)=1;

% visible_regions = unseen_regions .* largeMask;
% visible_regions = visible_regions .* smallMask;
visible_regions = unseen_regions;

% figure(48);
% colormap(gray(256));
% imagesc(unseen_regions);

% figure(46); axis image;
% colormap(gray(256));

% Return the wedge masks
wedgeMasks = false(rows,cols,6);

% invert sign for binCentres because y-axis in grid coordinates points down
%binCentres = -binCentres;

nBins = length(binCentres);
for j=1:nBins
    
%     Old way of creating wedge masks
%     minRange = binCentres(j) - pi/6;
%     maxRange = binCentres(j) + pi/6;
%     
%     ang = atan2(y,x);
%     
%     normMin = thetapp(minRange);
%     normMax = thetapp(maxRange);
%     valid = ang > minRange & ang < maxRange | ...
%     ang > normMin & normMin ~= minRange | ...
%     ang < normMax & normMax ~= maxRange;
%     
%     [ir ic] = find( x.^2+y.^2<maxDist^2 & valid);
%     ir=cl(ir)+gridPos(1);
%     ic=cl(ic)+gridPos(2);
%     validl=find((ir>0)&(ir<rows)&(ic>0)&(ic<cols));
%     il=sub2ind([rows cols],ir(validl),ic(validl));
%     wedgeMask(il)=1;

    rPose(3) = binCentres(j);
    cPose = rPose2cPose(rPose, position_offset);
    wedgeMask = buildWedgeMask(trueTilt,cPose,Zcamera,[rows cols]);

    %subplot(ceil(nBins/6),6,j);
    %imagesc(visible_regions.*wedgeMask);   
    %imagesc(wedgeMask);
    
    % Return the wedge masks
    wedgeMasks(:,:,j) = wedgeMask;
end;



