function wedgeMask = computeCoveredCells(corners,wedgeSize)
% Creates a wedgeMask of size wedgeSize encompassing all cells within the 
% quadrilateral defined by corners
% 

% First check to ensure sizes are valid
% for c1 = 1:4
% if corners(c1,1) < 1 || corners(c1,1) > wedgeSize(2) || corners(c1,2) < 1 || corners(c1,2) > wedgeSize(1)
% 
%     fprintf(1,'The robots pose is outside the coverage map!');
%     return;
% end;
% end;

% Compute the grid cells which have been observed by looping over the cells
% inside the bounding box and checking whether each grid cell lies in the
% interior of the region

wedgeMask = false(wedgeSize);
bbox = [ max(1,floor(min(corners(:,1)))) max(1,floor(min(corners(:,2)))) min(wedgeSize(2),ceil(max(corners(:,1)))) min(wedgeSize(1),ceil(max(corners(:,2)))) ];

for row = bbox(1):bbox(3)
    for col = bbox(2):bbox(4)
        
        % Need to check that the point lies on the correct side of 4 lines
        
        valid = true;
        c1 = 1;

        while c1 <= 4 && valid

            c2 = mod(c1,4)+1;

            if (corners(c2,2)-corners(c1,2))*(row-corners(c1,1)) - (corners(c2,1)-corners(c1,1))*(col-corners(c1,2)) < 0
                valid = false;
           end; 

           c1 = c1 + 1;
        end;
        
        if valid
            wedgeMask(row,col) = true;
            %coverageMap(row,col)=min(coverageMap(row,col)+25,255);
            %resultCells = [ resultCells; row col ];
            %plot(row,col,'r.');
        end;
    end;
end;

%plot(corners(:,1),corners(:,2),'bo');
%plot(resultCells(:,1),resultCells(:,2),'r.');
