function destOdom = getDestOdom(destCell)
% Function to convert a destination from an occupancy grid
% cell to the robot's odometry. Since the map maintains its
% own corrected poses, we must first find the difference in
% map poses, and then add that to the robot's odometry to
% find the destination robot odometry.
%
% destCell - [row col] of destination occupancy grid cell
%

% Current position
odom = powerbot('getodom');
mapPose = performSlam('get_odom');

% Next destination
nxtMapPos = performSlam('get_vecpos_from_gridpos',destCell);

% Calculate next odometry
dirVec = nxtMapPos-mapPose(1:2);
destOdom = odom + [dirVec atan2(dirVec(2),dirVec(1))];
