/*
** Compile command:
** make gaze
*/
#include <stdio.h>
#include "playerc.h"
#include "playerpacket.h"
//#include "player.h" // for player_camera_data
#include "pnmio.h"

//#define USE_POSITION
#define USE_CAMERA
#define USE_PTZ

#define PI 3.1415926536
#define PAN_SPEED PI/7.0
#define TILT_SPEED PI/7.0

//typedef short int16_t;

void save_image(playerc_camera_t *camera,char *filename) {
  FILE *fid;
  int format=0;
  switch(camera->bpp) {
  case 8:
    format=RPGM_FORMAT;
    break;
  case 24:
    format=RPPM_FORMAT;
    break;
  default:
    printf("Error: unsupported number of bits/pixel\n");
  }
  if(format) {
    fid=pnm_writehead(filename,format,camera->height,camera->width);
    fwrite(camera->image,sizeof(uint8_t),camera->image_size,fid);
    fclose(fid);  
    printf("Saved %dbpp %dx%d image as '%s'\n",camera->bpp,camera->width,
	   camera->height,filename);
  }
}

int shift_gaze(playerc_client_t *client,playerc_ptz_t *ptz,
	       double pan,double tilt)
{
  int i,r,cnt;
  void *rdevice;
  double rpan,rtilt;
  if (playerc_ptz_subscribe(ptz, PLAYER_ALL_MODE) != 0)
   {
    fprintf(stderr,"ptz error: %s\n", playerc_error_str());
    return -1;
   }
  printf("Subscribed to ptz.\n");
  printf("converted:%d\n",(int16_t) (pan * 180 / PI));
  printf("converted:%d\n",(int16_t) (tilt * 180 / PI));
  printf("converted:%d\n",(int16_t) (PAN_SPEED * 180 / PI));
  r=playerc_ptz_set_ws(ptz,pan,tilt,0.0,PAN_SPEED,TILT_SPEED);
  /*r=playerc_ptz_set(ptz,pan,tilt,0.0);*/
  printf("Requested pan:%g tilt:%g -> %d\n",pan,tilt,r);

  rpan=1e8;rtilt=1e8;cnt=0;
 for (i = 0; i < 100; i++)
  {
    printf("PTZ read %d: ", i);                                                                                
    do {
      rdevice = playerc_client_read(client);
      usleep(1000);
    } while (rdevice == client);
                                                                               
    if (rdevice == ptz)
    {
      if(rpan ==pan && rtilt == tilt) {
	printf("PTZ done.\n");cnt++;
      } else if(rpan ==ptz->pan && rtilt == ptz->tilt) {
	printf("PTZ stopped.\n");cnt++;
      } else {
	printf("ptz: [%g %g %g]\n",ptz->pan, ptz->tilt, ptz->zoom);
	rpan=ptz->pan;
	rtilt=ptz->tilt;
	cnt=0;
      }
    }
    else
    {
      printf("Not ptz\n");
    }
    if(cnt>3) break;
  }
  playerc_ptz_unsubscribe(ptz);
}

int main(int argc, const char **argv)
{
  int i,k;
  playerc_client_t *client;
  void *rdevice;
#ifdef USE_POSITION
  playerc_position_t *position;
#endif
#ifdef USE_CAMERA
  playerc_camera_t *camera;
#endif
#ifdef USE_PTZ
  playerc_ptz_t *ptz;
#endif  

  printf("Client program running.\n");
  // Create a client object and connect to the server; the server must
  // be running on "localhost" at port 6665
  client = playerc_client_create(NULL, "lisa.ubcrobots.net", 6665);
  if (playerc_client_connect(client) != 0)
  {
    fprintf(stderr, "client error: %s\n", playerc_error_str());
    return -1;
  }
  printf("Connected to server.\n");

  if(playerc_client_datamode(client, PLAYER_DATAMODE_PULL_NEW) != 0) {
    fprintf(stderr, "client error: %s\n", playerc_error_str());
    return -1;
  }

#ifdef USE_POSITION
  // Create a position proxy (device id "position:0") and susbscribe
  // in read/write mode
  position = playerc_position_create(client, 0);
  if (playerc_position_subscribe(position, PLAYER_ALL_MODE) != 0)
  {
    fprintf(stderr, "position error: %s\n", playerc_error_str());
    return -1;
  }
  printf("Subscribed to position.\n");
#endif

#ifdef USE_CAMERA
  // Create camera proxy
  camera = playerc_camera_create(client, 0);  // int index
#endif

#ifdef USE_PTZ
  // Create ptz proxy
  ptz = playerc_ptz_create(client, 0);  // int index
#endif

#ifdef USE_POSITION
  // Enable the robots motors
  playerc_position_enable(position, 1);

  // Start the robot turning slowing
  //playerc_position_set_cmd_vel(position, 0, 0, 0.1, 1);

  // Move forward instead
  //playerc_position_set_cmd_vel(position, 0.1, 0, 0, 1);
  for (i = 0; i < 5; i++)
  {
    sleep(1);
    // Read data from the server and display current robot position
    playerc_client_read(client);
    printf("position : %f %f %f\n",
           position->px, position->py, position->pa);
  } 
#endif
#ifdef USE_POSITION
  playerc_position_unsubscribe(position);
  playerc_position_destroy(position);
#endif

#ifdef USE_CAMERA
  if (playerc_camera_subscribe(camera, PLAYER_READ_MODE) != 0)
   {
    fprintf(stderr,"camera error: %s\n", playerc_error_str());
    return -1;
   }
  printf("Subscribed to camera.\n");
  do {
   rdevice = playerc_client_read(client);
   printf(".");usleep(1000);
  } while (rdevice == client);

  // Read out camera info
  printf("camera=%d\n",camera);
  printf("camera.info.drivername=%s\n",camera->info.drivername);
  printf("camera.width=%d\n",camera->width);
  printf("camera.height=%d\n",camera->height);
  printf("camera.bpp=%d\n",camera->bpp);
  printf("camera.format=%d\n",camera->format);
  printf("camera.fdiv=%d\n",camera->fdiv);
  printf("camera.compression=%d\n",camera->compression);
  printf("camera.image_size=%d\n",camera->image_size);
  printf("camera.image=");
  for(k=0;k<255;k++) {
    printf("%d ",camera->image[k]);
  }
  printf("\n");
  save_image(camera,"/home/perfo/gaze/test_image.pnm");
  playerc_camera_unsubscribe(camera);
#endif

#ifdef USE_PTZ
  shift_gaze(client,ptz,45*PI/180.0,19.5*PI/180.0);
#endif

#ifdef USE_CAMERA
  if (playerc_camera_subscribe(camera, PLAYER_READ_MODE) != 0)
   {
    fprintf(stderr,"camera error: %s\n", playerc_error_str());
    return -1;
   }
  printf("Subscribed to camera.\n");
  do {
   printf(".");usleep(1000);
   rdevice = playerc_client_read(client);
  } while (rdevice == client);

  printf("camera.image=");
  for(k=0;k<255;k++) {
    printf("%d ",camera->image[k]);
  }
  printf("\n");
  save_image(camera,"/home/perfo/gaze/test_image2.pnm");
  playerc_camera_unsubscribe(camera);
#endif

#ifdef USE_PTZ
  /* One degree accuracy due to player */
  shift_gaze(client,ptz,-PI/180.0,-PI/180.0);
#endif


  // Shutdown and tidy up
#ifdef USE_CAMERA
  playerc_camera_destroy(camera);
#endif
#ifdef USE_PTZ
  playerc_ptz_destroy(ptz);
#endif
  playerc_client_disconnect(client);
  playerc_client_destroy(client);

  return 0;
}
