/*
 *  Simple test of PTU control without player
 */

#include <stdio.h>
#include "pnmio.h"
#include "ptu46_mini.h"

#include <string.h>
#include <libraw1394/raw1394.h>
#include <libdc1394/dc1394_control.h>
#include "bb_mini.h"

int main(int argc, const char **argv)
{
  /* PTU46_mini variables */
  int fd;
  char *port,*cmd;
  char *buffer;
  float pos_pan,pos_tilt;
  /* dc1394 variables */
  char *dma_dev_name = DMA_DEV_NAME;
  dc1394_cameracapture camera;
  raw1394handle_t handle;
  /* Other stuff */
  int infofl=1;       /* Print info if set */
  char filename[50];
  int k;

  /* Setup for PTU46 */
  port=strdup(PTU46_PORTNAME);
  fd=PTU46_Init(port,&buffer);

  // Move PTU to initial position
  PTU46_SetPanTilt(fd,buffer,0.0,0.0,PTU46_BLOCK);
  pos_pan  = PTU46_GetPos(fd,buffer,PTU46_PAN);
  pos_tilt = PTU46_GetPos(fd,buffer,PTU46_TILT);
  printf("PTU position: p %g t %g\n",pos_pan,pos_tilt);

  /* Find first camera on 1394 bus */
  BB_find_camera(&handle,&camera,infofl);

  if(BB_setup_capture(handle,&camera,dma_dev_name)==-1) return;    
  if(BB_start_iso_transmission(handle,&camera)==-1) return;

  for(k=0;k<10;k++) {
    pos_pan=k*5.6;
    pos_tilt=k*2.6;

    // Move PTU
    PTU46_SetPanTilt(fd,buffer,pos_pan,pos_tilt,PTU46_BLOCK);
    pos_pan  = PTU46_GetPos(fd,buffer,PTU46_PAN);
    pos_tilt = PTU46_GetPos(fd,buffer,PTU46_TILT);
    printf("PTU position: p %g t %g\n",pos_pan,pos_tilt);

    if(infofl) printf("BB:single_capture\n"); 
    if(BB_single_capture(handle,&camera)==-1) return;
    if(infofl) printf("BB:write to file\n"); 
    
    sprintf(filename,"camera_image%.2d.pnm",k);
    if(BB_save_image(filename,&camera)==-1) return;
    BB_buffer_free(&camera);
  }
 
  // Move PTU to initial position
  PTU46_SetPanTilt(fd,buffer,0.0,0.0,PTU46_BLOCK);
  pos_pan  = PTU46_GetPos(fd,buffer,PTU46_PAN);
  pos_tilt = PTU46_GetPos(fd,buffer,PTU46_TILT);
  printf("PTU position: p %g t %g\n",pos_pan,pos_tilt);

  printf("At end.\n");
  PTU46_Release(fd,buffer);

  /* Close camera */
  BB_release(handle,&camera);
}
