/*
 *  Minimal socket interface (based on seriald from Rob)
 */

#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/poll.h>
#include <string.h>
#include <netdb.h>
#include <time.h>

#include "pnmio.h"
#include "socket_mini.h"
#include "server_mini.h"

static char requestData[MSG_BUFSIZE];

void write_time_diff(char *strg,clock_t t0,clock_t t1) {
 printf("%s %g\n",strg,(double)(t1-t0)/CLOCKS_PER_SEC);
}
#define SERVER_TIMEOUT 2.0

int Message_send(Message *msg, int fd) {
/*   fprintf(stderr,"Sending message: %d bytes to %d\n", msg->size,fd); */
/*   fflush(stderr); */
  int num_wrote=0;
  unsigned char * buf=msg->data;
  clock_t t1,t2;
  t1=t2=clock();
  while (num_wrote<msg->size) {
    //int nw=write(fd,buf,msg->size-num_wrote);
    int nw = send(fd,buf,msg->size-num_wrote,MSG_DONTWAIT);
    if (nw==-1) {
      switch(errno) {
      case EAGAIN:
	break;
      default:
	fprintf(stderr,"Message_send: write error\n");
	fflush(stderr);
	return -1;
      }
      t2=clock();
      if((t2-t1)>SERVER_TIMEOUT*CLOCKS_PER_SEC) {
	fprintf(stderr,"Message_send: write abort\n");
	fflush(stderr);
	return -1;
      }
    } else {
      t1=clock();
      num_wrote+=nw;
      buf+=nw;
      fprintf(stderr,"Wrote %d, %d left.       \r",nw,msg->size-num_wrote);
      fflush(stderr);
    }
  }
  /* fprintf(stderr,"Sent %d bytes to %d\n",num_wrote, fd); */
  fflush(stderr);
  return 0;
}

void Message_printf(char *strg,Message *msg) {
  if(msg->size>0) {
    /* fprintf(stderr,"%s%s\n",strg,msg->data); */
  } else {
    fprintf(stderr,"%s<NULL>\n",strg);
  }
  fflush(stderr);
}

Message *Message_new(char *strg) {
  Message *msg;
  msg = (Message *)calloc(1,sizeof(Message));
  if(strg != NULL) {
    msg->data=(unsigned char *)strdup(strg);
    msg->size=strlen((char *)msg->data);
  } else {
    msg->data=NULL;
    msg->size=0;
  }
  return msg;
}

Message *Message_request(char *strg) {
  Message *msg;
  msg = (Message *)calloc(1,sizeof(Message));
  if(strg != NULL) {
    memset(requestData,'\0',MSG_BUFSIZE);
    sprintf(requestData,strg);
    msg->data=(unsigned char *)calloc(MSG_BUFSIZE,sizeof(unsigned char));//(unsigned char *)strdup(strg);
    strncpy((char *)msg->data,requestData,MSG_BUFSIZE);
    msg->size=MSG_BUFSIZE;//strlen((char *)msg->data);
  } else {
    msg->data=NULL;
    msg->size=0;
  }

  /* fprintf(stderr,"Sending message %s.\n",msg->data); */
  return msg;
}

Message *Message_posn(float pos_pan,float pos_tilt) {
  Message *msg;
  msg = (Message *)calloc(1,sizeof(Message));
  msg->data=(unsigned char *)calloc(MSG_BUFSIZE,sizeof(unsigned char));
  snprintf((char *)msg->data,MSG_BUFSIZE,"p%g;%g\n",pos_pan,pos_tilt);
  msg->size=MSG_BUFSIZE;//strlen((char *)msg->data);
  return msg;
}

Message *Message_moveTo(float *odom) {
  Message *msg;
  msg = (Message *)calloc(1,sizeof(Message));
  msg->data=(unsigned char *)calloc(MSG_BUFSIZE,sizeof(unsigned char));
  snprintf((char *)msg->data,MSG_BUFSIZE,"n%g;%g;%g\n",odom[0],odom[1],odom[2]);
  msg->size=MSG_BUFSIZE;//strlen((char *)msg->data);
  return msg;
}

Message *Message_zoom(int zoom) {
  Message *msg;
  msg = (Message *)calloc(1,sizeof(Message));
  msg->data=(unsigned char *)calloc(MSG_BUFSIZE,sizeof(unsigned char));
  snprintf((char *)msg->data,MSG_BUFSIZE,"z%d\n",zoom);
  msg->size=MSG_BUFSIZE;//strlen((char *)msg->data);
  return msg;
}

void Message_free(Message *msg) {
  if(msg->size>0) free(msg->data);
  free(msg);
}

int establish_local_socket(int port, struct sockaddr_in* skname) {
  int lsk;
  int flags;
  int enable = 1;
  /** socket **/
  //if (-1 == (lsk= socket( AF_INET, SOCK_STREAM, 0))) 
  if (-1 == (lsk= socket( PF_INET, SOCK_STREAM, 0))) 
    exit(1);

  if((flags = fcntl(lsk, F_GETFL)) == -1) {
    close(lsk);
    exit(1);
  }

  if(fcntl(lsk, F_SETFL, flags | O_NONBLOCK ) == -1) {
    close(lsk);
    exit(1);
  }

  if(setsockopt(lsk, SOL_SOCKET, SO_REUSEADDR, (const char*)&enable, 
		sizeof(enable)) == -1) {
    close(lsk);
    exit(1);
  }
  /** bind **/
  skname->sin_family= AF_INET;
  skname->sin_port= htons(port);
  skname->sin_addr.s_addr= htonl( INADDR_ANY);
  if (-1 == bind( lsk, (struct sockaddr*)skname, sizeof(*skname))) 
    { perror("bind"); exit(2);}
  
  /** listen **/
  listen( lsk, BACKLOG);
  return lsk;
}

int establish_remote_socket(int port, struct sockaddr_in* skname,char *hostname) {
  int rsk;
  socklen_t addrlen;
  unsigned long inaddr;
  struct hostent *host;
  int enable = 1;
  int flags;
  host = gethostbyname(hostname);
  
  /** socket **/
  if (-1 == (rsk= socket( PF_INET, SOCK_STREAM, 0))) 
    return -1;

  /** connect **/
  skname->sin_family = PF_INET;
  skname->sin_port = htons(port);
  memcpy(&(skname->sin_addr), host->h_addr_list[0], host->h_length);
  if(-1 == connect(rsk,(struct sockaddr *)skname,sizeof(*skname))) {
    close(rsk);
    return -1;
  }

  if((flags = fcntl(rsk, F_GETFL)) == -1) {
    close(rsk);
    return -1;
  }

  if(fcntl(rsk, F_SETFL, flags | O_NONBLOCK ) == -1) {
    close(rsk);
    return -1;
  }

  if(setsockopt(rsk, SOL_SOCKET, SO_REUSEADDR, (const char*)&enable, 
		sizeof(enable)) == -1) {
    close(rsk);
    return -1;
  }
  return rsk;
}

int accept_connection(int socket) {
  /** accept **/
  int csk;
  struct sockaddr_in peername;
  unsigned int peerlen= sizeof(struct sockaddr_in);
  while (1) {
    if (-1 == (csk= accept( socket, (struct sockaddr*)&peername, &peerlen)) &&
	errno!=EINTR)  /* ignore syscall interrupts- if we catch sighup we
			  don't want to die */  
      {	perror("accept"); exit(2);}
    
    if(csk!=-1)
      return csk;
  }
  return -1;
}

void close_connection(int fd) {
  close(fd);
}

int server_listen(int fd) {
  struct pollfd pfd[1];
  pfd[0].fd=fd;
  pfd[0].events=POLLIN;
  pfd[0].revents=0;
  while(pfd[0].revents!=POLLIN) {
    int rval;
    while ((rval=poll(pfd,1,5000))==0) {
      fprintf(stderr,"Watchdog...\n");
    }
    if (rval==-1) {
      perror("poll");
      sig_shutdown(0);
      break;
    } 
  }
  return pfd[0].revents;
}

#define READ_BUFFER1  256
Message *read_server_command(int fd) {
  unsigned char buf[READ_BUFFER1+1];
  Message *msg = NULL;
  int numread=0;
  while(numread==0) {
    numread=read(fd,buf,READ_BUFFER1);
    if(numread==-1) if(errno==EAGAIN) numread=0;
  }
  if(numread > 0) {
    msg=(Message *)calloc(1,sizeof(Message));
    msg->size=numread;
    msg->data=(unsigned char*)calloc(numread,sizeof(char));
    memcpy(msg->data,buf,numread);
  }
  return msg;
}

#define CLIENT_TIMEOUT 2.0
Message *read_fixlength_packet(int fd) {
  unsigned char buf[MSG_BUFSIZE];
  Message *msg = NULL;
  clock_t t1,t2;

  memset(buf,'\0',MSG_BUFSIZE);

  t1=t2=clock();
  /* fprintf(stderr,"read_data_header start.\n"); */
  /* fflush(stderr); */

  unsigned char *cur = buf;
  int numread=0,numleft=MSG_BUFSIZE;
  while (numleft) {
    numread=recv(fd,cur,numleft,0);

    /* fprintf(stderr,"recv got %s.\n",cur); */

    switch(numread) {
    case -1:
      if(errno!=EAGAIN) {
	/* Early end of file */
	fprintf(stderr,"read error:%d\n",errno);
	fflush(stderr);
	return NULL;
      }
      /* If EAGAIN, continue here: */
    case 0:
      /* Possible connection lockup */
      t2=clock();
      if((t2-t1)>CLIENT_TIMEOUT*CLOCKS_PER_SEC) {
	fprintf(stderr,"read_data_header: read abort\n");
	fflush(stderr);
	return NULL;
      }
      break;
    default:
      t1=clock();
      fprintf(stderr,"numleft: %d numread: %d       \r", numleft,numread);
      fflush(stderr);
      cur+=numread;
      numleft-=numread;
    }
  }

  int size = strlen((char*)buf);
  msg=(Message *)calloc(1,sizeof(Message));
  msg->size=size;
  msg->data=(unsigned char*)calloc(size+1,sizeof(char));
  memset(msg->data,'\0',size+1);
  memcpy(msg->data,buf,size);

  /* fprintf(stderr,"msg->data holds %s.\n",msg->data); */

  return msg;
}

#define CLIENT_TIMEOUT 2.0
Message *read_data_packet(int fd, int size) {
  unsigned char *buf = (unsigned char *)calloc(size,sizeof(unsigned char));
  Message *msg = NULL;
  clock_t t1,t2;

  t1=t2=clock();
  /* fprintf(stderr,"read_data_packet start.\n"); */
  /* fflush(stderr); */

  unsigned char *cur = buf;
  int numread=0,numleft=size;
  while (numleft) {
    numread=recv(fd,cur,numleft,0);
    switch(numread) {
    case -1:
      if(errno!=EAGAIN) {
	/* Early end of file */
	fprintf(stderr,"read error:%d\n",errno);
	fflush(stderr);
	free(buf);
	return NULL;
      }
      /* If EAGAIN, continue here: */
    case 0:
      /* Possible connection lockup */
      t2=clock();
      if((t2-t1)>CLIENT_TIMEOUT*CLOCKS_PER_SEC) {
	fprintf(stderr,"read_data_packet: read abort\n");
	fflush(stderr);
	free(buf);
	return NULL;
      }
      break;
    default:
      t1=clock();
      /*
      fprintf(stderr,"numleft: %d numread: %d       \r", numleft,numread);
      fflush(stderr);
      */
      cur+=numread;
      numleft-=numread;
    }
  }

  msg=(Message *)calloc(1,sizeof(Message));
  msg->size=size;
  msg->data=(unsigned char*)calloc(size,sizeof(char));
  memcpy(msg->data,buf,size);

  free(buf);
  return msg;
}

#define READ_BUFFER2  614408
#define PACKET_HEADER_SIZE 8
/* Returns NULL on failure
 *  or a camera_packet struct is successful
 */
camera_packet *read_camera_packet(int fd) {
  short *short_ptr;
  long  *long_ptr;
  unsigned char buf[READ_BUFFER2+1];
  unsigned char *buff;
  int numleft,numread;
  clock_t t1,t2;
  camera_packet *camera;

  t1=t2=clock();
  /*
   fprintf(stderr,"read_camera_packet start.\n");
   fflush(stderr);
  */
  /* Read header */
  numleft=PACKET_HEADER_SIZE;
  buff=buf;
  while(numleft) {
    /* numread=read(fd,buf,PACKET_HEADER_SIZE);*/
    numread=recv(fd,buff,numleft,0);
    switch(numread) {
    case -1:
      if(errno==EAGAIN)	break;
      /* Early end of file */
      fprintf(stderr,"header read error:%d\n",errno);
      fflush(stderr);
      return NULL;
    case 0:
      /* Possible connection lockup */
      t2=clock();
      if((t2-t1)>CLIENT_TIMEOUT*CLOCKS_PER_SEC) {
	fprintf(stderr,"read_camera_packet: header read abort\n");
	fflush(stderr);
	return NULL;
      }
      break;
    default:
      t1=clock();
      buff+=numread;
      numleft-=numread;
    }
  }
  camera=(camera_packet *)calloc(1,sizeof(camera_packet));
  /* Parse header */
  short_ptr = (short *)buf;
  camera->width = ntohs(short_ptr[0]);
  camera->height = ntohs(short_ptr[1]);
  long_ptr = (long *)&buf[4];
  camera->size=ntohl(long_ptr[0]);
  printf("Packet size: width=%d height=%d size=%d\n",camera->width,
	 camera->height,camera->size);
  /* Read data */
/*   numleft=(camera->width)*(camera->height)*3; */
  numleft=camera->size;
  camera->data=(char *)calloc(numleft,sizeof(char));
  buff = (unsigned char *)camera->data;
  while (numleft) {
    numread=recv(fd,buff,numleft,0);
    switch(numread) {
    case -1:
      if(errno!=EAGAIN) {
	/* Early end of file */
	fprintf(stderr,"read error:%d\n",errno);
	fflush(stderr);
	free(camera->data);
	free(camera);
	return NULL;
      }
      /* If EAGAIN, continue here: */
    case 0:
      /* Possible connection lockup */
      t2=clock();
      if((t2-t1)>CLIENT_TIMEOUT*CLOCKS_PER_SEC) {
	fprintf(stderr,"read_camera_packet: read abort\n");
	fflush(stderr);
	free(camera->data);
	free(camera);
	return NULL;
      }
      break;
    default:
      t1=clock();
      fprintf(stderr,"numleft: %d numread: %d       \r", numleft,numread);
      fflush(stderr);
      buff+=numread;
      numleft-=numread;
    }
  }
  
  fprintf(stderr,"bytes received=%d, bytes left=%d\n",(int)buff-(int)camera->data,numleft);
  fflush(stderr);
  return camera;
}

void free_camera_packet(camera_packet *camera) {
  if(camera->data != NULL)
    free(camera->data);
  free(camera);
}
/*
 * Re-package camera image into a packet for transmission.
 *  Packet structure:
 *  short width,short height,long size,data
 */
Message *Message_camera_packet(camera_packet *camera) {
  Message *msg;
  int width,height,image_bytes;
  short *short_ptr;
  long  *long_ptr;

  msg = (Message *)calloc(1,sizeof(Message));
  width = camera->width;
  height = camera->height;
  /* image_bytes = width*height*3;  /* Size of image in bytes */
  image_bytes = camera->size;
  printf("camera.size=%d width*height*3=%d\n",camera->size,width*height*3);
  msg->size = PACKET_HEADER_SIZE+image_bytes;
  msg->data = (unsigned char *)calloc(msg->size,sizeof(unsigned char));
  short_ptr = (short *)&msg->data[0];
  short_ptr[0] = htons(width);
  short_ptr[1] = htons(height);
  long_ptr = (long *)&msg->data[4];
  long_ptr[0] = htonl(image_bytes);  /* left and right */
  memcpy(&msg->data[8],camera->data,image_bytes);
  return msg;
}

/* For debug */
void parse_camera_packet(Message *msg,int *width,int *height,int *size,int **capture_buffer) {
  short *short_ptr;
  long  *long_ptr;
  short_ptr = (short *)&msg->data[0];
  width[0]=ntohs(short_ptr[0]);
  height[0]=ntohs(short_ptr[1]);
  long_ptr = (long *)&msg->data[4];
  size[0]=ntohl(long_ptr[0]);
  long_ptr = (long *)&msg->data[8];
  capture_buffer[0]= (int *)long_ptr;   /* Header is followed by data */
}

int save_image(char *filename, camera_packet *camera) {
  FILE *fid;
  fid=pnm_writehead(filename,RPPM_FORMAT,camera->height,
		    camera->width);
  fwrite(camera->data,sizeof(char),
	 camera->height*camera->width*3,fid);
  fclose(fid);  
  printf("Saved %dbpp %dx%d image as '%s'\n",24,camera->width,
	   camera->height,filename);
  return 0;
}
