/*
 *  Minimal socket interface (based on seriald from Rob)
 */

#ifndef __SOCKET_MINI
#define __SOCKET_MINI

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define MSG_BUFSIZE 256
#define BACKLOG 5

typedef struct Message {
  int size;
  unsigned char * data;
} Message;

typedef struct camera_packet {
  int width;
  int height;
  int size;
  char *data;
} camera_packet;

int Message_send(Message *msg, int fd);
void Message_printf(char *strg,Message *msg);
Message *Message_new(char *strg);
Message *Message_request(char *strg);
Message *Message_posn(float pos_pan,float pos_tilt);
Message *Message_moveTo(float *odom);
Message *Message_zoom(int zoom);
void Message_free(Message *msg);
int establish_local_socket(int port, struct sockaddr_in* skname);
int establish_remote_socket(int port, struct sockaddr_in* skname,
			    char *hostname);
int accept_connection(int socket);
void close_connection(int fd);
int server_listen(int fd);
Message *read_server_command(int fd);
Message *read_fixlength_packet(int fd);
Message *read_data_packet(int fd, int size);
void parse_camera_packet(Message *msg,int *width,int *height,int *size,
			 int **capture_buffer);
camera_packet *read_camera_packet(int fd);
Message *Message_camera_packet(camera_packet *camera);
void free_camera_packet(camera_packet *camera);
int save_image(char *filename, camera_packet *camera);

#endif /* __SOCKET_MINI */
