function [ rewardMap frontiers planningPars ] = calcFrontierRewards ( oGrid, planningPars )
% [ rewardMap ] = calcFrontierRewards ( oGrid, planningPars )
% 
% This function creates and returns a rewardMap of type double that is the same 
% size as the occupancy grid, but that has rewards on all the cells that are
% frontiers. 
% 
% oGrid := the occupancy grid
%
% planningPars := the planner parameters
% 
% rewardMap := the double array of size(oGrid), where each cell on the frontier
%     has value frontierReward and every other cell has 0 value
% 
rewardMap = zeros(size(oGrid));

% Let each frontier cell contribute to the reward map:
[ frontiers lengths ] = getFrontiers(oGrid,floor(255*planningPars.OBSTACLE_THRESHOLD),20,planningPars.explored);
nFrontiers = length(frontiers);
for i = 1:nFrontiers
    
    % Let each frontier cell contribute to the reward function:
    frontier = frontiers{i};
    
    % Rows 1 and 2 correspond to the center and the nearest cell, respectively:
    for f = 3:size(frontier,1)
        
        frontierCell = frontier(f,:); 
        j = frontierCell(1); % vertical/row coord
        i = frontierCell(2); % horizontal/column coord
        
        % Both explored and rewardMap are (row, column)
        % since frontier coordinates are organized this way.
        planningPars.explored(j,i) = 1;
        rewardMap(j,i) = 1;
        
    end;
    
end;

% Blur the frontier map:
f1 = normpdf(-3:1:3, 0, 1.5);
f2 = f1' * f1;
f2 = f2 ./ sum(f2(:));
rewardMap = filter2(f2, rewardMap);

