function [ minHorzDist ] = calcMinHorizonDistance ( objectHeight, cameraPars )
% Calculates the minimum horizontal distance that the robot can stand next
% to an object at the specified height.
%
% objectHeight: The height of a particular object
% cameraPars: Parameters of the camera. Specifically, the height and max angle
% constraint of the ptu.

% Get the camera height:
if cameraPars.canonfl
    cameraHeight = cameraPars.CC_HEIGHT45;
else
    cameraHeight = cameraPars.BB_HEIGHT45;
end;

% Get the ptu max angle:
ptu_lim = cameraPars.ptu_lim;

% Calculate the minimum horizontal distance between the objects in order
% for the ptu to be able to see the object:
minHorzDist = 10 * (cameraHeight-objectHeight) / tan(abs(ptu_lim(3))*pi/180); 
