function [ samples ] = createPoseSamples ( N, configPars, worldState )
% [ samples ] = createPoseSamples ( N, worldState )
% 
%   Creates N rows of "pose" samples of the format [ x, y, theta ] where i is an 
% index into the occupancy grid.
% 
% N := the number of samples to create
% worldState := the current worldState and maps
% 

% Use the obstacle_map to restrict the domain of sampling:
obstacle_map = worldState.obstacle_map;

% Find the set of all legal locations, according to the obstacle map. These
% will be stored as linear indexes.
gridSize = size(worldState.oGrid);
validIndexes = find(worldState.parents(:,:,1));
numValidIndexes = length(validIndexes);
if configPars.PRINTLEVEL >= 1 && numValidIndexes == 0
    fprintf('[createPoseSamples.m] Error: there is no room on the obstacle_map to search in for a goal.\n');
end;

% Create a sample of appropriate size, then transform the random variables into 
% the appropriate domain.
samples = [ rand(N,1) zeros(N,1) rand(N,1) ];
samples(:,1) = validIndexes(ceil(samples(:,1) * numValidIndexes));
[ samples(:,2) samples(:,1) ] = ind2sub(gridSize, samples(:,1));
samples(:,3) = samples(:,3) * 2 * pi - pi;

