function [ samples ] = permutePoseSamples ( samples, worldState )
%
% Permutes the remaining samples after they've been scored. This function
% allows for local exploration of the goal space after they've been scored.
%
% samples := the set of samples (x,y,theta) that we'll be modifying
% worldState := current state 
% elitism := determines how likely we are to favor high score

% Grid size is used for linear indexing
gridSize = size(worldState.obstacle_map);

% the viewing angle can change up to +/- 30 degrees in either direction
% (? parameterize this ?)
yawVariance = pi / 6;

% This is the square root of the area of cells that we can choose from when
% permuting a sample.
permuteSize = 5;

% For all samples, make some small adjustments:
for x = 1 : size(samples,1)
    
    % Read the old sample row into a separate array
    newSample = samples(x,:);
    ridx = ceil(rand * permuteSize * permuteSize);
    
    % Calculate change in yaw
    dtheta = rand () * yawVariance - yawVariance/2;
    newSample(3) = rem(newSample(3) + dtheta, pi);
    
    % Calculate change in x,y location
    [ dy dx ] = ind2sub ( [ permuteSize permuteSize ], ridx );
    newSample(1:2) = newSample(1:2) + [ dx-2 dy-2 ];
    newSample(1) = max(1,newSample(1));
    newSample(1) = min(gridSize(2),newSample(1));
    newSample(2) = max(1,newSample(1));
    newSample(2) = min(gridSize(1),newSample(2));
    
    % Make sure the new location is viable:
    if (~worldState.obstacle_map(newSample(2),newSample(1)))
        samples(x,:) = newSample;
    end;
    
end;
