function [varargout] = stopBot (varargin)
% [] = stopBot ()
% [] = stopBot ('non-blocking');
% [] = stopBot ('blocking');
% [odom rPose rGridPos] = stopBot ('blocking', 'doSlam');
%
%   Issues a powerbot('turnbot',0) command to the robot, then optionally blocks 
% while waiting for the command to complete. Finally, we'll doSlam() in order to 
% sync up the mapping module. The blocking and doSlam functionality is optional.
%   The default parameterless operation of this function is 'non-blocking'.
%
% odom := refreshed odometry value that was passed to the slam module
% rPose := refreshed rPose after filtering
% rGridPos := refreshed rGridPos after filtering

varargout = cell(1,nargout);

% To stop, issue a turnbot command with a 0 angle.
odom = powerbot('getodom');
powerbot('movebotTo', odom);

if nargin >= 1 && strcmp(varargin{1},'blocking')
    
    % Block on this output
    speeds = powerbot('getspeeds');
    while abs(speeds(3)) > 0.1
        speeds = powerbot('getspeeds');
    end;
    
    % Do SLAM at this point, since we know we're fully stopped:
    if nargin == 2 && strcmp(varargin{2},'doSlam')
        if nargout == 3
            [odom rPose rGridPos] = doSlam(1,1);
            varargout{1} = odom;
            varargout{2} = rPose;
            varargout{3} = rGridPos;
        else
            fprintf('[stopBot.m] Error: if you call this method with ''doSlam'', you must provide 3 output arguments.\n');
        end;
    end;
    
end;

