function [ desiredOdomYaw varargout] = turnBot(radians, varargin)
% [ desiredOdomYaw odom rPose rGridPos] = turnbot ( radians )
% [ desiredOdomYaw odom ] = turnbot ( radians, 'odomonly' )
% [ desiredOdomYaw odom rPose rGridPos] = turnbot ( radians, 'slam')
% [ desiredOdomYaw ] = turnbot ( radians, 'noslam', odom, rPose, rGridPos)
%
% Non-blocking call to turn the robot by a given number of radians, 
% relative to current orientation. 
% 
% When 'slam' is specified with this method, doSlam(1,1) is called beforehand
% and the pose reflects the pre-turn pose.
%
% When 'odomonly' is specified with this method, only powerbot('getodom')
% is called before 
%
% radians := the relative radians to turn
% odom := odom pose of the robot
% rPose := slam pose of the robot
% rGridPos := slam grid position of the robot

if nargin >= 2 && strcmp(varargin{1}, 'noslam')
    % Don't do slam, read poses from arguments:
    if nargin == 5
        odom = varargin{1};
        rPose = varargin{2};   % unused, but pass anyway
        rGridPos = varargin{3}; % unused, but pass anyway
    else
        fprintf('[turnBot.m] Error: You must provide 3 input arguments if you run this method with the ''noslam'' option.\n');
        return;
    end;
else
    % Do slam right away:
    [odom rPose rGridPos] = doSlam(1,1);
    if nargout == 4
        varargout{1} = odom;
        varargout{2} = rPose;
        varargout{3} = rGridPos;
    else
        fprintf('[turnBot.m] Error: You must provide 3 output arguments if you run this method with the ''slam'' option.\n');
    end;
end;


% Calculate the odom coordinates of the desired yaw and position
% Call these yawOdom and vecOdom:
thetaOdom = thetapp( radians + odom(3) );
destOdom = [ odom(1) odom(2) thetaOdom ];
desiredOdomYaw = thetaOdom;

% Issue the non-blocking call:
powerbot('movebotTo', destOdom);

