#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <stdio.h>
#include <string>
#include <algorithm>
#include <sstream>
#include <ctype.h>

/*===========================================================================
** Point Grey Research Includes
**=========================================================================*/
#include "triclops.h"
#include "pnmutils.h"

#include "FPSProfiler.h"

// MSER related includes
#include "libExtrema.h"
#include <optionGM.h>
#include <optionPriv.h>
#include <imageread.h>
#include <timeutls.h>
#include <error.h>
#include <assert.h>
#include <math.h>
#include <semaphore.h>


#include <ary.h>
#include <matrix.h>
#include <timeutls.h>  

#include "shared_memory.h"

#include "preprocess.h"
#include "suballoc.h"
#include "libExtrema.h"
#include "sortPixels.h"
#include "optThresh.h"
#include "getExtrema.h"
#include "boundary.h"
using namespace utls;
using namespace std;
using namespace extrema;
// end saliency includes

#include "bumblebee.h"

#include "match.cc"

//#define MSER_DEBUG

#define MAX_DISP_SIZE_RATIO 0.1		


void imshow(IplImage* image, char* window_name = NULL)
{
    static int num = 0;
    char name[20];
    if (window_name!=NULL) {
        strcpy(name,window_name);
    } else {
        sprintf(name,"Window %d",num++);
    }
    
    cvNamedWindow(name, CV_WINDOW_AUTOSIZE); 
    
    IplImage* img = cvCreateImage( cvGetSize(image), IPL_DEPTH_8U, image->nChannels);
    cvConvertScale(image,img,1,0);
    
    cvShowImage(name, img );
    cvReleaseImage(&img);
}

void imagesc(IplImage* image, char* window_name = NULL)
{
    static int num = 0;
    char name[20];
    if (window_name!=NULL) {
        strcpy(name,window_name);
    } else {
        sprintf(name,"Window %d",num++);
        }
    
    cvNamedWindow(name, CV_WINDOW_AUTOSIZE); 
    
    double min, max;
    cvMinMaxLoc(image,&min,&max);

    IplImage* img = cvCreateImage( cvGetSize(image), IPL_DEPTH_8U, image->nChannels);
    cvScale(image,img,255/(max-min),min/(min-max));
    cvShowImage(name, img );
    cvReleaseImage(&img);
}

void save_mat(char* name, IplImage* src)
{
    FILE *f = fopen(name,"w");
    
    float* srcPix = (float*)src->imageData;
    
    for (int i=0;i<src->height;++i) {
        for (int j=0;j<src->width;++j) {
            fprintf(f,"%f ",*srcPix);
            srcPix++;
        }
        fprintf(f,"\n");
    }
    fclose(f);
}

void save_mat_(char* name, CvMat* src)
{
    FILE *f = fopen(name,"w");
    
    float* srcPix = src->data.fl;
    
    for (int i=0;i<src->height;++i) {
        for (int j=0;j<src->width;++j) {
            fprintf(f,"%f ",*srcPix);
            srcPix++;
        }
        fprintf(f,"\n");
    }
    fclose(f);
}

class ImagePyramid {
	IplImage** imp;
	int levels;
public:

	ImagePyramid(int width, int height, int levels_, int depth = IPL_DEPTH_32F, int channels = 3) : levels(levels_)
	{
		imp = new IplImage*[levels];
		
// 		width /= 2;
// 		height /= 2;
		for (int i=0;i<levels;++i) {
			imp[i] = cvCreateImage(cvSize(width, height), depth, channels);
			width /= 2;
			height /= 2;
		}
	}
	
	void initialize(IplImage *src) 
    {
        assert(src->width/2 == imp[1]->width);
        assert(src->height/2 == imp[1]->height);
        assert(src->nChannels == imp[1]->nChannels);
        
        cvConvert(src, imp[0]);
        for (int i=1;i<levels;++i) {
            cvPyrDown(imp[i-1], imp[i], CV_GAUSSIAN_5x5);
        }
    }
    
    IplImage* operator[](int index)
    {
        return imp[index];
    }
    ~ImagePyramid()
    {	
	for (int i=0;i<levels;++i) 
	{
           cvReleaseImage(&(imp[i]));
        }
	
    }				

};


CvMat* createDiskFilter(int size) 
{
	int n = 2*size+1;
	CvMat* filter= cvCreateMat( 2*size+1, 2*size+1, CV_32FC1 );
	cvZero(filter);
	float sum = 0;
	
	for(int i=0;i<n;++i) {
		for(int j=0;j<n;++j) {
			if ( (i-size)*(i-size)+(j-size)*(j-size)<=size*size) {
				filter->data.fl[i*n+j] = 1;
				sum+=1;
			}
		}
	}
	
	cvConvertScale(filter, filter, 1.0/sum,0);
	    
	return filter;
}

void image_spectral_saliency(IplImage* src, IplImage* dst)
{
    CvMat * real;
    CvMat * imaginary;
    CvMat * dft;
    CvMat tmp;
    
    int dft_width = cvGetOptimalDFTSize( src->width - 1 );
    int dft_height = cvGetOptimalDFTSize( src->height - 1 );

    real = cvCreateMat( dft_height, dft_width, CV_32FC1 );
    imaginary = cvCreateMat( dft_height, dft_width, CV_32FC1 );
    dft = cvCreateMat( dft_height, dft_width, CV_32FC2 );
	
    cvGetSubRect( real, &tmp, cvRect(0,0, src->width, src->height));
    cvCopy( src, &tmp, NULL );
    cvZero(imaginary);
    
    cvMerge(real, imaginary, NULL, NULL, dft);
    // do the fft
    cvDFT( dft, dft, CV_DXT_FORWARD, src->height );
    cvSplit(dft, real, imaginary, 0, 0);
    
    
    cvCartToPolar(real, imaginary, real, imaginary,0);
    cvLog(real,real);
    CvMat* filtered = cvCreateMat(dft_height, dft_width, CV_32FC1 );
    cvCopy(real,filtered);
    cvSmooth(filtered,filtered, CV_BLUR);
    
    cvSub(real,filtered,real, NULL);
    cvExp(real,real);
    cvPolarToCart(real, imaginary, real, imaginary,0);
    
    // do inverse fourier transform
    cvMerge(real, imaginary, NULL, NULL, dft);
    cvDFT( dft, dft, CV_DXT_INV_SCALE, src->height );
    cvSplit(dft, real, imaginary, 0, 0);

    // get magnitude
    cvCartToPolar(real, imaginary, real, NULL,0);
    cvPow(real,real,2.0);
    
    
    static CvMat* disk = NULL;
    
    int filter_rad = 3;
//     if (disk == NULL) {
//         disk = createDiskFilter(filter_rad);
//     }
    CvMat* sfiltered = cvCreateMat( real->height+filter_rad*2, real->width+filter_rad*2, CV_32FC1 );
    cvCopyMakeBorder(real, sfiltered, cvPoint(3,3),IPL_BORDER_CONSTANT);
    //cvFilter2D(sfiltered,sfiltered,disk);
    cvSmooth(sfiltered,sfiltered, CV_GAUSSIAN, 7);
    
    double min, max;
    cvMinMaxLoc(sfiltered,&min,&max);
    cvScale(sfiltered,sfiltered,1/(max-min),min/(min-max));
    
    // copy result to output image
    cvGetSubRect( sfiltered, &tmp, cvRect(3,3, src->width, src->height));
    cvCopy( &tmp, dst, NULL );
 
    cvReleaseMat(&sfiltered);
    cvReleaseMat(&real);
    cvReleaseMat(&filtered);
    cvReleaseMat(&imaginary);
    cvReleaseMat(&dft);
}


// PRECONDITION: src image is a 3 channel float image
void spectral_saliency(IplImage* src, IplImage* dst)
{
    float projections[][3] = { { 1.0/3, 1.0/3, 1.0/3},  // gray
                                { 1, -1, 0},            // r-g
                                {1.0/2, 1.0/2, -1} };   // y-b

	IplImage* c1 = cvCreateImage(cvGetSize(src), IPL_DEPTH_32F, 1);
	IplImage* c2 = cvCreateImage(cvGetSize(src), IPL_DEPTH_32F, 1);
	IplImage* c3 = cvCreateImage(cvGetSize(src), IPL_DEPTH_32F, 1);

    cvZero(dst);
    
    for (int i=0;i<3;++i) {
        cvSplit(src,c1,c2,c3,0);
        cvAddWeighted(c1,projections[i][0],c2,projections[i][1],0,c1);
        cvAddWeighted(c1,1,c3,projections[i][2],0,c1);
            
        image_spectral_saliency(c1,c1);
        
        cvAdd(dst,c1,dst);
    }
    cvScale(dst,dst,1.0/3);
	
	cvReleaseImage(&c1);
	cvReleaseImage(&c2);
	cvReleaseImage(&c3);	
}

class FrameCapture
{
public:
   virtual IplImage* captureFrame() = 0;
};


class CamFrameCapture : public FrameCapture
{
public:
   CamFrameCapture(int device)
   {
      capture = cvCaptureFromCAM(device);
      if( !capture )
      {
         perror("Could not initialize capturing...\n");
         exit(1);
      }
   }
   
   ~CamFrameCapture()
   {
      cvReleaseCapture(&capture);
   }
   
   virtual IplImage* captureFrame()
   {
      return cvQueryFrame(capture);
   }

private:
   CvCapture* capture;
};


class FileFrameCapture : public FrameCapture
{
public:
   FileFrameCapture(char* name, int width_, int height_, int depth_ = IPL_DEPTH_8U, int channels_ = 3) 
      : width(width_), height(height_), depth(depth_), channels(channels_)
   {
      input = fopen(name,"r");
      
      if (input == NULL) {
         perror("Cannot open input file");
      }
      
      frame = cvCreateImage(cvSize(width,height), depth, channels);
   }
   
   ~FileFrameCapture()
   {
      fclose(input);
      cvReleaseImage(&frame);
   }
   
   virtual IplImage* captureFrame()
   {
      if (fread( frame->imageData, frame->width*frame->height*channels, 1, input )==1) {
         return frame;
      }
      else {
         return NULL;
      }
   }

private:
   FILE* input;
   int width;
   int height;
   int depth;
   int channels;
   IplImage* frame;
};

class PtGrey
{
public:

   TriclopsContext      triclops;
   TriclopsError        te;
   TriclopsCamera       field;
   TriclopsInput        triclopsInput; 

   static char default_filename[]; 
    
    PtGrey( int width, int height )
    {
        field = TriCam_REFERENCE;
        /* Fill in stuct fields */
        triclopsInput.nrows=height;
        triclopsInput.ncols=width;
        triclopsInput.inputType=TriInp_RGB;
        triclopsInput.rowinc=width;   

	char* SRVC_ROOT = getenv( "SRVC_ROOT" );
	char filename[1000];

	sprintf( filename, "%s/%s", SRVC_ROOT, default_filename ); 
            
        /* extract Triclops context from file */
        te =  triclopsGetDefaultContextFromFile( &triclops, filename);
        if ( te != TriclopsErrorOk ) {
            perror("TriclopsError: GetDefaultContextFromFile");
            exit(1); 
        }

        /* Over-ride the resolution set in the config file. */  
        triclopsSetResolution(triclops,height,width);
    }

    ~PtGrey()
    {
        //freeInput(&triclopsInput);
    }
};   


char PtGrey::default_filename[] = "ROBOT/main/config/bb_id5020143.config";

class ImageRectifier : public PtGrey
{
    TriclopsColorImage   colorImage;
    
    IplImage*   red;
    IplImage*   green;
    IplImage*   blue;
    
    IplImage* out_red;
    IplImage* out_green;
    IplImage* out_blue;

public:
    ImageRectifier(int width, int height) : PtGrey( width, height )
    {
        red = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
        green = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
        blue = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
              
        out_red = cvCreateImageHeader(cvSize(width,height),IPL_DEPTH_8U,1);
        out_green = cvCreateImageHeader(cvSize(width,height),IPL_DEPTH_8U,1);
        out_blue = cvCreateImageHeader(cvSize(width,height),IPL_DEPTH_8U,1);
    }
    
    ~ImageRectifier()
    {
        cvReleaseImage(&red);
        cvReleaseImage(&green);
        cvReleaseImage(&blue);
        
        cvReleaseImageHeader(&out_red);
        cvReleaseImageHeader(&out_green);
        cvReleaseImageHeader(&out_blue);
    }
    
    void operator()(IplImage* src, IplImage* dst, int iField=0)
    {
        switch(iField)
        {
          case 0:
            field = TriCam_LEFT;
            break;
          case 1:
            field = TriCam_RIGHT;
            break;
          default:
            printf("Invalid field passed to ImageRectifier::operator().\n");
            field = TriCam_REFERENCE;
        }
          
        cvSplit( src, red, green, blue,0);
    
        triclopsInput.u.rgb.red   = red->imageData;
        triclopsInput.u.rgb.green = green->imageData;
        triclopsInput.u.rgb.blue  = blue->imageData;
        
        /* The actual rectification call */    
        triclopsRectifyColorImage( triclops, field, &triclopsInput, &colorImage );
    
        out_red->imageData = (char*)colorImage.red;
        out_green->imageData = (char*)colorImage.green;
        out_blue->imageData = (char*)colorImage.blue;
        
        cvMerge( out_red, out_green, out_blue, 0, dst );
    }
};

class Point
{
public:

    int imR, imC;
    short unsigned disparity;
    float X, Y, Z;

    Point() : imR(0), imC(0), disparity(0xFFFF),X(0.0),Y(0.0),Z(0.0)
    {
    }

    void Transform( float** T )
    {
        if( disparity != 0xFFFF )
        {
            float X_new = T[0][0]*X + T[0][1]*Y + T[0][2]*Z + T[0][3];
            float Y_new = T[1][0]*X + T[1][1]*Y + T[1][2]*Z + T[1][3];
            float Z_new = T[2][0]*X + T[2][1]*Y + T[2][2]*Z + T[2][3];  
            X = X_new;
            Y = Y_new;
            Z = Z_new;
        }
    }
   
    void setCoords(float X_new, float Y_new, float Z_new)
    {
    	X = X_new;
        Y = Y_new;
        Z = Z_new;

    }		
};

class Centroid
{
public:

    float avg_disparity;
    float range_depth;
    float X, Y, Z;
    float X_r, Y_r, Z_r;	

    Centroid() : avg_disparity(0), range_depth(0), X_r(0.0), Y_r(0.0), Z_r(0.0),  X(0.0), Y(0.0), Z(0.0)
    {
    }

    void Transform( float** T )
    {
        if( avg_disparity != 0xFFFF )
        {
            float X_new = T[0][0]*X + T[0][1]*Y + T[0][2]*Z + T[0][3];
            float Y_new = T[1][0]*X + T[1][1]*Y + T[1][2]*Z + T[1][3];
            float Z_new = T[2][0]*X + T[2][1]*Y + T[2][2]*Z + T[2][3];
            X_r = X_new;
            Y_r = Y_new;
            Z_r = Z_new;
        }
    }

    void setCoords(float X_new, float Y_new, float Z_new, float avg_disparity_new, float range_depth_new)
    {
        X = X_new;
        Y = Y_new;
        Z = Z_new;
	avg_disparity = avg_disparity_new;
	range_depth = range_depth_new;
    }
};

class TransMatrix
{
public:

    float** T;
    float tilt_angle;	
    TransMatrix(float aTilt): tilt_angle(aTilt)
    {	
	// Construct tranformation matrix
	float tilt = tilt_angle * PI / 180.0;  
	T = new float*[4];
        for( int i=0; i<4; ++i )
	{
        	T[i]=new float[4];
	}
    	T[0][0] = 0.0;
    	T[0][1] = -sin(tilt);;
    	T[0][2] = cos(tilt);
    	T[0][3] = POSITION_X_OFFSET;
    	T[1][0] = -1.0;
    	T[1][1] = 0.0;
    	T[1][2] = 0.0;
    	T[1][3] = 0.0;
    	T[2][0] = 0.0;
    	T[2][1] = -cos(tilt);
    	T[2][2] = -sin(tilt);
    	T[2][3] = CAMERA_HEIGHT;
    	T[3][0] = 0.0;
    	T[3][1] = 0.0;
    	T[3][2] = 0.0;
    	T[3][3] = 1.0;
    }
    	
    ~TransMatrix()
    {
        if( T != NULL )
        {
        	for( int i=0; i<4; ++i )
        		delete[] T[i];
        	delete[] T;
	}
    }
	
};

class PointCloud
{
public:

    Point* tempP;
    Point** points;
    int width, height;

    PointCloud( int aWidth, int aHeight ) : width(aWidth), height(aHeight)
    {
        tempP = NULL;
        points = NULL;

        tempP = new Point[aWidth*aHeight];
        points = new Point*[aHeight];        

        for( int r=0; r<aHeight; ++r ){

            points[r] = tempP + r*aWidth;

            for( int c=0; c<aWidth; ++c ){
                points[r][c].imR = r;
                points[r][c].imC = c;
            }
        }
    }
    void Copy(PointCloud *pc_new)
    {   for( int r=0; r<height; r++ )
	{   
            for( int c=0; c<width; c++ )
	    {
                pc_new->points[r][c].imR = points[r][c].imR;
                pc_new->points[r][c].imC = points[r][c].imC;
                pc_new->points[r][c].X = points[r][c].X;
                pc_new->points[r][c].Y = points[r][c].Y;
                pc_new->points[r][c].Z = points[r][c].Z;
                pc_new->points[r][c].disparity = points[r][c].disparity;

            }
        }
    }			

    void Transform( float** T  )
    {
        for( int r=0; r<height; ++r )
        {
            for( int c=0; c<width; ++c )
            {
                if( points[r][c].disparity != 0xFFFF )
                {
                    points[r][c].Transform( T );
                }
            }
        }
    }
//Pooja: Added argument binary for binary vs. filtered depth maps - June 7 08  
    void ThresholdHeight( float threshold, IplImage *dst, int binary )
    {
        for( int r=0; r<dst->height; ++r )
        {
            for( int c=0; c<dst->width; ++c )
            {
                if( points[r][c].disparity < 0xFFFF )//&& points[r][c].Z != 0.0 )
                {
                    //printf( "Height %f.\n", points[r][c].Z);

                    if( points[r][c].Z > threshold )
                    {
                        if(binary==1)
			{
				cvSet2D( dst, r, c, cvRealScalar(255.0) );
			}
			else
			{	
				
				//printf( "The value of pixel %d, %d is %d.\n", r, c,  points[r][c].disparity  );
				//cvSet2D( dst, r, c, cvRealScalar(255.0) );
				cvSet2D( dst, r, c, cvRealScalar(points[r][c].disparity) );

			}
                    }

                    //CvScalar val = cvGet2D( dst, r, c );
                    //printf( "The value of pixel %d, %d is %d.\n", r, c, (int)(val.val[0]) );
                }
            }
        }
    }

    ~PointCloud()
    {
        if( tempP != NULL )
        {
            delete tempP;
            tempP = NULL;
        }
        
        if( points != NULL )
        {
            delete points;
            points = NULL;
        }
    }
};

class StereoProcessing : public PtGrey
{
    TriclopsImage16 depthImage16;   
    TriclopsImage depthImage; 

    IplImage*   red;
    IplImage*   green;
    IplImage*   blue;
    
public:
    StereoProcessing(int width, int height) : PtGrey( width, height )
    {
        red = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
        green = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
        blue = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);

        triclopsSetSubpixelInterpolation( triclops, 1 ) ;
        triclopsSetResolutionAndPrepare( triclops, stereo_height, stereo_width, width, height);

        // Configuration settings recommended by Pooja
        te = triclopsSetTextureValidationThreshold( triclops, 1.1 );
        te = triclopsSetUniquenessValidationThreshold( triclops, 0.8 );
        //triclopsSetDisparity( triclops, 0, 768 );
        
        //te = triclopsSetDisparityMapping( triclops, 128, 255 );
        //te = triclopsSetDisparityMappingOn( triclops, 1 );

    }

    ~StereoProcessing()
    {   cvReleaseImage(&red);
        cvReleaseImage(&green);
        cvReleaseImage(&blue);

    }

    void operator()(IplImage* src, IplImage* dst, PointCloud *pc )
    {
        cvSplit( src, red, green, blue,0);
    
        triclopsInput.u.rgb.red   = red->imageData;
        triclopsInput.u.rgb.green = green->imageData;
        triclopsInput.u.rgb.blue  = blue->imageData;

        te = triclopsPreprocess( triclops, &triclopsInput );
        if ( te != TriclopsErrorOk ) {
            printf("TriclopsError: triclopsPreprocess.\n");
            return;
        }

        te = triclopsStereo( triclops );
        if ( te != TriclopsErrorOk ) {
            printf("TriclopsError: triclopsStereo.\n");
            return;
        }

        triclopsGetImage16( triclops, TriImg16_DISPARITY, TriCam_REFERENCE, &depthImage16 );
        dst->imageData = (char*)depthImage16.data;

        //triclopsGetImage( triclops, TriImg_DISPARITY, TriCam_REFERENCE, &depthImage );

        //triclopsSaveImage( &depthImage, "depth.pgm" );
        //triclopsSaveImage16( &depthImage16, "depth.pgm" );
        unsigned short* disparityPixel = depthImage16.data;

        for( int row=0; row<dst->height; ++row ){
            for( int col=0; col<dst->width; ++col ){

                Point* p = &(pc->points[row][col]);

                if ( *disparityPixel < 0xFF00 && *disparityPixel > 1536 )
                {
                    p->disparity = *disparityPixel;
                
                    te = triclopsRCD16ToXYZ( triclops, row, col, *disparityPixel, &(p->X), &(p->Y), &(p->Z) );
                    if( te != TriclopsErrorOk )
                    {
                        printf("TriclopsError: triclopsRCD16ToXYZ\n");
                    }
                }
                else
                {
                    p->disparity=0xFFFF;
                }  

                disparityPixel++;  
            }
        }
    }
};

void displayShortImage( IplImage* src, IplImage* dst )
{
    unsigned short *srcPix = (unsigned short*)src->imageData;
    unsigned char *dstPix = (unsigned char*)dst->imageData;

    double tempData;
    double min = 0xFFFF;
    double max = 0;

    for( int p=0; p<src->width*src->height*src->nChannels; ++p,++srcPix,++dstPix )
    {
        if( *srcPix <= 64257 )
        {
            tempData = *srcPix;

            if( tempData > max )
            {
                max = tempData;
            }
        
            if( tempData < min )
            {
                min = tempData;
            }
        }
    }

    //printf( "Max = %f. Min = %f.\n", max, min );
    srcPix = (unsigned short*)src->imageData;
    dstPix = (unsigned char*)dst->imageData;

    for( int p=0; p<src->width*src->height*src->nChannels; ++p,++srcPix,++dstPix )
    {
        if( *srcPix >=  0xFF00 || *srcPix == 0 )
        {
            *dstPix = 0;
        }
        else
        {
            tempData = *srcPix;
//             printf( "Max %f, min %f, orig %f, ", max, min, tempData );
            tempData = ((tempData - min) / (max - min) * 128.0)+128.0;  
//             printf( "scaled %f, ", tempData );
            *dstPix = (unsigned char)ceil(tempData);
//             printf( "final %d.\n", *dstPix );
        }
    }
}


void cvClearBorder( int border, IplImage *src )
{
    float *srcPix = (float*)src->imageData;
    for( int r=0; r<src->height; ++r )
    {
        for (int b = 0;b<border;++b) {
            srcPix[r*src->width+b] = 0;
            srcPix[r*src->width+src->width - b-1] = 0;
        }
    }
    
    for (int b = 0;b<border;++b) {
        for( int c=0; c<src->width; ++c )
        {
            srcPix[b*src->width+c] = 0;
            srcPix[(src->height-b-1)*src->width+c] = 0;
        }
    }
}

class BBCorner
{
public:
    unsigned int x, y;

    BBCorner( unsigned int _x, unsigned int _y ) : x(_x), y(_y) {};
};

class BoundingBox
{
public:
    BBCorner c_min;
    BBCorner c_max;

    BoundingBox() : c_min( 0xFFFFFFFF, 0xFFFFFFFF ), c_max( 0, 0 ) {}

    unsigned int width()
    {
        return (c_max.x - c_min.x);
    }

    unsigned int height()
    {
        return  (c_max.y - c_min.y);
    }

    unsigned int area()
    {
        return width() * height();
    }

	BBCorner centre()
	{
		BBCorner c( (c_min.x + c_max.x) / 2, (c_min.y + c_max.y) / 2 );
		return c;
	}

    void grow( unsigned x, unsigned y )
    {
        if( c_min.x > x ){ c_min.x = x; }
        if( c_min.y > y ){ c_min.y = y; }
        if( c_max.x < x ){ c_max.x = x; }
        if( c_max.y < y ){ c_max.y = y; }
    }

    void fitBoundaryRegion( const BoundaryRegion *r )
    {
        for( int j=0; j<r->boundary.size(); j++)
            grow( r->boundary[j].col, r->boundary[j].line );
    }

    unsigned int box_intersection( BoundingBox &other )
    {
        unsigned int x_inter, y_inter;

        x_inter = (unsigned int)max( 0, (int)(min( c_max.x, other.c_max.x ) - max( c_min.x, other.c_min.x )) );
        y_inter = (unsigned int)max( 0, (int)(min( c_max.y, other.c_max.y ) - max( c_min.y, other.c_min.y )) );
 
        return x_inter * y_inter;
    }

    unsigned int box_union( BoundingBox &other )
    {
        unsigned int x_union, y_union;

        x_union = max( c_max.x, other.c_max.x ) - min( c_min.x, other.c_min.x );
        y_union = max( c_max.y, other.c_max.y ) - min( c_min.y, other.c_min.y );

        return x_union * y_union;
    }

    void drawBB( IplImage *img, CvScalar color = CV_RGB(0,0,255)) const
    {
/*        int x, y;
        CvScalar value = cvRealScalar(255.0);
        
        for( x=c_min.x, y=c_min.y; x<=c_max.x; x++ ) cvSet2D( img, y, x, value );
        for( x=c_min.x, y=c_min.y; y<=c_max.y; y++ ) cvSet2D( img, y, x, value );
        for( x=c_max.x, y=c_min.y; y<=c_max.y; y++ ) cvSet2D( img, y, x, value );
        for( x=c_min.x, y=c_max.y; x<=c_max.x; x++ ) cvSet2D( img, y, x, value );*/
        
        cvRectangle(img, cvPoint(c_min.x,c_min.y),cvPoint(c_max.x,c_max.y), color);
    }
};

class PotentialObject
{
public:

    BoundingBox bbox;
    const BoundaryRegion *boundary;
    const RLERegion *rle;
    int average;
    Centroid centre;	

    PotentialObject( BoundingBox _bbox, const BoundaryRegion *_boundary, const RLERegion* _rle, int _average, Centroid _centre ) :
        bbox( _bbox ),
        boundary( _boundary ),
        rle( _rle ),
        average( _average ),
	centre(_centre)
    {}

    bool operator<( const PotentialObject &other ) const
    {
        return average > other.average;
    }
};

class MserWrapper
{
public:
    ExtremaParams p;
    ExtremaImage im;
	vector<PotentialObject> obj_vec;

    MserWrapper( int width, int height, int channels )
    {
        p.preprocess = 0;
        p.max_area = 0.5;
        p.min_size = 10;
        p.min_margin = 5;
        p.relative = 0;
        p.verbose = 0;
        p.debug = 0;
        im.width = width;
        im.height = height;
        im.channels = channels;

        if (im.channels<3)
            p.preprocess = PREPROCESS_CHANNEL_none;
         else
            p.preprocess = PREPROCESS_CHANNEL_intensity;
    }

    void operator()( IplImage *f_input, PointCloud *pc, TransMatrix *tm, IplImage* floorMask, IplImage *output, int sal)

    { 
        RLEExtrema result;
        BoundaryExtrema boundaryResult;
        vector<BoundingBox> object_boxes;
	vector<Point> centroids;

        unsigned int area_int, area_union;

        double Imin, Imax;
        cvMinMaxLoc(f_input,&Imin,&Imax);

        IplImage* input = cvCreateImage( cvGetSize(f_input), IPL_DEPTH_8U, f_input->nChannels);
	cvScale(f_input,input,255/(Imax-Imin),Imin/(Imin-Imax));
	cvCopy( input, output );
	CvScalar iMean = cvAvg( input );
        CvScalar pixelVal;
	CvScalar depthVal;

        #ifdef MSER_DEBUG
        printf( "Image mean is %g.\n", iMean.val[0] );
        #endif

        const double factor = 2.0;

        if( input->width != im.width || input->height != im.height || input->nChannels != im.channels )
        {
            printf( "Incorrect image type passed to mser!\n" );
            return;
        }

        im.data = (unsigned char*)input->imageData;

        result = getRLEExtrema(p, im);
        boundaryResult = getBoundaryExtrema(p, im);

        #ifdef MSER_DEBUG
        printf("Total MSER+ %d, MSER- %d detected.\n",
                   result.MSERplus.size(),
                   result.MSERmin.size() );
        #endif

//         CvScalar value = cvRealScalar(255.0);
// 
//         for(int i=0; i < result.MSERmin.size(); i++)
//         { 
//             RLERegion *r = &(result.MSERmin)[i];
// 
//             double barX, barY, sumX2, sumY2, sumXY;
//             RLE2Ellipse(r->rle, barX, barY, sumX2, sumXY, sumY2);
//             Matrix2 C(sumX2, sumXY, sumXY, sumY2);
//             Matrix2 U, T, A;
//             C=C*(factor*factor);
//             A = C.inv();
// 
//             for( int k=0; k < r->rle.size(); ++k )
//             {
//                 for( int col = r->rle[k].col1; col < r->rle[k].col2; ++col )
//                 {
//                     //cvSet2D( output, r->rle[k].line, col, value );
// 
//                 }
//             }
//         }
        
        bool keepBB;
	int numChopped=0;
        for(int i=0; i < boundaryResult.MSERmin.size(); i++)
        {
            const BoundaryRegion *r = &(boundaryResult.MSERmin[i]);
            const RLERegion *rle = &(result.MSERmin)[i];
            int count=0;
	    int countValid = 0;	
            int average=0;
            float fgRatio=0.0;
            float averageX=0;
	    float averageY=0;
	    float averageZ=0;
	    float averageDisp=0;
	    float rangeDepth;
	    float minDepth = 1.0e38;
	    float maxDepth = 1.0e-45;
            for( int j=0; j<rle->rle.size(); j++ )
            {
                int line = rle->rle[j].line;
                for( int col=rle->rle[j].col1; col<=rle->rle[j].col2; col++ )
                {
                    count++;
                    pixelVal = cvGet2D( input, line, col );
                    average += (int)(pixelVal.val[0]);

                    pixelVal = cvGet2D( floorMask, line, col );
                    fgRatio += pixelVal.val[0] / 255.0;
		
		    Point *tempP = &(pc->points[line][col]);		
		    if(tempP->disparity != 0xFFFF )
		    {	countValid++;
			//printf("Data X: %f, Data Y: %f, Data Z: %f", tempP->X, tempP->Y, tempP->Z);
			averageX += tempP->X;
		    	averageY += tempP->Y;
		    	averageZ += tempP->Z;
			averageDisp += tempP->disparity;
			minDepth = min(tempP->Z, minDepth);	
			maxDepth = max(tempP->Z, maxDepth);
			
			
		    }	
		    				
                }
            }


            average = average / count;
	    fgRatio = fgRatio / count;
            averageX = averageX / float(countValid);
            averageY = averageY / float(countValid);
            averageZ = averageZ / float(countValid);
            averageDisp = averageDisp / float(countValid);
	    rangeDepth = maxDepth - minDepth; 
	    Centroid centroid;
	    centroid.setCoords(averageX, averageY, averageZ, averageDisp, rangeDepth); 	
	    centroid.Transform(tm->T);		
            // Only count regions which are significantly salient
            //printf( "Region min: %d, region mean %d, image mean: %d.\n", r->minI, average, (int)(iMean.val[0]) );

            if( sal==1 && (average < (int)(iMean.val[0]*MSER_INTENSITY_THRESH) || fgRatio < MSER_FOREGROUND_THRESH )) 
            {
                continue;
            }
			
            BoundingBox bb;

            bb.fitBoundaryRegion( r );
            
	    //ignore small objects that are close		
	    if( sal==0 && (float(average)/float(bb.area()) > MAX_DISP_SIZE_RATIO ))
	    {	
		numChopped++;
		//printf("Regions chopped!");
            	//printf( "Region mean: %d, ", average );
            	//printf( "Region area: %d \n", bb.area() );
		continue;
	    }
	   /* 
	    if(sal==0 && bb.area()>0.25*stereo_width * stereo_height)
	    {
		 numChopped++;
                //printf("Regions chopped!");
                //printf( "Region area: %d \n", bb.area() );
                continue;
	    }	
       */
	    keepBB = true; 
	
            for( int j=0; j<obj_vec.size(); j++ )
            {
                if( (area_int = bb.box_intersection( obj_vec[j].bbox )) > 0 )
                {
                    area_union = bb.box_union( obj_vec[j].bbox );

                    if( (double)area_int/area_union > MSER_OVERLAP_THRESH )
                    {
                        keepBB = false;
                        break;
                    }
                }
            }
		
            if( keepBB )
            {
                PotentialObject po( bb, r, rle, average,centroid ); 
                obj_vec.push_back( po );
                //bb.drawBB( output );
            }
            //fprintf(fid, "%d", r->boundary.size());
            //for (int j=0; j<r->boundary.size(); j++)
            //    cvSet2D( output, r->boundary[j].line, r->boundary[j].col, value );
        }
	if(sal==0)
	{	//printf("%d Regions chopped!", numChopped);
	}
        sort( obj_vec.begin(), obj_vec.end() );

        for( int i=0; i< min(MSER_MAX_REGIONS,(int)obj_vec.size()); i++ )
        {
            //printf("X_r: %f, Y_r: %f, Z_r: %f \n",obj_vec[i].centre.X_r, obj_vec[i].centre.Y_r, obj_vec[i].centre.Z_r);
            //printf("Average: %d\n",obj_vec[i].average);
	    obj_vec[i].bbox.drawBB( output );
            //obj_vec[i].bbox.drawBB( floorMask );
        }
	cvReleaseImage(&input);
     }
};

void write_to_shared_memory(SharedMemory& shmem, IplImage* src)
{
    IplImage* h = shmem.reserve<IplImage>();
    *h = *src;
    char* imageData = shmem.reserve<char>(src->imageSize);
    memcpy(imageData, src->imageData, src->imageSize);
}

void write_objects_to_shared_memory(SharedMemory& shmem, const vector<PotentialObject>& obj_vec )
{
	int* i_ptr = shmem.reserve<int>();
	double* d_ptr = shmem.reserve<double>(12*obj_vec.size());

	*i_ptr = obj_vec.size();

	BBCorner c(0,0);
	BoundingBox bbox;

    //bbox = obj_vec[0].bbox;
    //printf( "%d objects located. First object has bbox %d,%d,%d,%d.\n", obj_vec.size(),bbox.c_min.x, bbox.c_min.y, bbox.c_max.x, bbox.c_max.y );

    for( int i=0; i<obj_vec.size(); ++i, d_ptr+=12 )
	{
		bbox = obj_vec[i].bbox;
		c = bbox.centre();
		d_ptr[0] = c.x;
		d_ptr[1] = c.y;
		d_ptr[2] = bbox.c_min.x;
		d_ptr[3] = bbox.c_min.y;
		d_ptr[4] = bbox.c_max.x;
		d_ptr[5] = bbox.c_max.y;
        d_ptr[6] = obj_vec[i].centre.X_r;
        d_ptr[7] = obj_vec[i].centre.Y_r;
        d_ptr[8] = obj_vec[i].centre.Z_r;
        d_ptr[9] = obj_vec[i].centre.X;
        d_ptr[10] = obj_vec[i].centre.Y;
        d_ptr[11] = obj_vec[i].centre.Z;
        
	}
}


void write_pc_to_shared_memory(SharedMemory& shmem, PointCloud* pc )
{
        int* i_ptr = shmem.reserve<int>();
        double* d_ptr = shmem.reserve<double>(6*pc->width*pc->height);

        *i_ptr = pc->width*pc->height;

        Point p;
    
//printf( "%d points located. First point has %d,%d,%d,%d,%d,%d.\n", p.imR, p.imC, p.X, p.Y, p.Z, p.disparity );

    for( int i=0; i<pc->height; ++i)
    {	for(int j=0; j<pc->width; ++j, d_ptr+=6)
	{
                p = pc->points[i][j];
                d_ptr[0] = p.imR;
                d_ptr[1] = p.imC;
                d_ptr[2] = p.X;
                d_ptr[3] = p.Y;
                d_ptr[4] = p.Z;
		d_ptr[5] = p.disparity; 
        }
    }	

}

template<typename T>
void write_data_to_shared_memory(SharedMemory& shmem, T data)
{
    T* ptr = shmem.reserve<T>();
    *ptr = data;
}



void save_pgm_16(char* name, IplImage* src)
{
    FILE *f = fopen(name,"w");
    fprintf(f,"P5\n");
    fprintf(f,"%d %d\n",src->width,src->height);
    fprintf(f,"65536\n");
    fwrite( src->imageData, 2*src->width*src->height, 1, f );
    fclose(f);
    
}
/*
void buildTiltTransform( float** T, float tilt )
{
    T[0][0] = 0.0;
    T[0][1] = -sin(tilt);;
    T[0][2] = cos(tilt);
    T[0][3] = POSITION_X_OFFSET;
    T[1][0] = 1.0;
    T[1][1] = 0.0;
    T[1][2] = 0.0;
    T[1][3] = 0.0;
    T[2][0] = 0.0;
    T[2][1] = -cos(tilt);
    T[2][2] = -sin(tilt);
    T[2][3] = CAMERA_HEIGHT;
    T[3][0] = 0.0;
    T[3][1] = 0.0;      
    T[3][2] = 0.0;
    T[3][3] = 1.0;
}

void doTransform(float tilt, PointCloud *pc)
{	
    // Construct tranformation matrix and apply
    float** T;
    T = new float*[4];
    for( int i=0; i<4; ++i )
        T[i]=new float[4];

    buildTiltTransform( T, tilt * PI / 180.0 );
    pc->Transform( T );

    for( int i=0; i<4; ++i )
        delete[] T[i];
    delete[] T;

} 
*/



//Pooja: Added argument binary for binary vs. filtered depth maps - June 7 08  
void objectMask(PointCloud *pc, IplImage *dst, int bin)
{
    // Initialize output
    cvSetZero( dst );

    //printf("%d", HEIGHT_THRESHOLD);
    pc->ThresholdHeight( HEIGHT_THRESHOLD, dst,bin);

}




void drawKeypoints(IplImage* img, Keypoint keys)
{
    Keypoint key = keys;
    
    while (key!=NULL) {
        cvCircle(img, cvPoint((int)(key->col*3.2),(int)(key->row*3.2)), (int)(10*key->scale), CV_RGB( 0, 255, 0 ));
        
        key = key->next;
    }
}




void run_sift_matcher(SiftMatcher& sm, const MserWrapper& mser, IplImage* img, float s)
{
    cvNamedWindow("region",1);
    if (mser.obj_vec.size()>0) {
        BoundingBox bb = mser.obj_vec[mser.obj_vec.size()-1].bbox;
        
        CvRect rect = cvRect((int)(s*bb.c_min.x), (int)(s*bb.c_min.y), (int)(s*(bb.c_max.x-bb.c_min.x)), (int)(s*(bb.c_max.y - bb.c_min.y)));        
        cvSetImageROI(img,rect);
        IplImage* region = cvCreateImage(cvSize(rect.width,rect.height), img->depth, img->nChannels);
        cvCopy(img,region);
        cvResetImageROI(img);
        cvShowImage("region",region);
        sm.test_image(region);
        
        cvReleaseImage(&region);
    }
}


const int BB_MIN_WIDTH = 10;
const int BB_MIN_HEIGHT = 10;
const int BB_MAX_ASPECT = 2.5;


bool prune_bbox(const BoundingBox& bb)
{
    int x,y,w,h;
    x = bb.c_min.x; 
    y = bb.c_min.y; 
    w = bb.c_max.x-bb.c_min.x;
    h = bb.c_max.y-bb.c_min.y;
        
    float aspect = (float) w/h;
    aspect = (aspect>1)?aspect:(1/aspect);
        
    if ( (w<BB_MIN_WIDTH) || (h<BB_MIN_HEIGHT) || (aspect>BB_MAX_ASPECT)) {
        return true;
    }
    return false;
}

const float BB_MAX_CENTER_DIST = 15;
const float BB_MAX_SLOPE_DIFF = 2;

bool approx_equal(BoundingBox a, BoundingBox b)
{
    
    pair<int,int> a_center;
    pair<int,int> b_center;
    
    a_center.first = (a.c_min.x+a.c_max.x)/2;
    a_center.second = (a.c_min.y+a.c_max.y)/2;
    b_center.first = (b.c_min.x+b.c_max.x)/2;
    b_center.second = (b.c_min.y+b.c_max.y)/2;

    float centers_dist = (a_center.first-b_center.first)*(a_center.first-b_center.first)+
            (a_center.second-b_center.second)*(a_center.second-b_center.second);
    
    bool ok = false;
    if (centers_dist<(BB_MAX_CENTER_DIST*BB_MAX_CENTER_DIST)) {
        float slope1 = fabs((float)(a.c_max.y-b.c_min.y)/(a.c_max.x-b.c_min.x));
        float slope2 = fabs((float)(b.c_max.y-a.c_min.y)/(b.c_max.x-a.c_min.x));
        
//         printf("Slope diff: %g\n",slope1-slope2);
        if (slope1-slope2<BB_MAX_SLOPE_DIFF) {
            ok = true;
        }
    }
    

    
    return ok;
}


float compute_diag(const BoundingBox& bbox)
{
    return (bbox.c_max.x-bbox.c_min.x)*(bbox.c_max.x-bbox.c_min.x)+
            (bbox.c_max.y-bbox.c_min.y)*(bbox.c_max.y-bbox.c_min.y);
}


void compute_regions_score(float& score, const vector<PotentialObject>& vec, IplImage* img, vector<PotentialObject>& nested, vector<PotentialObject>& non_nested)
{
    int n = vec.size();
    
    int points = 0;
    vector<int> ind;
    score = 0;
    for (int i=0;i<n;++i) {
        BoundingBox bb = vec[i].bbox;
        
        if (!prune_bbox(bb)) {
//             score++;
            ind.push_back(i);
        }
#ifdef DISPLAY
        else {
            bb.drawBB(img,CV_RGB(255,0,0));
        }
#endif
    }
    
//     printf("Pruned %d boxes, %d remaining\n",n-score, score);
    
//     vector<PotentialObject> final_ind;
    vector<int> nested_ind;
    nested_ind.reserve(10);
    int c_size = ind.size();
    
    for (int i=0;i<ind.size();++i) {
        vec[ind[i]].bbox.drawBB(img,CV_RGB(255,255,0));
    }

    
    vector<int> non_nested_ind;
        
    for (int i=0;i<c_size;++i) {
        int cnt = 1;
        nested_ind.clear();
        nested_ind.push_back(ind[i]);
        for (int j=i+1;j<c_size;++j) {
//             printf("Checking: %d - %d\n",ind[i],ind[j]);
            if (approx_equal(vec[ind[i]].bbox,vec[ind[j]].bbox)) {
//                 printf("Nested: %d - %d\n",ind[i],ind[j]);
                nested_ind.push_back(ind[j]);
                ind[j] = ind[c_size-1];
                j--;
                c_size--;
                cnt++;
            }
        }
        
        
        if (cnt>1) {
            int max_k;
            float max_diag = 0;
            for (int k=0;k<cnt;++k) {
                float diag_val = compute_diag(vec[nested_ind[k]].bbox);
                
                if (max_diag<diag_val) {
                    max_diag = diag_val;
                    max_k = k;
                }
             	vec[nested_ind[k]].bbox.drawBB(img,CV_RGB(255,255,255));
            }
            nested.push_back(vec[nested_ind[max_k]]);
            vec[nested_ind[max_k]].bbox.drawBB(img,CV_RGB(0,255,0));
            printf("*** Found %d same center boxes\n",cnt);
            score += cnt*5;
        }
        else {
            non_nested.push_back(vec[ind[i]]);
        }
    }
    
    
}

void save_web_images(IplImage* mserResult,IplImage* view_disparity_objects,IplImage* disparity_segmented)
{	

    	char* dir = getenv( "DISPLAY_IMAGES" );
    	//char* dir = "talkImages";
	char filename[300];

	sprintf(filename,"%s/mserResult.png",dir);			
	printf("Saving %s.\n",filename);
	cvSaveImage(filename,mserResult);

	sprintf(filename,"%s/view_disparity_objects.png",dir);			
	printf("%s.\n",filename);
	cvSaveImage(filename,view_disparity_objects);

	sprintf(filename,"%s/disparity_segmented.png",dir);			
	printf("%s.\n",filename);
	cvSaveImage(filename,disparity_segmented);	

}


int main( int argc, char** argv )
{
    FPSProfiler prof( 10 );
    int numFrames = 0;
    
    char* image_path;
    
    FrameCapture* capture = NULL;
    if( argc == 2 || (argc == 3 && strlen(argv[1]) == 1 && isdigit(argv[1][0]))) {
        capture = new CamFrameCapture(argc == 2 ? argv[1][0] - '0' : 0);
        if (argc==1) {
            image_path = argv[1];
        } else {
            image_path = argv[2];
        }
    }
    else if( argc == 3 ) {
        capture = new FileFrameCapture(argv[1], 1024, 768);
        image_path = argv[2];
    } else {
        printf("Usage: %s [image_source] images_dir\n\n",argv[0]);
        exit(1);
    }
    
    
    
    //cvNamedWindow( "Left", 1 );
    //cvNamedWindow( "Right", 1 );

#ifdef DO_STEREO
    //cvNamedWindow( "Disparity", 1 );
    //cvNamedWindow( "Floor Segmentation", 1 );
#ifdef DISPLAY
    cvNamedWindow( "Object Disparity", 1 );
#endif
#endif

#ifdef DO_SALIENCY
    //cvNamedWindow( "Saliency", 1 );
#ifdef DO_MSER
#ifdef DISPLAY
    cvNamedWindow( "Saliency Regions", 1 );
    cvNamedWindow( "Depth + Saliency Regions", 1 );
#endif
#endif // DO_MSER
#endif // DO_SALIENCY
        
    IplImage* frame = capture->captureFrame();
    
    int width = frame->width;
    int height = frame->height;
    
    int levels = 4;
    int scale = 1<<(levels-1);
		    
    printf( "Width %d, Height %d, Channels %d\n", width, height, frame->nChannels );			 
				    
    IplImage* img = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,2);
	    
    IplImage* img_1 = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
    IplImage* img_2 = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
    IplImage* img_3 = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,1);
    
    IplImage* left = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,3);
    IplImage* right = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,3);
    
    IplImage* left_rect = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,3); 
    IplImage* right_rect = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,3); 
    
    IplImage* disparity = cvCreateImage( cvSize(stereo_width, stereo_height),IPL_DEPTH_16U,1);
    IplImage* view_disparity = cvCreateImage( cvSize(stereo_width, stereo_height),IPL_DEPTH_8U,1);
    
    IplImage* left_ss = cvCreateImage(cvSize(stereo_width, stereo_height),IPL_DEPTH_8U,3); 

    IplImage* saliency = cvCreateImage(cvSize(stereo_width/scale, stereo_height/scale),IPL_DEPTH_32F,1);
    IplImage* saliency_up = cvCreateImage(cvSize(stereo_width, stereo_height),IPL_DEPTH_32F,1);

    IplImage* floorImage = cvCreateImage( cvSize( stereo_width, stereo_height), IPL_DEPTH_8U, 1 );
    IplImage* disparity_objects = cvCreateImage( cvSize( stereo_width, stereo_height), IPL_DEPTH_16U, 1 );
    IplImage* view_disparity_objects = cvCreateImage( cvSize( stereo_width, stereo_height), IPL_DEPTH_8U, 1 );
    IplImage* disparity_objects_in = cvCreateImage( cvSize( stereo_width, stereo_height), IPL_DEPTH_8U, 1 );


    ImageRectifier rectifyImage(width, height);
    StereoProcessing processStereo( width, height );
    PointCloud pc( stereo_width, stereo_height);
    PointCloud untrans_pc(stereo_width, stereo_height); //copy pc instead
    MserWrapper mser( stereo_width, stereo_height, 1);
    MserWrapper mserDepth( stereo_width, stereo_height, 1);
    IplImage *mserResult = cvCreateImage( cvSize( stereo_width, stereo_height), IPL_DEPTH_8U, 1 );
    IplImage *disparity_segmented = cvCreateImage( cvSize( stereo_width, stereo_height), IPL_DEPTH_8U, 3 );
    TransMatrix tm(PTU_TILT);	
    
    ImagePyramid image_pyramid(stereo_width,stereo_height,levels);
    
    
    float img_scale = (double)width/stereo_width;
    
#ifdef DO_SIFT_MATCH
    SiftMatcher sm;
    sm.loadFeatures(image_path); // use hardcoded part for now... testing
    sm.build_index();
#endif    
    
    SharedMemory shmem(SHM_NAME);
    shmem.create(SHM_SIZE);
    
    sem_unlink(SEM_NAME);
    sem_t* sem = sem_open(SEM_NAME,O_CREAT,S_IRWXU | S_IRWXG,1);

    if (sem==SEM_FAILED) {
        printf("Cannot create semaphore\n");
        exit(1);
    }
    
    
    int delay = 5;

    while( 1 )
    {
        usleep(500000);
        prof.tick();
    
        frame = capture->captureFrame();
        
        if (frame == NULL) break;
        
        cvSplit(frame,img_1,img_2,img_3,0);
        
        cvCvtColor(img_3,left, CV_BayerBG2RGB);
        cvCvtColor(img_2,right, CV_BayerBG2RGB);
        

#ifdef DO_STEREO
    	// do stereo        
        cvSplit(left,0,img_1,0,0);
        cvSplit(right,0,img_2,0,0);
        cvMerge(img_1,img_2,img_2,0,left_rect);
        
        processStereo( left_rect,disparity, &pc );
        pc.Copy(&untrans_pc);
	
        displayShortImage( disparity, view_disparity );
        //cvShowImage( "Disparity", view_disparity );

      
        //cvSetZero( floorImage );
	
	//doTransform( 45.0, &pc);
        pc.Transform(tm.T);
	objectMask(&pc, floorImage,1);
	objectMask(&pc, disparity_objects,0);

#endif // DO_STEREO

#ifdef DO_SALIENCY  
        // color rectify
        rectifyImage(left,left_rect,1);
//         rectifyImage(right,right_rect,0);
        
        //cvShowImage("Right", right );

        cvResize(left_rect,left_ss,CV_INTER_CUBIC);
        
  //      cvSaveImage("saliency_image.pgm",left_ss);
        
        // compute spectral saliency
        image_pyramid.initialize(left_ss);
        
        spectral_saliency(image_pyramid[levels-1],saliency);
//        save_mat("saliency.dat",saliency);   

        cvClearBorder( 2, saliency );
        cvResize(saliency,saliency_up,CV_INTER_LINEAR);
        //imagesc(saliency_up,"Saliency");
        
#ifdef DO_MSER
        cvSetZero( disparity_segmented );
        cvSetZero( mserResult );
        mser.obj_vec.clear();
        mserDepth.obj_vec.clear();
	//cvCopy( saliency_up, mserResult );

	//printf("MSER:\n");
	mser( saliency_up, &untrans_pc, &tm, floorImage, mserResult,1 );
#ifdef DISPLAY
        cvShowImage( "Saliency Regions", mserResult );
#endif
   	//printf("Disparity:\n");
        cvCopy(left_ss,disparity_segmented);
        displayShortImage( disparity_objects, disparity_objects_in );
	mserDepth(disparity_objects_in, &untrans_pc, &tm, floorImage, view_disparity_objects,0);
// #ifdef DISPLAY
        //cvShowImage( "Depth Regions", view_disparity_objects );
// #endif

#endif // DO_MSER
#endif // DO_SALIENCY

        //cvShowImage( "Floor Segmentation", floorImage );
#ifdef DISPLAY
	cvShowImage( "Object Disparity", view_disparity_objects );
#endif
/*        cvSplit(left_rect,img_1,img_2,img_3,0);
        cvZero(left_gray);

        cvAddWeighted(img_1,1.0/3,img_2,1.0/3,0,left_gray);
        cvAddWeighted(left_gray,1.0,img_3,1.0/3,0,left_gray);*/
        
/*        cvSaveImage("left_rect.pgm",left_rect);
        cvSaveImage("left_gray.pgm",left_gray);*/
        
        
        
        
        //cvShowImage("Left", left_rect );
        //cvShowImage("Left", left_ss );
        
/*        
         cvWaitKey(0);
         break;*/
        
//         cvShowImage( "Saliency", image_pyramid[3] );

        
        //cvSaveImage("sftp://dpmeger@lapinkulta.cs.ubc.ca/.autofs/homes/ubccshome/d/dpmeger/World/test/left.pgm",left_rect);
        //cvSaveImage("left.jpg",view_disparity);
        //system( "scp left.jpg ${CS}:World/test/" );
/*        cvSaveImage("right.pgm",right);
        cvSaveImage("left_rect.pgm",left_rect);
        cvSaveImage("right_rect.pgm",right_rect);
        cvScale(disparity,view_disparity,1.0/255);
        displayShortImage( disparity, view_disparity );
        cvSaveImage("disparity.pgm",view_disparity);
        save_mat("disparity.dat",disparity);
        break;*/


//         save_pgm_16("disparity.pgm",disparity);

    

    
#ifdef DO_SIFT_MATCH
    run_sift_matcher(sm,mser,left_rect, img_scale);
#endif
	//break;
    
    float score;
    mser.obj_vec.insert(mser.obj_vec.end(),mserDepth.obj_vec.begin(),mserDepth.obj_vec.end());
    
//     for (int i=0;i<mser.obj_vec.size();++i) {
//         mser.obj_vec[i].bbox.drawBB(disparity_segmented,CV_RGB(255,255,0));
//     }

    
    vector<PotentialObject> nested;
    vector<PotentialObject> non_nested;
    compute_regions_score(score,mser.obj_vec, disparity_segmented,nested,non_nested);
    printf("Image score: %g\n",score);
    
    vector<PotentialObject> for_matlab(nested);
    
    for_matlab.insert(for_matlab.end(),non_nested.begin(),non_nested.end());
    
    
    for (int i=0;i<for_matlab.size();++i) {
        for_matlab[i].bbox.drawBB(disparity_segmented,CV_RGB(0,255,255));
    }


#ifdef DISPLAY
        cvShowImage( "Depth + Saliency Regions", disparity_segmented );
#endif

    
/*        mserDepth.obj_vec.insert(mserDepth.obj_vec.end(),nested.begin(),nested.end());
        mserDepth.obj_vec.insert(mserDepth.obj_vec.end(),non_nested.begin(),non_nested.end());*/
        
        
        
	if(numFrames%50==0)
	{	
		save_web_images(mserResult, view_disparity_objects, disparity_segmented);
	}
        
        sem_wait(sem);
        shmem.reset();
        write_data_to_shared_memory(shmem, score);
        write_objects_to_shared_memory(shmem, nested);
        write_to_shared_memory(shmem,left_ss);
        write_to_shared_memory(shmem,disparity);
        write_to_shared_memory(shmem,saliency);
        write_pc_to_shared_memory(shmem, &pc);
        write_to_shared_memory(shmem,mserResult);
        //write_objects_to_shared_memory(shmem, mser.obj_vec);
        write_objects_to_shared_memory(shmem, for_matlab);
        sem_post(sem);
//      
   
        int key = cvWaitKey( delay );
//         printf("Key pressed was: %d\n",(char)key);
        if ((char)key==' ' ) {
            if (delay) {
                printf("Space pressed... pausing\n");
                delay = 0;
            } else {
                printf("Space pressed... resuming\n");
                delay = 5;
            }
        }
//         if (key >=0) {
//             printf("Key pressed: %c\n",key);
//             if ( (char)key == 'c') {
// /*                printf("Writing frame to disk\n");
//                 // capture a frame
//                 cvSaveImage("left.pgm",left);
//                 cvSaveImage("right.pgm",right);
//                 cvSaveImage("left_rect.pgm",left_rect);
//                 cvSaveImage("right_rect.pgm",right_rect);
//                 cvSaveImage("disparity.pgm",disparity);*/
//             } else break;
//         }
    
	    if( numFrames++ % 20 == 0 )
	    {
		    fprintf( stderr, "Frame-rate = %g.\n", prof.getFPS() );
	    }
    }
    
    delete capture;
#ifdef DISPLAY
    cvDestroyWindow("Left");
    cvDestroyWindow("Right");
#endif
//     shmem.remove();
//    
//     sem_unlink(SEM_NAME);
//     
    return 0;
}
