/************************************************************************
  Copyright (c) 2004. David G. Lowe, University of British Columbia.
  This software is being made available for research purposes only
  (see file LICENSE for conditions).  This notice must be retained on
  all copies and modified versions of this software.
*************************************************************************/

/* From the standard C libaray: */
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

/*These defines are if you are compiling to use as a mex file*/
//#include "mex.h"
#define FREE free
#define ALLOC malloc

#define THRESHOLD_GRAD_IMAGE 0
#define THRESHOLD 0.02
/*Usse 0 if you want to drop the rotation invariance*/

#define UseRotationInvariance 1


/*------------------------- Macros and constants  -------------------------*/

/* Following defines TRUE and FALSE if not previously defined. */
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/* Value of PI, rounded up, so orientations are always in range [0,PI]. */
#define PI 3.1415927

#define ABS(x)    (((x) > 0) ? (x) : (-(x)))
#define MAX_(x,y)  (((x) > (y)) ? (x) : (y))
#define MIN_(x,y)  (((x) < (y)) ? (x) : (y))

/* Given the name of a structure, NEW allocates space for it in the
   given pool (see util.c) and returns a pointer to the structure.
*/
#define NEW(s,pool) ((struct s *) MallocPool(sizeof(struct s),pool))

/* Assign a unique number to each pool of storage needed for this application. 
*/
#define PERM_POOL  0     /* Permanent storage that is never released. */
#define IMAGE_POOL 1     /* Data used only for the current image. */
#define KEY_POOL   2     /* Data for set of keypoints. */

/* These constants specify the size of the index vectors that provide
   a descriptor for each keypoint.  The region around the keypoint is
   sampled at OriSize orientations with IndexSize by IndexSize bins.
   VecLength is the length of the resulting index vector.
*/
#define OriSize 8
#define IndexSize 4
#define VecLength (IndexSize * IndexSize * OriSize)


/*---------------------------- Structures -------------------------------*/

/* Data structure for a float image.
*/
typedef struct ImageSt {
    int rows, cols;          /* Dimensions of image. */
    float **pixels;          /* 2D array of image pixels. */ /*[r][c]*/
} *Image;


/* This structure describes a keypoint that has been found in an image.
*/
typedef struct KeypointSt {
    float row, col;      /* Row, column location relative to input image.  */
    float scale;         /* Scale (in sigma units for smaller DOG filter). */
    float ori;           /* Orientation in radians (-PI to PI). */
    unsigned char *ivec; /* Vector of gradient samples for indexing.*/
    struct KeypointSt *next;   /* Links all keypoints for an image. */
	
	float *vec; /*Scott */
} *Keypoint;


typedef struct DescriptorImageSt {
	int rows,cols;
	float scale;
	float offset[2];
	float ***values;
	struct DescriptorImageSt *next;
} *DescriptorImage;

/*------------------------------- Externals -----------------------------*/

extern const int MagFactor;
extern const float GaussTruncate;
/*These externs were once not, but they are needed if we want to request keypoints at specific locations*/
extern const int Scales;
extern const float InitSigma;


/*-------------------------- Function prototypes -------------------------*/
/* The are prototypes for the external functions that are shared
   between files.
*/

/* Only interface needed to key.c. */
Keypoint GetKeypoints(Image image, int maxKeys);
Keypoint MakeKeypoint(Image grad, Image ori, float octSize, float octScale,
		      float octRow, float octCol, float angle, Keypoint keys);
Keypoint AssignOriHist(Image grad, Image ori, float octSize, float octScale,
		      float octRow, float octCol, Keypoint keys);
int length(Keypoint keys);

/* From sample.c */
float ***Alloc3DMatrix(int rows, int cols, int angles, int pool);
DescriptorImage createDescriptorImage(Image im, double *scale);
DescriptorImage createDescriptorImage2(Image grad, Image ori, float scale,float offset[2]);
DescriptorImage createDescriptorPyramid(Image image, int SCALES);

/* Following are from util.c */
void *MallocPool(int size, int pool);
void FreeStoragePool(int pool);
float **AllocMatrix(int rows, int cols, int pool);
Image CreateImage(int rows, int cols, int pool);
Image CopyImage(Image image, int pool);
Image DoubleSize(Image image);
Image HalfImageSize(Image image);
Image MyDoubleSize(Image image,int pool);
Image MyHalfImageSize(Image image, int pool);
void SubtractImage(Image im1, Image im2);
void GradOriImages(Image im, Image grad, Image ori);
void GaussianBlur(Image image, float sigma);
void SolveLeastSquares(float *solution, int rows, int cols, float **jacobian,
		       float *errvec, float **sqarray);
void SolveLinearSystem(float *solution, float **sq, int size);
float DotProd(float *v1, float *v2, int len);
void thresholdImage(Image image);

