function [configPars mappingPars planningPars] = exploreLocation(cameraPars,gazePars,mappingPars,configPars,planningPars,takeCurrent,nestOnly)

% Explore this location (look for objects):
% (a) grab bumblebee image and pre-process 
% (b) find K interesting regions
% (c) for region =  1:K
%
% cameraPars - camera parameters
% gazePars - gaze parameters
% mappingPars - mapping parameters
% configPars - configuration parameters
% ptu_lim - PTU settings limits 
%

if nargin < 6 
    takeCurrent = false;
end;

if nargin < 7
    
    nestOnly = false;
end;

if ( configPars.exploreOnly || ~configPars.havePTU || ~configPars.usingRobot || ~cameraPars.canonfl)
    return;
end

unpackExploreParams;

% Find views which will cause us to cover some new ground with the camera's
% FOV
powerbot('movebotTo',powerbot('getodom')); % stops the robot wherever it is

doSlam(1,true,0);
% Update Object locations
mappingPars.objs = updateObjectLocations(mappingPars.objs);

% move the ptu to a standard position
powerbot('setpos',0,-tilt);

increment=pi/18;
binCentres = (-pi:increment:pi-increment);

positionPars = updatePositionPars();
% positionPars.oGrid = imread('map.pgm');
% positionPars.rGridPos = [60 190];
% positionPars.rPose = [2.9 6.1 0];

% Package camera state info into one variable
cameraState.trueTilt = tilt-tilt_offset;
cameraState.CAMERAHEIGHT = CAMERAHEIGHT;
cameraState.position_offset = position_offset;

[visible_regions wedgeMasks mappingPars] = buildWedgeMasks(binCentres,configPars.map_path,positionPars,cameraState,mappingPars,planningPars);
[binCentres wedgeMasks] = chooseBestAngles(binCentres,visible_regions,wedgeMasks);

%[binScores wedgeMasks] = getUnseenViews(binCentres,positionPars,mappingPars,planningPars);

% Display the views we want to look at
if DISPLAY
    figure(10);
    colormap(gray(256));
    viewsPlot = positionPars.oGrid;
    
    wedgeMask = false(size(viewsPlot));
    nAngles = length(binCentres);
    for k=1:nAngles
        wedgeMask = wedgeMask | wedgeMasks(:,:,k);
    end;
    viewsPlot(wedgeMask) = max(0,viewsPlot(wedgeMask)-25);
    image(viewsPlot);
    hold on;
    plot(positionPars.rGridPos(2),positionPars.rGridPos(1),'g+');
    mappingPars.mapId = saveMap(mappingPars.mapId,configPars.map_path);
end;

% Remember initial heading before starting to collect new views so that
% we can return to it before moving on afterwards.
initialYaw = positionPars.rPose(3);

if takeCurrent
	if nestOnly
		binCentres = [ initialYaw ];
    else
		binCentres = unique([ initialYaw binCentres ]);
	end;
end;

while ~isempty(binCentres)
    
    % Sort binCentres by proximity to current angle and turn the robot to
    % explore that bin
   
	doSlam(1,1);
	powerbot('getodom');
	powerbot('getodom');
	powerbot('getodom'); 
    positionPars = updatePositionPars();
    yaw = positionPars.rPose(3);
    
    diff = [];
    
    for k=1:length(binCentres)
        diff(k) = thetapp(binCentres(k)-yaw);
    end;
    
    [sortedDiff idx] = sort(abs(diff));
    
    minRange = binCentres(idx(1)) - pi/6;
    maxRange = minRange + pi/3;
    
    binCentres = binCentres([1:idx(1)-1 idx(1)+1:length(binCentres)]);
    
    turnbotOld(diff(idx(1))*180/pi,configPars);
    doSlam(1,true);
    
    % Update coverage map with this new view
    rPose = performSlam('get_curvecpos');
    cPose = rPose2cPose(rPose, position_offset);
    mappingPars.coverageMap = updateCoverage(mappingPars.coverageMap,cameraState.trueTilt,cPose,CAMERAHEIGHT);
        
    if DISPLAY,

        figure(99);
        clf;
        subplot(2,1,1);
        imagesc(mappingPars.coverageMap);
        subplot(2,1,2);
        imagesc(performSlam('get_map'));
        drawnow;

    end;

    fprintf('Robot will look at regions between %g and %g.\n',minRange*180/pi,maxRange*180/pi);

    exploreOneLocation;
    
    % Save acquired views to disk
    for m=1:length(objects)

        if ~isempty(views{m})

            objId = mappingPars.curObjId+m-1;
            % increment bin of angle that we're about to see
            rPose = performSlam('get_curvecpos');
            pan_rad = states{m}(1) *pi/180;
            angle = (rPose(3) + pan_rad+pi) *18/pi;
            index = 5 + floor(mod(angle,36));
            objs(objId,index) = objs(objId,index) + 1;
            % Increment nPicsTaken of this
            objs(objId,4) = objs(objId,4) + 1;
            configPars.attentionPoseNums = [ configPars.attentionPoseNums configPars.NunZoomedPics-1 ];
            nTotalPics = nTotalPics + 1;

    %             fname = sprintf('%s%d.jpg',output_path,nTotalPics);
    %             if canonfl
    %                 %system(sprintf('mv lastframe%d.jpg %s',m,fname));
    %             else
    %                 %imwrite( views{m}, fname, 'JPG','Quality',95 );
    %             end;

            wco=[size(views{m},1); size(views{m},2)]/2; % Image centre
            bbox=centreBoundingBox(objects(m),sc_tots(m),wco);
            imb=cropImage(views{m},bbox,2);
            [s,r] = system('date +%s%N');
            r=r(1:end-1);
            fname = sprintf('%s/%s.jpg',output_path2,r);
            temp_fname = sprintf('%s/%s.jpg',getenv('HOME'),r);  
            % writing to home dir first
            fprintf('[exploreLocation.m] Saving image of object %d.\n', m);
            imwrite( imb, temp_fname, 'JPG','Quality',95 );
            % now moving to destination directory
            system(['mv ' temp_fname ' ' fname]);
            
            displayCommand = sprintf( 'ln -sf %s %s/cannon.jpg', fname, configPars.displayDir );
            system( displayCommand );

            %fname = sprintf('%s/disp/%d.jpg',output_path,nTotalPics);
            %imwrite( uint8(disps{m}./256), fname, 'JPG','Quality',95 );

            %fname = sprintf('%s/bb/%d.jpg',output_path,nTotalPics);
            %imwrite( bbims{m}, fname, 'JPG','Quality',95 );

            %fname = sprintf('%s/sms/%d.jpg',output_path,nTotalPics);
            %imwrite( sms{m}, fname, 'JPG','Quality',95 );

            
            %fid = fopen(sprintf('%s/pc/%d',output_path,nTotalPics),'w');
            %fwrite( fid, pc{m}(:,3:5),'float');

	    % Write id of object into objects.txt
            %entry = sprintf('%d\n',objId);
            %fwrite(objectFile1,entry,'char');
            %fwrite(objectFile2,entry,'char');          
        end;
    end;

    mappingPars.objs = objs;
    mappingPars.curObjId = nxtObjId;
    configPars.nTotalPics = nTotalPics;

    if DISPLAY
        figure(3);clf
        for m=1:min(4,length(objects)),
            subplot(2,2,m)
            if(length(views{m}))
                image(views{m});axis image
                hold on
                h=plot(xx*msz+wco(2),yy*msz+wco(1),'r');
                set(h,'LineWidth',3.0);
                bbox=centreBoundingBox(objects(m),sc_tots(m),wco);
                plot_bbox(bbox,'r-');
            else
                h=text(0,0,'Gaze failed');
                set(h,'FontSize',15);
                set(h,'HorizontalAlignment','center');
                set(h,'VerticalAlignment','middle');
                axis([-1 1 -1 1]);axis off
            end
            title(['View ' num2str(m)]);
        end
        drawnow;
    end;
   
end;

% Face back at our original heading
positionPars = updatePositionPars();
yaw = positionPars.rPose(3);
%turnbotOld((initialYaw-yaw)*180/pi,configPars);
doSlam(1,true);

