%function Im=draw_keypoints(Im0,kp,lw,sc)
%
% Visualise a list of keypoints
%
% IM0    Image to draw in [rows cols]
% KP     Keypoint array 4xN
% LW     Line width (default 0.5)
% SC     Keypoint scaling (default 6)
%
%Per-Erik Forssen, May 2006

function Im=draw_keypoints(Im0,kp,lw,sc)

if nargin<3,lw=0.5;end
if nargin<4,sc=6;end  % Scale used in Lowe's arrows

% Define box
bx=[-1 1 1 0 1 1 -1 -1];
by=[-1 -1 0 0 0 1 1 -1];
Nl=length(bx)-1;

Np=size(kp,2);
P1=zeros(2,Np*Nl);
P2=zeros(2,Np*Nl);
for k=1:Np,
  bxc = sin(kp(4,k))*bx+cos(kp(4,k))*by;
  byc = cos(kp(4,k))*bx-sin(kp(4,k))*by;
  px = sc*kp(3,k)*bxc+kp(2,k);
  py = sc*kp(3,k)*byc+kp(1,k);
  P1(:,[1+(k-1)*Nl:k*Nl])=[px(1:end-1); py(1:end-1)];
  P2(:,[1+(k-1)*Nl:k*Nl])=[px(2:end); py(2:end)];
end

Im=draw_lines(Im0,P1,P2,lw);
