/*
**  draw_points.c
*/

#include <math.h>
#include "mex.h"
#define PI 3.141592653589793143328792211

void draw_points(double *img0,double *img1,const int *dims,double *pointlist,
		 int npts,double width) {
int rows,cols,x,y,k,cind;
int xl,xh,yl,yh; 
double xp,yp,nval,d;
 rows=dims[0];
 cols=dims[1];
 /* First copy image */
 for(x=0;x<cols;x++) {
   for(y=0;y<rows;y++) {
     cind=x*rows+y;
     img1[cind]=img0[cind];
   }
 }
 /* Now paint points */
 for(k=0;k<npts;k++) {
   xp=pointlist[k*2]-1.0;  /* Matlab compensation */
   yp=pointlist[k*2+1]-1.0;
   xl=ceil(xp-2*width);if(xl<0) xl=0;
   xh=floor(xp+2*width);if(xh>cols-1) xh=cols-1;
   yl=ceil(yp-2*width);if(yl<0) yl=0;
   yh=floor(yp+2*width);if(yh>rows-1) yh=rows-1;
   for(x=xl;x<=xh;x++) {
     for(y=yl;y<=yh;y++) {
       cind=x*rows+y;
       /* (1+cos(f))/2 */
       /*d=sqrt((xp-x)*(xp-x)+(yp-y)*(yp-y))/width;*/
       d=sqrt((xp-x)*(xp-x)+(yp-y)*(yp-y))-width+1;
       if(d<0) d=0;
       if(d<2) {
	 nval=(1+cos(PI/2*d))/2;
	 if(nval>img1[cind]) img1[cind]=nval;
       }
     }
   }
 }
 
}
/*
** Function to check wether an argument is
** a double 2d matrix of given size
** -1 as size means any size
*/
int has_size(const mxArray *arg,int mdims,int *msize)
{
const int *dims;
int k,retval;
  dims = mxGetDimensions(arg);
  retval=1;
  if(mxGetClassID(arg) != mxDOUBLE_CLASS) retval=0;
  if(mxGetNumberOfDimensions(arg) != mdims) retval=0;
  for(k=0;k<mdims;k++) {
    if((msize[k] != -1) && (dims[k] != msize[k])) retval=0;
  }
  return retval;
}
/*
**  mexFunction is the interface function to MATLAB.
**
** plhs - pointer to left-hand OUTPUT mxArrays
** nlhs - number of left-hand OUTPUT arrays
** prhs - pointer to right-hand INPUT mxArrays
** nrhs - number of right-hand INPUT arrays
**
*/
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
const int *img_dims,*arg_dims;
int arg_ndims;
int out_dims[]={0,0,0};
double *img0,*img1,*pointlist;
double width=1.0;  /* Default value */
int npts;
mxArray *mx_img1;
/* Argument count check */
 if((nrhs <2)||(nrhs>3)) {
   if(nrhs !=0) mexPrintf("Error: 2-3 input arguments expected.\n");
   goto mexErrExit;
 }
    
/* Parse arg 1 (image) */
 img_dims = mxGetDimensions(prhs[0]);
 arg_ndims = mxGetNumberOfDimensions(prhs[0]);
 if(mxGetClassID(prhs[0]) != mxDOUBLE_CLASS) {
   mexPrintf("Type mismatch: <Im0> should be of class double.\n");
   goto mexErrExit;
 }
 if(arg_ndims !=2) {
   mexPrintf("Type mismatch: <Im0> should be 2D.\n");
   goto mexErrExit;
 }
/* Parse arg 2 (point list) */
 arg_dims = mxGetDimensions(prhs[1]);
 arg_ndims = mxGetNumberOfDimensions(prhs[1]);
 if(mxGetClassID(prhs[1]) != mxDOUBLE_CLASS) {
   mexPrintf("Type mismatch: <Pl> should be of class double.\n");
   goto mexErrExit;
 }
 if(arg_ndims !=2) {
   mexPrintf("Type mismatch: <Pl> should be 2D.\n");
   goto mexErrExit;
 }
 if(arg_dims[0]!=2) {
   mexPrintf("Type mismatch: <Pl> should be 2xN.\n");
   goto mexErrExit;
 }
 pointlist=mxGetPr(prhs[1]);
 npts=arg_dims[1];

/* Parse arg 3 (width) */
 if(nrhs>2) {
   arg_dims = mxGetDimensions(prhs[2]);
   arg_ndims = mxGetNumberOfDimensions(prhs[2]);
   if(mxGetClassID(prhs[2]) != mxDOUBLE_CLASS) {
     mexPrintf("Type mismatch: <width> should be of class double.\n");
     goto mexErrExit;
   }
   if((arg_dims[0]!=arg_dims[1])||(arg_dims[1]!=1)) {
     mexPrintf("Type mismatch: <width> should be a scalar.\n");
     goto mexErrExit;
   }
   width=mxGetScalar(prhs[2]);
 }

/* Call visualisation functions */
 if(nlhs>0) {
   /* Create an empty image */
   mx_img1 = mxCreateNumericArray(2,img_dims,mxDOUBLE_CLASS,mxREAL);
   plhs[0] = mx_img1;
   /* plot points in image */
   draw_points(mxGetPr(prhs[0]),mxGetPr(mx_img1),img_dims,pointlist,npts,width);
 }
     
/* Jump to normal exit */
 goto mexExit;

/* Exit with error message */
mexErrExit:
 mexPrintf("Usage: <Im>=draw_points(<Im0>,<Pl>[,<width>])\n");
mexExit: {}
}
