% Takes a list of file names, each of which contains a list of files
% For each file, run the processing set by mode
% 1 -- Average edge
% 2 -- Color histograms
% 3 -- Generate data vectors
function val=process_images(files, directories, mode, submode, high_edge, low_edge, high_hist, low_hist)
    % Initialize according to the mode
    switch mode
        case 1
            val = zeros(100, 100);
        case 2
            val = zeros(16, 16, 16, 5000);
        case 3
            val = [];
    end

    total = 1;

    for i = 1:size(files, 2)
        fin = fopen(files{i}, 'r');
        if fin < 0
            error(['Could not open ', files{i}, ' for input']);
        end

        while 1
            [files{i} ' - ' int2str(mode) '/' int2str(submode) ' - ' int2str(total)]

            tline = fgetl(fin);
            if ~ischar(tline),   break,   end
            im = double(imread([directories{i} '/' tline]));

            switch mode
                case 1
                    % Add edge map to average
                    val = val + edge_feature(im);
                case 2
                    % Add color histogram to list
                    val(:,:,:,total) = color_hist(im);
                case 3
                    % Add data vector to list
                    %[q_edge1, q_edge2, q_color, q_hue, q_blur, q_contrast, q_brightness, q_edgetype, q_texture, q_edge_direction, q_sat, q_gran, q_wavelet] = run_features(im, high_edge, low_edge, high_hist, low_hist);
                    %val = [val ; q_edge1 q_edge2 q_color q_hue q_blur q_contrast q_brightness q_edgetype q_texture(:)' q_edge_direction(:)' q_sat q_gran' q_wavelet(:)'];
                    val = [val ; run_features(im, submode, high_edge, low_edge, high_hist, low_hist)];
            end
            
            total = total + 1;
        end

        fclose(fin);
    end

    % Finalize according to the mode
    switch mode
        case 1
            val = val ./ total;
        case 2
        case 3
    end
end