% Based on 'How realistic is photorealistic?' 2005 II
function val=wavelet_dist_old(im)
    addpath simplegabortb-v0.2.2
    
    % Making this greyscale only to fix some singularity problems with b/w
    % images, also for speed
    
    im = imresize(im, [min(200, size(im, 1)) min(200, size(im, 2))]);
        
    m = zeros(size(im, 1), size(im, 2), 16, size(im, 3));
    bank = sg_createfilterbank([size(im, 1) size(im, 2)], 0.6 , 4, 4);
    
    % Generate complete output: all scales, orientations, colors
    for dim = 1:size(im, 3)
        r = sg_filterwithbank(im(:,:,dim), bank, 'method', 1);
        m(:, :, :, dim) = sg_resp2samplematrix(r);
    end

    % Collapse diagonals

    % 1 | 5 | 9 | 13 - horizontal
    % 2, 4 | 6, 8 | 10, 12 | 14, 16 - diagonal
    % 3 | 7 | 11 | 15 - vertical

    % Include a fake 5th scale (copy of the 4th, might want to do something
    % else)
    % [H1 H2 H3 H4 (H5) V1 V2 V3 V4 (V5) D1 D2 D3 D4 (D5)]
    
    l = zeros(size(im, 1), size(im, 2), 15, 3);
    for c = 1:size(im, 3)
        index = 1;
        for o = [1 5 9 13]
            l(:,:,index,c) = abs(sum(m(:,:,o,c),3));
            index = index + 1;
        end
        l(:,:,index,c) = l(:,:,index - 1,c);
        index = index + 1;
        for o = [3 7 11 15]
            l(:,:,index,c) = abs(sum(m(:,:,o,c),3));
            index = index + 1;
        end
        l(:,:,index,c) = l(:,:,index - 1,c);
        index = index + 1;
        for o = [2 6 10 14]
            l(:,:,index,c) = abs(sum(m(:,:,o,c),3)) + abs(sum(m(:,:,o + 2,c),3));
            index = index + 1;
        end
        l(:,:,index,c) = l(:,:,index - 1,c);
    end

    if size(im, 3) < 3
        l(:, :, :, 2) = l(:, :, :, 1) * 0.5;
        l(:, :, :, 3) = l(:, :, :, 1) * 0.25;
    end
    
    val = [];
    
    for o = [1 2 3 4   6 7 8 9   11 12 13 14]
        for c = 1:size(im, 3)
            % Build Q
            Q = zeros((size(im, 1) - 2) * (size(im, 2) - 2), 9);
            index = 1;
            for x = 2:size(im, 2)-1
                for y = 2:size(im, 1)-1
                    Q(index, :) = [abs(l(y, x - 1, o, c)) ...
                                   abs(l(y, x + 1, o, c)) ...
                                   abs(l(y - 1, x, o, c)) ...
                                   abs(l(y + 1, x, o, c)) ...
                                   abs(l(round(y / 2), round(x / 2), o + 1, c)) ...
                                   abs(l(y, x, mod(o + 4, 15) + 1, c)) ...
                                   abs(l(round(y / 2), round(x / 2), mod(o + 5, 15) + 1, c)) ...
                                   abs(l(y, x, o, mod(c, 3) + 1)) ...
                                   abs(l(y, x, o, mod(c + 1, 3) + 1))];
                    index = index + 1;
                end
            end

            % Calculate p
            v = l(2:end-1,2:end-1,o,c);
            v = v(:);
            Q' * Q
            w = inv(Q' * Q) * Q' * v;
            p = log(v) - log(abs(Q * w));

            % Add p characteristics to the stats vector
            val = [val mean(p) var(p) skewness(p) kurtosis(p)];
            
            if size(im, 3) < 3
                val = [val mean(p) var(p) skewness(p) kurtosis(p)];
                val = [val mean(p) var(p) skewness(p) kurtosis(p)];
            end
        end
    end
end