#!/bin/bash

# multifetcher is an utility to search multiple image search engines for a given
# query and to cross-reference the results, preferring those images that appear
# on more than one engine, and downloading them first

# Usage: $> multifetcher.sh <query phrase> <destination dir> <number of pages> <number of images>
# the query phrase is the search phrase, separate multiple words with the '+' symbol
# e.g: basketball+ball+spalding
# the number of pages is the number of pages to query from each search engine.  Yahoo's
# pages contain 20 images each, as do google's on default settings  

# so to search for 30 logitech mouse images, I would do the following:
# $multifetcher.sh logitech+mouse logitechmouse 20 30

# the images will be downloaded to a directory specified
# images will be renamed in order 1.jpg, 2.gif, 3.png etc etc...

#remove old URL file (if any)
rm -f Image_URLs.html

USER_AGENT="Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.0.7) Gecko/20060909 Firefox/1.5.0.7"

#check for the correct number of arguments
#if it's not 4, then exit with error code and print a usage string
if [ $# -ne 4 ]
then 
    echo "Usage: `basename $0` search_term dir_name num_pages num_images"
    exit 64
fi

SEARCH_TERM=$1
DIR_NAME=$2
NUM_PAGES=$3
NUM_IMAGES=$4

GOOGLE_LIST=googleList.txt
SHORTLIST=shortlist.txt

export PATH=$PATH:.

echo "Searching Google..."
googleSearch.sh $SEARCH_TERM $NUM_PAGES $GOOGLE_LIST

#remove any flickr results
removeFlickr.sh $GOOGLE_LIST > $SHORTLIST

echo "Downloading images..."
downloadList.sh $SHORTLIST $DIR_NAME $NUM_IMAGES

rm -f $GOOGLE_LIST
rm -f $SHORTLIST

# start sift extraction
#../../../CLASSIFY/BatchExtract/BatchExtract "--exe=../../../CLASSIFY/Keypoints/keypoints" "$DIR_NAME" &

echo "Done."
